/**********************************************************************/
/*                                                                    */
/* RexxMail installation program.                                     */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* This program forms part of the RexxMail package, and may not be    */
/* distributed separately.                                            */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Use RexxComm.CMD to reformat the comments in this program.         */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* The latest versions of RexxMail and RexxComm can be found at       */
/*               www.degeus.com/rexx                                  */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* This program is released under the terms of the GNU license, see   */
/*               www.gnu.org/copyleft/gpl.html                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* (c) 2000-2001 Marcus C. de Geus                                    */
/*               marcus@degeus.com                                    */
/*               www.degeus.com                                       */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* History:                                                           */
/*                                                                    */
/* 2001-04-03 = Corrected "chaining error" bug when using CMD.EXE.    */
/* 2001-05-11 = Complete rebuild; now includes update intelligence.   */
/* 2001-06-14 = Bugfix: CMD.EXE's 'move' cannot overwrite files.      */
/*              Added RexxMail build numbers to log entries.          */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Use it if you like it. Don't if you don't. No legalese.            */
/*                                                                    */
/**********************************************************************/

/**********************************************************************/
/* Some general bits                                                  */
/**********************************************************************/

signal on halt  /* handle halt condition */
signal on syntax  /* handle bad syntax */

/**********************************************************************/
/* Define the build number                                            */
/**********************************************************************/

Global.Build = '20010617.094841'  /* define the build number */

/**********************************************************************/
/* Define the program directory                                       */
/**********************************************************************/

parse source . . ProgSpec  /* get the program name */
ProgDir = strip(filespec('D',ProgSpec)||filespec('P',ProgSpec),'T','\')  /* get the program directory */

/**********************************************************************/
/* Start a log file                                                   */
/**********************************************************************/

Global.LogFile = ProgDir||'\install.log'  /* define the log file name */

Message = date('S')||' '||time('N')||' : Installation started.'  /* end of the message to log */
call WriteLog Global.LogFile,Message  /* report */

/**********************************************************************/
/* Check for the REXX utilities library                               */
/**********************************************************************/

if (\LoadRexxUtils()) then  /* if we cannot use or load the Rexx utility functions */
do
 call Halt  /* quit */
end

/**********************************************************************/
/* Prompt for action                                                  */
/**********************************************************************/

call StartExplain  /* explain to the user what's going to happen */
if (\UserQuery('Continue?')) then  /* if the user gives a negative reply */
do
 Message = '         Installation aborted by user.'  /* the text to report */
 call WriteLog Global.LogFile,Message  /* report */
 call Halt  /* quit */
end

/**********************************************************************/
/* See what the current update status looks like                      */
/**********************************************************************/

LocFile = ProgDir||'\location.txt'  /* the user location file */

NewInstall = (stream(LocFile,'C','QUERY EXISTS') = '')  /* if we have no location file, it is a new installation */

if (NewInstall) then  /* if we have a new installation */
do
 Message = '         Creating new installation.'  /* the text to report */
 call WriteLog Global.LogFile,Message  /* report */
 call InstallExplain  /* explain what's going to happen */
 if (UserQuery('Install the RexxMail program files?')) then  /* if the user reply is positive */
 do
  if (\DoInstall(NewInstall)) then  /* if we cannot install the program */
  do
   call Halt  /* quit */
  end
 end
 else  /* if the user reply was negative */
 do
  Message = '         Program installation refused by user.'  /* the text to report */
  call WriteLog Global.LogFile,Message  /* report */
 end
end
else  /* if it is not a new installation */
do
 if (stream(ProgDir||'\update\program\location.txt','C','QUERY EXISTS') = '') then  /* if we have already installed or updated the program */
 do
  Message = 'There are no program components to install or update.'  /* the text to report */
  call WriteLog Global.LogFile,'         '||Message  /* report */
  call ClearScreen  /* clear the screen */
  say Message  /* report */
  say ''  /* empty line */
  say ''  /* empty line */
  call charout StdOut,'Press [Enter] to continue'  /* report */
  parse pull  /* wait for [Enter] */
 end
 else  /* if we still have an update to do */
 do
  Message = '         Updating existing installation to build '||Global.Build||'.'  /* the text to report */
  call WriteLog Global.LogFile,Message  /* report */
  call UpdateExplain  /* explain to the user */
  if (UserQuery('Update the RexxMail program?')) then  /* if the user reply is positive */
  do
   if (\DoInstall(NewInstall)) then  /* if we cannot update the program */
   do
    call Halt  /* quit */
   end
  end
  else  /* if the user reply is negative */
  do
   Message = '         Program update refused by user.'  /* the text to report */
   call WriteLog Global.LogFile,Message  /* report */
  end
 end
end

if (stream(LocFile,'C','QUERY EXISTS') = '') then  /* if we have no location file */
do
 Message = '  ERROR: Missing location file.'  /* the text to report */
 call WriteLog Global.LogFile,Message  /* report */
 call Halt  /* and quit */
end

UserName = value('USER',,'OS2ENVIRONMENT')  /* look for a user name */

if (UserName = '') then  /* if no user name is found */
do
 UserName = 'DEFAULT'  /* use the default value */
end

if (\GotUser(LocFile,UserName)) then  /* if we do not find this user in the location file */
do
 Message = '         User "'||UserName||'" is not in "'||LocFile||'"; installing new user.'  /* text to report */
 call WriteLog Global.LogFile,Message  /* report */

 call UserInstallExplain  /* explain to the user */
 if (UserQuery('Install mail folders for user '||UserName||'?')) then  /* if the user reply is positive */
 do
  if (\InstallUser()) then  /* if we cannot install a new user */
  do
   call Halt  /* quit */
  end
 end
 else  /* if the user reply is negative */
 do
  Message = '         New user installation refused by user.'  /* the text to report */
  call WriteLog Global.LogFile,Message  /* report */
 end
end
else  /* if we found this user in the location file */
do
 Message = '         User "'||UserName||'" is in "'||LocFile||'"; no new user installation required.'  /* text to report */
 call WriteLog Global.LogFile,Message  /* report */
end

Message = date('S')||' '||time('N')||' : Installation completed.'  /* the message to log */
call WriteLog Global.LogFile,Message  /* report */

call ClearScreen  /* clear the screen */

say 'Installation completed.'  /* report */

call Quit  /* that's all, folks! */

/**********************************************************************/
DoInstall:  /* optionally (re)installs all the bits and pieces */
/**********************************************************************/

parse arg NewInstall  /* get the argument */

if (NewInstall) then  /* if this is a new installation */
do
 GotResult = InstallProgram()  /* install the program files */
end
else  /* if this is not a new installation */
do
 call sysfiledelete ProgDir||'\update\program\location.txt'  /* get rid of the location.txt template */
 if (UserQuery('Overwrite the program files?')) then  /* if the user reply is positive */
 do
  GotResult = InstallProgram()  /* install the program files */
 end
 else  /* if the user reply is negative */
 do
  GotResult = 1  /* the result is always OK */
 end
end

if (\GotResult) then  /* if we have a negative result */
do
 return 0  /* quit with an error */
end

call sysdestroyobject ProgDir||'\update\program'  /* get rid of the program files update folder */

if (NewInstall) then  /* if this is a new installation */
do
 GotResult = InstallIcons()  /* install the icons */
end
else  /* if it is an update */
do
 if (UserQuery('Overwrite the icon files?')) then  /* if the user wants to update the icons */
 do
  GotResult = InstallIcons()  /* install the icons */
 end
 else  /* if the user does not want to update the icons */
 do
  GotResult = 1  /* the result is always OK */
 end
end

if (\GotResult) then  /* if we have a negative result */
do
 return 0  /* quit with an error */
end

call sysdestroyobject ProgDir||'\update\icons'  /* get rid of the icons update folder */

if (NewInstall) then  /* if this is a new installation */
do
 GotResult = InstallInfo()  /* install the info files */
end
else  /* if it is an update */
do
 if (UserQuery('Overwrite the HTML information files?')) then  /* if the user wants to update the info files */
 do
  GotResult = InstallInfo()  /* install the info files */
 end
 else  /* if the user does not want to update the info files */
 do
  GotResult = 1  /* the result is always OK */
 end
end

if (\GotResult) then  /* if we have a negative result */
do
 return 0  /* quit with an error */
end

call sysdestroyobject ProgDir||'\update\information'  /* get rid of the information update folder */

if (NewInstall) then  /* if this is a new installation */
do
 GotResult = InstallHelp()  /* install the help files */
end
else  /* if it is an update */
do
 if (UserQuery('Overwrite the help topic files?')) then  /* if the user wants to update the info */
 do
  GotResult = InstallHelp()  /* install the help files */
 end
 else  /* if the user does not want to update the help files */
 do
  GotResult = 1  /* the result is always OK */
 end
end

if (\GotResult) then  /* if we have a negative result */
do
 return 0  /* quit with an error */
end

call sysdestroyobject ProgDir||'\update\help'  /* get rid of the help update folder */

if (NewInstall) then  /* if this is a new installation */
do
 GotResult = InstallObjects()  /* install the program objects */
end
else  /* if it is an update */
do
 if (UserQuery('Recreate the program objects?')) then  /* if the user wants to update the objects */
 do
  GotResult = InstallObjects()  /* install the objects */
 end
 else  /* if the user does not want to update the objects */
 do
  GotResult = 1  /* the result is always OK */
 end
end

if (\GotResult) then  /* if we have a negative result */
do
 return 0  /* quit with an error */
end

return 1  /* end of DoInstall */

/**********************************************************************/
InstallProgram:  /* (re)installs the program files */
/**********************************************************************/

if (\CheckDir(ProgDir||'\update\program')) then  /* if the program dir is not OK */
do
 return 0  /* return with an error */
end

'@copy update\program\* > nul'  /* use the command processor's copy command to copy the program files */

if (RC >< 0) then  /* if we got an error */
do
 Message = '  ERROR: Cannot copy new program files to "'||ProgDir||'".'  /* the text to report */
 call WriteLog Global.LogFile,Message  /* report */
 return 0  /* return with an error */
end

Message = '         New program files copied to "'||ProgDir||'".'  /* the text to report */
call WriteLog Global.LogFile,Message  /* report */

return 1  /* end of InstallProgram */

/**********************************************************************/
InstallIcons:  /* installs the icons */
/**********************************************************************/

if (\CheckDir(ProgDir||'\update\icons')) then  /* if the icons dir does not check out OK */
do
 return 0  /* return with an error */
end

call sysfiletree ProgDir||'\update\icons\*','PaperDirs.','DO'  /* look for subdirectories of the new icons dir */
do Index = 1 to PaperDirs.0  /* for each one we find */
 if (\CheckDir(PaperDirs.Index)) then  /* if the dir does not check out OK */
 do
  return 0  /* return with an error */
 end
end

call sysdestroyobject ProgDir||'\icons'  /* get rid of any old icons dir */

if (\sysmoveobject(ProgDir||'\update\icons',ProgDir)) then  /* if we cannot move the icons folder to the prog dir */
do
 Message = '  ERROR: Cannot move new icons folder to "'||Progdir||'".'  /* the text to report */
 call WriteLog Global.LogFile,Message  /* report */
 return 0  /* return with an error */
end

Message = '         Moved new icons folder to "'||ProgDir||'".'  /* the text to report */
call WriteLog Global.LogFile,Message  /* report */

IconsDir = ProgDir||'\Icons'  /* the new icons dir */

return 1  /* end of InstallIcons */

/**********************************************************************/
InstallInfo:  /* installs the on-line info files */
/**********************************************************************/

if (\CheckDir(ProgDir||'\update\information')) then  /* if the program dir itself does not check out OK */
do
 return 0  /* return with an error */
end

call sysdestroyobject ProgDir||'\information'  /* get rid of any old info dir */

if (\sysmoveobject(ProgDir||'\update\information',ProgDir)) then  /* if we cannot move the information folder to the prog dir */
do
 Message = '  ERROR: Cannot move new information folder to "'||Progdir||'".'  /* the text to report */
 call WriteLog Global.LogFile,Message  /* report */
 return 0  /* return with an error */
end

Message = '         Moved new information folder to "'||ProgDir||'".'  /* the text to report */
call WriteLog Global.LogFile,Message  /* report */

return 1  /* end of InstallInfo */

/**********************************************************************/
InstallHelp:  /* installs the help text files */
/**********************************************************************/

if (\CheckDir(ProgDir||'\update\help')) then  /* if the program dir itself does not check out OK */
do
 return 0  /* return with an error */
end

call sysdestroyobject ProgDir||'\help'  /* get rid of any old help dir */

if (\sysmoveobject(ProgDir||'\update\help',ProgDir)) then  /* if we cannot move the help folder to the prog dir */
do
 Message = '  ERROR: Cannot move new help folder to "'||Progdir||'".'  /* the text to report */
 call WriteLog Global.LogFile,Message  /* report */
 return 0  /* return with an error */
end

Message = '         Moved new help folder to "'||ProgDir||'".'  /* the text to report */
call WriteLog Global.LogFile,Message  /* report */

return 1  /* end of InstallHelp */

/**********************************************************************/
InstallObjects:  /* installs the program objects */
/**********************************************************************/

/**********************************************************************/
/* Note order of calling MakeObject to make sure the program/file     */
/*  associations are created in the right order of preference.        */
/**********************************************************************/

Location = ProgDir||'\Objects'  /* the objects will be created in the Objects subdir of the program dir */

if (\MakeDir(Location)) then  /* if we cannot make the dir if necessary */
do
 return 0  /* return with an error */
end

Settings = 'MINIMIZED=NO;'||,  /* do not start the program minimized */
           'NOAUTOCLOSE=NO;'||,  /* close the program session after completion */
           'PROGTYPE=WINDOWABLEVIO;'||,  /* use a VIO (OS/2) window */
           'STARTUPDIR='||ProgDir  /* start the program in the RexxMail program directory */

AssocType = ''  /* no associations for the next one */

ExeName = ProgDir||'\maillock.cmd'  /* the following object uses maillock.cmd in the program directory */
Title = 'RexxMail - Lock/Unlock Mail Folders'  /* the title to use */
ObjectID = '<REXXMAIL_LOCK>'  /* the object id to use */
IconFile = IconsDir||'\MailLock.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

Settings = 'MINIMIZED=YES;'||,  /* start the program minimized */
           'NOAUTOCLOSE=NO;'||,  /* close the program session after completion */
           'PROGTYPE=WINDOWABLEVIO;'||,  /* use a VIO (OS/2) window */
           'STARTUPDIR='||ProgDir  /* start the program in the RexxMail program directory */

AssocType = 'Mail Message Out'  /* the association to use for the next few objects */

ExeName = ProgDir||'\rexxmail.cmd'  /* the following object uses rexxmail.cmd in the program directory */
Title = 'RexxMail - Edit Message'  /* the title to use */
ObjectID = '<REXXMAIL_EDIT>'  /* the object id to use */
Parameters = '/Edit'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailOpen.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

ExeName = '*'  /* use the default, i.e. the command processor */
Title = 'RexxMail - Dispatch'  /* the title to use */
ObjectID = '<REXXMAIL_DISPATCH>'  /* the object id to use */
Parameters = '/c start /c /min "Dispatching Mail" '||ProgDir||'\rexxmail.cmd /dispatch'  /* start a new session running RexxMail.CMD with the /dispatch switch */
IconFile = IconsDir||'\MailDisp.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

AssocType = 'Mail Message In'  /* the associations to use */

ExeName = ProgDir||'\rexxmail.cmd'  /* the following objects all share the same executable name, i.e. rexxmail.cmd in the program directory */

Title = 'RexxMail - View Message'  /* the title to use */
ObjectID = '<REXXMAIL_VIEW>'  /* the object id to use */
Parameters = '/View'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailOpen.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

Title = 'RexxMail - View Raw Message'  /* the title to use */
ObjectID = '<REXXMAIL_RAW>'  /* the object id to use */
Parameters = '/Raw'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailOpen.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

AssocType = 'Mail Message In,Mail Message Out'  /* the associations to use */

Title = 'RexxMail - Address'  /* the title to use */
ObjectID = '<REXXMAIL_ADDRESS>'  /* the object id to use */
Parameters = '/Address'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailAddr.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

AssocType = 'Mail Message Out'  /* the association to use for the next few objects */

Title = 'RexxMail - ASCII/Quoted-Printable'  /* the title to use */
ObjectID = '<REXXMAIL_SETASCIIQP>'  /* the object id to use */
Parameters = '/SetASCIIQP'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailOpen.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

AssocType = 'Mail Message In'  /* the associations to use */

Title = 'RexxMail - Reply'  /* the title to use */
ObjectID = '<REXXMAIL_REPLY>'  /* the object id to use */
Parameters = '/Reply'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailOpen.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

Title = 'RexxMail - Reply to All'  /* the title to use */
ObjectID = '<REXXMAIL_REPLYTOALL>'  /* the object id to use */
Parameters = '/ReplyToAll'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailOpen.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

Title = 'RexxMail - Reply to Sender'  /* the title to use */
ObjectID = '<REXXMAIL_REPLYTOSENDER>'  /* the object id to use */
Parameters = '/ReplyToSender'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailOpen.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

Title = 'RexxMail - Reply to Recipients'  /* the title to use */
ObjectID = '<REXXMAIL_REPLYTORECIPIENTS>'  /* the object id to use */
Parameters = '/ReplyToRecipients'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailOpen.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

AssocType = ''  /* no associations for the next ones */

Title = 'RexxMail - Export Addresses'  /* the title to use */
ObjectID = '<REXXMAIL_EXPORT>'  /* the object id to use */
Parameters = '/Export'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailAdEx.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

Title = 'RexxMail - Import Addresses'  /* the title to use */
ObjectID = '<REXXMAIL_IMPORT>'  /* the object id to use */
Parameters = '/Import'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailAdIm.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

Title = 'RexxMail - Info'  /* the title to use */
ObjectID = '<REXXMAIL_INFO>'  /* the object id to use */
Parameters = '/Info'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailInfo.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

Title = 'RexxMail'||d2c(10)||'Open Mail Folders'  /* the title to use */
ObjectID = '<REXXMAIL_OPEN>'  /* the object id to use */
Parameters = '/Open'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailOpen.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

call ClearScreen  /* clear the screen */

say 'For further instructions, please refer to section 6, "What to do after'  /* report */
say 'installation", of the readme.txt file in the RexxMail program directory.'  /* report */

return 1  /* end of InstallObjects */

/**********************************************************************/
InstallUser:  /* Install a new user */
/**********************************************************************/

MailLoc = GetLoc(ProgDir)  /* get the location for the Rexxmail mail folder */

if (MailLoc = '*') then  /* if we have no location */
do
 Message = '         User refused creation of main folder'  /* the message to log */
 call WriteLog Global.LogFile,Message  /* report */
 return 1  /* quit with no error */
end

if (\CheckDir(ProgDir||'\update\configuration')) then  /* if the original files dir is not OK */
do
 return 0  /* return with an error */
end

FileList = '*MAILLIST*'  /* a file to look for */

if (\CheckDir(ProgDir||'\update\configuration\lists')) then  /* if the original files dir is not OK */
do
 return 0  /* return with an error */
end

Settings = 'ALWAYSSORT=YES;'||,  /* always sort the next folders */
           'CCVIEW=NO;'||,  /* resurface existing folder view */
           'WORKAREA=NO;'||,  /* do not make the next folders work areas */
           'DEFAULTVIEW=ICON;'||,  /* use icon views for the next folders */
           'ICONVIEW=NORMAL,NONGRID;'||,  /* use standard view for these folders */
           'NORENAME=YES;'||,  /* the next folders cannot be renamed */
           'NOTVISIBLE=NO;'||,  /* the next folders are visible (in the main folder) */
           'DEFAULTSORT=-2;'  /* sort the next folders' contents by name */

AddSettings = 'ICONFILE='||IconsDir||'\Mail_Acc.ICO;'  /* additional settings for the Accessories folder */
if (\MakeFolder('Accessories',MailLoc,Settings||AddSettings)) then  /* if we cannot make the Accessories folder */
do
 return 0  /* return with an error */
end

AddSettings = 'ICONFILE='||IconsDir||'\Mail_Cfg.ICO;'  /* additional settings for the Configuration folder */
if (\MakeFolder('Configuration',MailLoc,Settings||AddSettings)) then  /* if we cannot make the Configuration folder */
do
 return 0  /* return with an error */
end

if (\MakeFolder('Lists',MailLoc||'\Configuration',Settings)) then  /* if we cannot make the Lists folder */
do
 return 0  /* return with an error */
end

Settings = 'ALWAYSSORT=YES;'||,  /* always sort the next folders */
           'CCVIEW=NO;'||,  /* resurface existing folder view */
           'WORKAREA=YES;'||,  /* make the next folders work areas */
           'DEFAULTVIEW=ICON;'||,  /* use icon views for the next folders */
           'ICONVIEW=NORMAL,FLOWED;'||,  /* use flowed (multi-column) view for the folders */
           'NORENAME=YES;'||,  /* the next folders cannot be renamed */
           'NOTVISIBLE=YES;'||,  /* the next folders are invisible (in the main folder) */
           'DEFAULTSORT=-2'  /* sort the next folders' contents by name */

AddSettings = 'ICONFILE='||IconsDir||'\MailAddr.ICO;'  /* additional settings for the Addresses folder */
if (\MakeFolder('Addresses',MailLoc,Settings||AddSettings)) then  /* if we cannot make the Addresses folder */
do
 return 0  /* return with an error */
end

if (\MakeFolder('Temp',MailLoc,Settings)) then  /* if we cannot make the Temp folder, no special icon */
do
 return 0  /* return with an error */
end

if (\MakeFolder('In',MailLoc,Settings)) then  /* if we cannot make the In folder, no special icon */
do
 return 0  /* return with an error */
end

if (\MakeFolder('In Archive',MailLoc,Settings)) then  /* if we cannot make the In Archive folder, no special icon */
do
 return 0  /* return with an error */
end

if (\MakeFolder('Out',MailLoc,Settings)) then  /* if we cannot make the Out folder, no special icon */
do
 return 0  /* return with an error */
end

if (\MakeFolder('Out Archive',MailLoc,Settings)) then  /* if we cannot make the Out Archive folder, no special icon */
do
 return 0  /* return with an error */
end

Settings = 'ALWAYSSORT=YES;'||,  /* always sort the next folders */
           'CCVIEW=NO;'||,  /* resurface existing folder view */
           'WORKAREA=YES;'||,  /* make the next folders work areas */
           'DEFAULTVIEW=ICON;'||,  /* use icon views for the next folders */
           'ICONVIEW=NORMAL,FLOWED;'||,  /* use flowed (multi-column) view for the folders */
           'NORENAME=NO;'||,  /* the next folders can be renamed */
           'NOTVISIBLE=NO;'||,  /* the next folders are visible */
           'DEFAULTSORT=-2'  /* sort the next folders' contents by name */

if (\MakeFolder('Read',MailLoc||'\In Archive',Settings)) then  /* if we cannot make the Read folder */
do
 return 0  /* return with an error */
end

if (\MakeFolder('Sent',MailLoc||'\Out Archive',Settings)) then  /* if we cannot make the Sent folder */
do
 return 0  /* return with an error */
end

if (\MakeFolder('RexxMail',MailLoc||'\In Archive',Settings)) then  /* if we cannot make the incoming RexxMail folder */
do
 return 0  /* return with an error */
end

if (\MakeFolder('RexxMail',MailLoc||'\Out Archive',Settings)) then  /* if we cannot make the outgoing RexxMail folder */
do
 return 0  /* return with an error */
end

UpdateFile = ProgDir||'\settupda.txt'  /* the name of the settings update template file */
SettingsFile = MailLoc||'\configuration\settings.txt'  /* the name of the new user settings file */

if (stream(UpdateFile,'C','OPEN READ') >< 'READY:') then  /* if we cannot open the settings update template file */
do
 Message = '  ERROR: Cannot open settings template file "'||UpdateFile||'".'  /* the message to log */
 call WriteLog Global.LogFile,Message  /* report */
 return 0  /* return with an error */
end

if (stream(SettingsFile,'C','OPEN') >< 'READY:') then  /* if we cannot open the new settings file */
do
 Message = '  ERROR: Cannot open user settings file "'||SettingsFile||'".'  /* the message to log */
 call WriteLog Global.LogFile,Message  /* report */
 return 0  /* return with an error */
end

WatchNext = 0  /* nothing doing yet */
do while (lines(UpdateFile))  /* as long as we have lines left in the update template file */
 InLine = linein(UpdateFile)  /* get the next line */
 if (WatchNext) then  /* if we have to watch this one */
 do
  if (pos(' = ',InLine) > 0) then  /* if it is a parameter line */
  do
   Inline = subword(InLine,2)  /* get rid of the first word (the comment "#") */
  end
 end
 WatchNext = (InLine = copies('#',72))  /* if the line is a separator, watch the next one */
 call lineout SettingsFile,InLine  /* write the result to the new settings file */
end

call stream UpdateFile,'C','CLOSE'  /* close the template file */
call stream SettingsFile,'C','CLOSE'  /* close the settings file */

call sysfiletree ProgDir||'\update\configuration\*','Files.','FO'  /* look for files in the original files source directory */
do Index = 1 to Files.0  /* take each of the files we found */
 if (syscopyobject(Files.Index,MailLoc||'\Configuration')) then  /* if we can copy it to the Configuration folder */
 do
  Message = '         Copied file "'||Files.Index||'" to "'||MailLoc||'\Configuration".'  /* the message to log */
  call WriteLog Global.LogFile,Message  /* report */
 end
 else  /* if we cannot copy the file to the Configuration folder */
 do
  Message = '         Cannot copy "'||Files.Index||'" to "'||MailLoc||'\Configuration", file may already exist.'  /* the message to log */
  call WriteLog Global.LogFile,Message  /* report */
 end
end

ListFile = ProgDir||'\update\configuration\lists\maillist'  /* the mailing list sample file */
if (syscopyobject(ListFile,MailLoc||'\Configuration\Lists')) then  /* if we can copy the maillist sample to the Lists folder */
do
 Message = '         Copied file "'||ListFile||'" to "'||MailLoc||'\Configuration\Lists".'  /* the message to log */
 call WriteLog Global.LogFile,Message  /* report */
end
else  /* if we cannot copy the file to the Lists folder */
do
 Message = '         Cannot copy "'||ListFile||'" to "'||MailLoc||'\Configuration\Lists", file may already exist.'  /* the message to log */
 call WriteLog Global.LogFile,Message  /* report */
end

if (\MakeShadow('<REXXMAIL_ADDRESS>',MailLoc)) then  /* if we cannot make a shadow of the Add to Addresses program object */
do
 return 0  /* return with an error */
end

if (\MakeShadow('<REXXMAIL_DISPATCH>',MailLoc)) then  /* if we cannot make a shadow of the Dispatch program object */
do
 return 0  /* return with an error */
end

if (\MakeShadow('<REXXMAIL_EXPORT>',MailLoc||'\Accessories')) then  /* if we cannot make a shadow of the Export Addresses program object */
do
 return 0  /* return with an error */
end

if (\MakeShadow('<REXXMAIL_IMPORT>',MailLoc||'\Accessories')) then  /* if we cannot make a shadow of the Import Addresses program object */
do
 return 0  /* return with an error */
end

if (\MakeShadow('<REXXMAIL_INFO>',MailLoc||'\Accessories')) then  /* if we cannot make a shadow of the Info program object */
do
 return 0  /* return with an error */
end

if (\MakeShadow('<REXXMAIL_LOCK>',MailLoc||'\Accessories')) then  /* if we cannot make a shadow of the Lock & Unlock Mail Folders program object */
do
 return 0  /* return with an error */
end

if (stream(LocFile,'C','OPEN WRITE') >< 'READY:') then  /* if we cannot open the location file */
do
 Message = '  ERROR: Cannot write main mail folder location "'||MailLoc||'" to "'||LocFile||'".'  /* the message to log */
 call WriteLog Global.LogFile,Message  /* report */
 return 0  /* return with an error */
end

call lineout LocFile,MailLoc||' = '||UserName||';'  /* write the location and user name to the location file */

call stream LocFile,'C','CLOSE'  /* close the file */

Message = '         Main mail folder location "'||MailLoc||'" written to "'||LocFile||'".'  /* the message to log */
call WriteLog Global.LogFile,Message  /* report */

StartID = '<REXXMAIL_START>'  /* the object ID of the Start folder */

Settings = 'ALWAYSSORT=YES;'||,  /* always sort the RexxMail Start folder */
           'DEFAULTVIEW=ICON;'||,  /* use icon view in the RexxMail Start folder */
           'OBJECTID='||StartID  /* use this object ID for the RexxMail Start folder */

if (MakeFolder('RexxMail'||d2c(10)||'Start','<WP_DESKTOP>',Settings)) then  /* if we can make the RexxMail Start folder */
do
 if (\MakeShadow('<REXXMAIL_OPEN>',StartID)) then  /* if we cannot make a shadow of the Open program object */
 do
  Message = '  ERROR: Cannot create RexxMail Open object shadow.'  /* the message to log */
  call WriteLog Global.LogFile,Message  /* report */
 end
 if (\MakeShadow(ProgDir,StartID)) then  /* if we cannot make a shadow of the program dir */
 do
  Message = '  ERROR: Cannot create RexxMail program folder shadow.'  /* the message to log */
  call WriteLog Global.LogFile,Message  /* report */
 end
end

'call '||ProgDir||'\rexxmail.cmd /import'  /* import the default addresses */

if (RC = 0) then  /* if all went well */
do
 Message = '         Created sample message templates in "'||MailLoc||'\Addresses".'  /* the message to log */
 call WriteLog Global.LogFile,Message  /* report */
end
else  /* if an error occurred */
do
 Message = '  ERROR: Cannot create message templates in "'||MailLoc||'\Addresses".'  /* the message to log */
 call WriteLog Global.LogFile,Message  /* report */
end

call ClearScreen  /* clear the screen */

say 'The following combination has been added to "location.txt" :'  /* report */
say ''  /* empty line */
say '  User name   : "'||UserName||'"'  /* report the user name */
say '  Main folder : "'||MailLoc||'"'  /* report the main folder location */
say ''  /* empty line */
say ''  /* empty line */
call charout StdOut,'Press [Enter] to continue'  /* report */

parse pull  /* wait for [Enter] */

return 1  /* end of InstallUser */

/**********************************************************************/
GetLoc: procedure expose Global.  /* get a location for the main folder */
/**********************************************************************/

parse arg ProgDir  /* get the argument */

do until (Location >< '')  /* go on until we can leave */

 call ClearScreen  /* clear the screen */

 say 'Enter the full drive, path and name for the main directory:'  /* show a message */
 say ''  /* empty line */
 parse pull Location  /* get a location from the user */

 if (Location >< '') then  /* if we have something */
 do

  Location = strip(Location,'B',' ')  /* remove excess spaces */
  Location = strip(Location,'B','\')  /* remove any trailing backslash */

  DriveBit = filespec('D',Location)  /* get the drive bit */
  if (DriveBit = '') then  /* if we have no drive bit */
  do
   Location = ''  /* try again */
  end
  else  /* if we do have a drive bit */
  do
   PathBit = filespec('P',Location)||filespec('N',Location)  /* get the path (and name) bit */
   if (PathBit = '') then  /* if we have no path bit */
   do
    Location = ''  /* try again */
   end
  end

  if (Location >< '') then  /* if we still have something */
  do
   CurDir = directory()  /* store the current directory name */
   if (directory(Location) >< '') then  /* if we can change to the specified location */
   do
    Location = directory()  /* get the full location spec */
    call directory CurDir  /* return to the original dir */
    if (translate(Location) = ProgDir) then  /* if the location is the same as the program dir */
    do
     Location = ''  /* make sure we can try again */
    end
    else  /* if it is some other directory */
    do
     call sysfiletree Location||'\*.*','Stuff.','BO'  /* look for files and directories in the location */
     if (Stuff.0 > 0) then  /* if we found anything */
     do
      say ''  /* empty line */
      say '"'||Location||'" is not empty;'  /* report */
      say ''  /* empty line */
      if (\UserQuery('Use the directory anyway?')) then  /* if the reply is negative */
      do
       Location = ''  /* try again */
      end
     end
    end
   end
   else  /* if we cannot change to the location */
   do
    say ''  /* empty line */
    say '"'||Location||'" does not exist;'  /* report */
    say ''  /* empty line */
    if (\UserQuery('Create a new directory?')) then  /* if the reply is negative */
    do
     Location = ''  /* try again */
    end
    else  /* if we can go ahead */
    do
     if (sysmkdir(Location) = 0) then  /* if we created the directory */
     do
      Message = '         Created directory "'||Location||'".'  /* the message to log */
      call WriteLog Global.LogFile,Message  /* report */
     end
     else  /* if the directory could not be created */
     do
      Location = ''  /* try again */
     end
    end
   end
  end
 end

 if (Location = '') then  /* if we have no location, we can try again */
 do
  if (\UserQuery('You have not given a valid location. Try again?')) then  /* if the reply is negative */
  do
   Location = '*'  /* make sure we quit */
  end
 end

end

if (Location >< '*') then  /* if we have a valid location */
do
 Settings = 'ALWAYSSORT=YES;'||,  /* keep the main directory sorted */
            'WORKAREA=YES;'||,  /* make the main directory a work area */
            'DEFAULTVIEW=ICON;'  /* the default view of the main directory is icons */

 if (\syssetobjectdata(Location,Settings)) then  /* if we cannot do it */
 do
  Message = '  ERROR: Cannot attach settings to directory "'||Location||'".'  /* the message to log */
  call WriteLog Global.LogFile,Message  /* report */
  Location = '*'  /* return with an error */
 end
end

return Location  /* end of GetLoc */

/**********************************************************************/
CheckDir: procedure expose Global.  /* checks the existence of a directory */
/**********************************************************************/

parse arg DirName  /* get the name of the directory to check */

OrgDir = directory()  /* store the current directory name */

if (directory(DirName) = '') then  /* if we cannot change to the directory */
do
 Message = '  ERROR: Cannot find directory "'||DirName||'".'  /* the message to log */
 call WriteLog Global.LogFile,Message  /* report */
 return 0  /* return with an error */
end

call directory OrgDir  /* change back to the original directory */

Message = '         Checked directory "'||DirName||'".'  /* the message to log */
call WriteLog Global.LogFile,Message  /* report */

return 1  /* end of CheckDir */

/**********************************************************************/
MakeDir: procedure expose Global.  /* check for the existence of a dir, and if not found, create it */
/**********************************************************************/

parse arg NewDir  /* get the name of the dir to make */

call sysfiletree NewDir,'Dirs.','DO'  /* look for the dir */
if (Dirs.0 = 0) then  /* if it is not found */
do
 if (sysmkdir(NewDir) >< 0) then  /* if we cannot create the dir */
 do
  Message = '  ERROR: Cannot create directory "'||NewDir||'".'  /* the message to log */
  call WriteLog Global.LogFile,Message  /* report */
  return 0  /* return with an error */
 end
end

return 1  /* end of MakeDir */

/**********************************************************************/
MakeFolder: procedure expose Global.  /* creates a folder */
/**********************************************************************/

parse arg Name,Location,Settings  /* get the arguments */

if (\syscreateobject('WPFolder',Name,Location,Settings)) then  /* if we cannot create the folder */
do
 Message = '         Cannot create folder "'||Name||'" in "'||Location||'"; folder may already exist.'  /* the message to log */
 call WriteLog Global.LogFile,Message  /* report */
 return 0  /* return with an error */
end

Message = '         Created folder "'||Name||'" in "'||Location||'".'  /* the message to log */
call WriteLog Global.LogFile,Message  /* report */

return 1  /* end of MakeFolder */

/**********************************************************************/
MakeObject: procedure expose Global.  /* creates a program object */
/**********************************************************************/

parse arg Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,AddSettings  /* get the arguments */

Settings = 'EXENAME='||ExeName||';'||,  /* include the executable path and name */
           'PARAMETERS='||Parameters||';'||,  /* include the executable parameters */
           'ASSOCTYPE='||AssocType||';'||,  /* include the association type(s) */
           'OBJECTID='||ObjectID||';'||,  /* include the object ID */
           'ICONFILE='||IconFile||';'||,  /* inlcude the icon file path and name */
           AddSettings  /* include any additional settings */

if (syscreateobject('WPProgram',Title,Location,Settings,'UPDATE')) then  /* if we can create the object */
do
 Message = '         Created or updated object with ObjectID "'||ObjectID||'" and title "'||Title||'" in "'||Location||'".'  /* the message to log */
 call WriteLog Global.LogFile,Message  /* report */
end

return 1  /* end of MakeObject */

/**********************************************************************/
MakeShadow: procedure expose Global.  /* creates a shadow */
/**********************************************************************/

parse arg Name,Location  /* get the arguments */

if (\syscreateshadow(Name,Location)) then  /* if we cannot create the shadow */
do
 Message = '         Cannot create shadow of "'||Name||'" in "'||Location||'"; shadow may already exist.'  /* the message to log */
 call WriteLog Global.LogFile,Message  /* report */
 return 0  /* return with an error */
end

Message = '         Created shadow of "'||Name||'" in "'||Location||'".'  /* the message to log */
call WriteLog Global.LogFile,Message  /* report */

return 1  /* end of MakeShadow */

/**********************************************************************/
WriteLog: procedure expose Global.  /* writes a log entry to a log file */
/**********************************************************************/

parse arg Global.LogFile,Text  /* get the arguments */

if (stream(Global.LogFile,'C','OPEN WRITE') >< 'READY:') then  /* if we cannot open the log file for writing */
do
 say 'ERROR: Cannot write to log file.'  /* show a message */
 return 0  /* return with an error */
end

Text = translate(Text,' ',d2c(10))  /* turn line feeds into spaces */

call lineout Global.LogFile,Text  /* write the text to the file */
call stream Global.LogFile,'C','CLOSE'  /* close the file */

return 1  /* end of WriteLog */

/**********************************************************************/
GotUser: procedure expose Global.  /* looks for a user entry in the location.txt file */
/**********************************************************************/

parse arg LocFile,UserName  /* get the arguments */

if (stream(LocFile,'C','OPEN READ') >< 'READY:') then  /* if we cannot open the location file */
do
 return 0  /* return with no result */
end

GotUserName = 0  /* we haven't found it yet */

do while (lines(LocFile))  /* as long as we have lines left */
 NextLine = linein(LocFile)  /* get the next one */
 if (NextLine >< '') then  /* if the line is not empty */
 do
  if (substr(NextLine,1,1) >< '#') then  /* if it is not a comment */
  do
   parse var NextLine Location ' = ' User ';'  /* get the bits we want */
   if (translate(UserName) = translate(User)) then  /* if it is what we're looking for */
   do
    GotUserName = 1  /* we have a user name */
   end
  end
 end
end

call stream LocFile,'C','CLOSE'  /* close the locations file */

return GotUserName  /* end of GotUser */

/**********************************************************************/
StartExplain: procedure expose Global.  /* explain to the user what's going to happen */
/**********************************************************************/

call ClearScreen  /* clear the screen */

say 'This is the RexxMail installation program.'  /* screen info */
say ''  /* empty line */
say 'This program will prompt you before making any changes to your setup.'  /* screen info */
say ''  /* empty line */
say 'Any changes made by this program will be written to the "install.log" file in'  /* screen info */
say 'the RexxMail program directory (i.e. the one from which you are running this'  /* screen info */
say 'installation program) for future reference.'  /* screen info */
say ''  /* empty line */
say 'You can press [Ctrl]+[C] at any time to abort the installation program.'  /* screen info */

return 1  /* end of StartExplain */

/**********************************************************************/
UpdateExplain: procedure expose Global.  /* explain to the user what's going to happen when we update the program files */
/**********************************************************************/

call ClearScreen  /* clear the screen */

say 'A previous version of RexxMail has been found.'  /* screen info */

return 1  /* end of UpdateExplain */

/**********************************************************************/
InstallExplain: procedure expose Global.  /* explain to the user what's going to happen when we install the program files */
/**********************************************************************/

call ClearScreen  /* clear the screen */

say 'The RexxMail program files will now be installed in the current directory.'  /* screen info */
say ''  /* empty line */
say 'Apart from defining the file-program associations needed to make the system'  /* screen info */
say 'recognize mail message files, the installation program will not change any'  /* screen info */
say 'system files (e.g. CONFIG.SYS), nor will it install any DLLs.'  /* screen info */

return 1  /* end of InstallExplain */

/**********************************************************************/
UserInstallExplain: procedure expose Global.  /* Explain to the user what's going to happen */
/**********************************************************************/

call ClearScreen  /* clear the screen */

say 'You will now be asked for a main user directory name. This can not be the same'  /* screen info */
say 'directory as the program directory. The directory will be optionally created if'  /* screen info */
say 'it does not exist. The main directory will contain the mail folders (In, Out,'  /* screen info */
say 'In Archive, Out Archive) and the Addresses, Accessories, and Configuration'  /* screen info */
say 'folders.'  /* empty line */
say 'Some file objects and object shadows will be created in these folders (the'  /* screen info */
say 'objects themselves are in the "Objects" subfolder of the RexxMail program'  /* screen info */
say 'directory -- the directory from which you are running this program).'  /* screen info */
say 'The installation program will also create a start folder on your desktop.'  /* screen info */

return 1  /* end of UserInstallExplain */

/**********************************************************************/
UserQuery: procedure expose Global.  /* get a reply from the user */
/**********************************************************************/

parse arg Prompt  /* get the argument */

say ''  /* start a new line */
say ''  /* start a new line */
call charout StdOut,Prompt||' [Y/N] Y'||d2c(8)  /* user prompt */
Reply = (pos(sysgetkey('ECHO'),'Yy+'||d2c(13)) > 0)  /* affirmative = 1 */

return Reply  /* end of UpdateQuery */

/**********************************************************************/
ClearScreen: procedure expose Global.  /* clears the screen ("syscls" does not preserve screen colours) */
/**********************************************************************/

ScreenLines = word(systextscreensize(),1)  /* get the number of screen lines */
ScreenColumns = word(systextscreensize(),2)  /* get the number of screen columns */

do ScreenLines  /* repeat as many times as we have screen lines */
 say ''  /* write an empty line to clear the screen */
end

say copies('_',ScreenColumns-1)  /* line */
say 'RexxMail Installation Program for build '||Global.Build||' '||copies('_',ScreenColumns-57)  /* screen info */
say ''  /* empty line */
say ''  /* empty line */

return 1  /* end of ClearScreen */

/**********************************************************************/
LoadRexxUtils: procedure expose Global.  /* loads REXX utility functions if necessary */
/**********************************************************************/

if (rxfuncquery('SysLoadFuncs') >< 0) then  /* if we have to load the REXX utility functions */
do
 if (rxfuncadd('SysLoadFuncs','RexxUtil','SysLoadFuncs') = 0) then  /* if we can register the general loading function */
 do
  call sysloadfuncs  /* call the general loading function */
 end
 else  /* if we cannot register the general loading function */
 do
  Message = '  ERROR: Cannot load Rexx utility functions.'  /* report */
  call WriteLog Global.LogFile,Message  /* report */
  return 0  /* no success */
 end
end

return 1  /* end of LoadRexxUtils */

/*****************************************************************/
Syntax:  /* we end up here if a syntax error occurred */
/*****************************************************************/

Message = '  ERROR : Bad syntax in line '||sigl||'; contact the author at <marcus@degeus.com>.'  /* the message to log */
call WriteLog Global.LogFile,Message  /* report */

/*****************************************************************/
Halt:  /* we end up here if a halt condition occurred */
/*****************************************************************/

call beep 444,111  /* signal */
call beep 333,222  /* signal */

Message = date('S')||' '||time('N')||' : Installation aborted.'  /* the message to log */
call WriteLog Global.LogFile,Message  /* report */

call ClearScreen  /* clear the screen */

say 'Installation aborted.'  /* show a message */

/*****************************************************************/
Quit:  /* we end up here at the end of it all */
/*****************************************************************/

say ''  /* empty line */
say 'For details, see the install.log file in the RexxMail program directory.'  /* report */
say ''  /* empty line */
say ''  /* empty line */
call charout StdOut,'End of RexxMail installation - press [Enter] to exit'  /* report */

parse pull  /* wait for [Enter] */

exit  /* and exit */
