/**********************************************************************/
/*                                                                    */
/* RexxMail is a REXX program that uses the Workplace Shell, the      */
/* graphical user interface of the IBM OS/2 operating system, to      */
/* create an easy to use, object-oriented, highly flexible e-mail     */
/* message processing system.                                         */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* This program forms part of the RexxMail package, and may not be    */
/* distributed separately.                                            */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Use RexxComm.CMD to reformat the comments in this program.         */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* The latest versions of RexxMail and RexxComm can be found at       */
/*               www.degeus.com/rexx                                  */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* This program is released under the terms of the GNU license, see   */
/*               www.gnu.org/copyleft/gpl.html                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* (c) 2000-2001 Marcus C. de Geus                                    */
/*               marcus@degeus.com                                    */
/*               www.degeus.com                                       */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Use it if you like it. Don't if you don't. No legalese.            */
/*                                                                    */
/**********************************************************************/

/**********************************************************************/
/* Main                                                               */
/**********************************************************************/

drop User  /* start with nothing */
drop SettingsFile  /* start with nothing */
drop Global.  /* start with nothing */

Global.Signals = 1  /* start with signals on to catch errors before the settings file is read */
Global.ErrorMail = 0  /* start with error mail off to catch errors before the settings file is read */

signal on Halt  /* handles halt condition */
signal on Syntax  /* handles syntax errors */

/**********************************************************************/
/* Get the operating system, program directory and program name       */
/**********************************************************************/

parse source OS . ProgSpec  /* get the OS and ProgSpec */

Global.ProgDir = strip(filespec('D',ProgSpec)||filespec('P',ProgSpec),'T','\')  /* the program directory */
Global.ProgName = filespec('N',ProgSpec)  /* the program name */

/**********************************************************************/
/* initialize some global stuff we need for the bare bones stuff      */
/**********************************************************************/

Global.Build = '20010617.094841'  /* define the build number */
Global.CRLF = d2c(13)||d2c(10)  /* define CRLF */
Global.EmptyLine = Global.CRLF||Global.CRLF  /* define an empty line */
Global.Error = ''  /* no error messages yet */
Global.ConfDir = Global.ProgDir  /* use the program directory until we get the correct location (we may need to write an error log file here) */

/**********************************************************************/
/* Make sure this is OS/2                                             */
/**********************************************************************/

if (OS >< 'OS/2') then  /* if the OS is not OS/2 */
do
 call AddError 'This program requires OS/2'  /* report */
 call Halt  /* and quit */
end

/**********************************************************************/
/* Try to load the REXX utilities if necessary                        */
/**********************************************************************/

if (LoadRexxUtil() = 2) then  /* if we cannot load the Rexxx utilities library */
do
 call AddError 'Cannot load the REXX utilities library'  /* report */
 call Halt  /* and quit */
end

/**********************************************************************/
/* Make sure we have the utilities level we need                      */
/**********************************************************************/

if (rxfuncquery('sysutilversion')) then  /* if we cannot find the SysUtilVersion function */
do
 RexxVersionOK = 0  /* all is not well */
end
else  /* if we can find the SysUtilVersion function */
do
 RexxVersionOK = (sysutilversion() >= '2.00')  /* if the utilities level is at least 2.00, all is well */
end

if (\RexxVersionOK) then  /* if we do not have the right REXX utilities version */
do
 call AddError 'Your REXXUTIL.DLL is not at the required level'  /* report */
 call Halt  /* and quit */
end

drop RexxVersionOK  /* clean up */

/**********************************************************************/
/* Try to load code page 850                                          */
/**********************************************************************/

OriginalCP = sysqueryprocesscodepage()  /* get the current code page */

if (syssetprocesscodepage(850) >< 0) then  /* if we cannot load code page 850 */
do
 call AddError 'Cannot load code page 850'  /* report */
 call Halt  /* and quit */
end

/**********************************************************************/
/* initialize global directory variables based on Global.ProgDir      */
/**********************************************************************/

Global.IconDir = Global.ProgDir||'\icons'  /* the RexxMail icon directory */
if (\CheckDir(Global.IconDir)) then  /* if the directory does not exist */
do
 call Halt  /* quit */
end

Global.HelpDir = Global.ProgDir||'\help'  /* the help directory, which need not exist  */
Global.InfoDir = Global.ProgDir||'\information'  /* the information directory, which need not exist */

/**********************************************************************/
/* Process any command-line arguments                                 */
/**********************************************************************/

parse arg Commands  /* get the command-line arguments */

parse var Commands '/' Commands  /* get rid of the first switch delimiter */

drop NewCommands.  /* start with nothing */
CommCount = 0  /* start at 0 */

do while (Commands >< '')  /* as long as we have commands left */
 parse var Commands Command '/' Commands  /* get the next command sequence */
 parse var Command Switch Argument  /* get the relevant parts of the command sequence */
 Switch = translate(Switch)  /* make the switch upper case */
 if (Argument >< '') then  /* if we have an argument */
 do
  Argument = strip(Argument,'B',' ')  /* get rid of excess spaces */
  Argument = strip(Argument,'B','"')  /* get rid of any quotes */
 end

 select  /* do one of the following */
 
  when (Switch = 'POP3PASSWORD') then  /* if the switch is this one */
  do
   if (Argument = '') then  /* if we have no argument */
   do
    call AddError Switch||' requires an argument'  /* report */
    call Halt  /* and quit */
   end
   POP3Pw = Argument  /* store it to override any global POP3 password value from the settings file later on, even if the argument is missing */
  end

  when (Switch = 'USER') then  /* if the switch is this one */
  do
   if (Argument = '') then  /* if we have no argument */
   do
    call AddError Switch||' requires an argument'  /* report */
    call Halt  /* and quit */
   end
   User = Argument  /* this will be the user name, even if it is missing */
  end

  when (Switch = 'SETTINGSFILE') then  /* if the switch is this one */
  do
   if (Argument = '') then  /* if we have no argument */
   do
    call AddError Switch||' requires an argument'  /* report */
    call Halt  /* and quit */
   end
   SettingsFile = Argument  /* this will be our settings file instead of the one defined before, even if it is missing */
  end

  otherwise  /* if the switch is something else */
  do
   CommCount = CommCount + 1  /* up the command counter */
   NewCommands.CommCount.!Switch = Switch  /* store the switch */
   NewCommands.CommCount.!Argument = Argument  /* store the argument */
  end

 end

end

if (CommCount = 0) then  /* if we have nothing */
do
 CommCount = 1  /* we have one now */
 NewCommands.1.!Switch = 'HELP'  /* go for help */
 NewCommands.1.!Argument = 'SWITCHES'  /* and show the switches file */
end

/**********************************************************************/
/* Find the mail files location                                       */
/**********************************************************************/

LocationFile = Global.ProgDir||'\location.txt'  /* the location file */

if (stream(LocationFile,'C','QUERY EXISTS') = '') then  /* if we cannot find a location file */
do
 call AddError 'Could not find : '||LocationFile  /* report */
 call Halt  /* and quit */
end

if (symbol('User') >< 'VAR') then  /* if we have no (command-line) user name yet */
do
 User = value('USER',,'OS2ENVIRONMENT')  /* get the user name from the OS/2 environment */
 if (User = '') then  /* if we have no user name */
 do
  User = 'DEFAULT'  /* use this */
 end
end

Global.MainDir = GetFileEntry(LocationFile,User)  /* look for the user's location in the location file */

if (Global.MainDir = '') then  /* if we have no directory spec */
do
 drop Global.MainDir  /* dump the var */
 call AddError 'No user location defined for "'||User||'"'  /* report */
 call Halt  /* quit */
end

Global.MainDir = translate(Global.MainDir)  /* make it uppercase */

if (\CheckDir(Global.MainDir)) then  /* if the directory does not exist */
do
 call Halt  /* quit */
end

/**********************************************************************/
/* initialize global directory variables based on Global.MainDir      */
/**********************************************************************/

Global.ConfDir = Global.MainDir||'\configuration'  /* the configuration directory */
if (\CheckDir(Global.ConfDir)) then  /* if the directory does not exist */
do
 Global.ConfDir = Global.ProgDir  /* use the program dir once more */
 call Halt  /* quit */
end

Global.InDir = Global.MainDir||'\in'  /* the mail in folder */
if (\CheckDir(Global.InDir)) then  /* if the directory does not exist */
do
 call Halt  /* quit */
end

Global.OutDir = Global.MainDir||'\out'  /* the mail out folder */
if (\CheckDir(Global.OutDir)) then  /* if the directory does not exist */
do
 call Halt  /* quit */
end

Global.AddrDir = Global.MainDir||'\addresses'  /* the address book folder */
if (\CheckDir(Global.AddrDir)) then  /* if the directory does not exist */
do
 call Halt  /* quit */
end

Global.TempDir = Global.MainDir||'\temp'  /* the temporary files directory */
if (\CheckDir(Global.TempDir)) then  /* if the directory does not exist */
do
 call Halt  /* quit */
end

Global.AccessDir = Global.MainDir||'\accessories'  /* the accessories folder, which need not exist */
Global.InArchDir = Global.MainDir||'\in archive'  /* the in archive folder, which need not exist */
Global.OutArchDir = Global.MainDir||'\out archive'  /* the out archive folder, which need not exist */

if (symbol('SettingsFile') >< 'VAR') then  /* if we have no settings file name yet */
do
 SettingsFile = Global.ConfDir||'\settings.txt'  /* the RexxMail settings file name */
end

/**********************************************************************/
/* get more global variables from the settings file                   */
/**********************************************************************/

SettingsEntries = 'ADDRESS',  /* defines the default e-mail address of the sender for outgoing messages */
                  'ASCIILINELENGTH',  /* ASCIILineLength determines the maximum length of ASCII message lines; defaults to 72 */
                  'ASCIITEXT',  /* ASCIIText = YES causes RexxMail to convert outgoing messages to ASCII, word-wrapped at ASCIILineLength characters */
                  'EDITOR',  /* defines the command to run on editing outgoing messages */
                  'ERRORMAIL',  /* ErrorMail = YES causes RexxMail to send error messages to the user as mail */
                  'ISODATES',  /* ISODates = YES causes RexxMail to rewrite dates in ISO notation */
                  'MAILLOGS',  /* MailLogs = YES causes RexxMail to log sent and collected messages */
                  'NAME',  /* defines the default "real" name of the sender for outgoing messages */
                  'NAMES83',  /* defines the extensions of file names that will be reduced to 8+3 characters */
                  'OBJECTTITLE',  /* defines the parts to be included in mail message object titles */
                  'OPENFOLDERS',  /* defines the RexxMail folders to open on the desktop when the /open switch is used */
                  'POP3SERVER',  /* defines the POP3 server to contact, if any */
                  'POP3USER',  /* defines the POP3 user name to use */
                  'POP3PASSWORD',  /* defines the POP3 password to use */
                  'READER',  /* defines the command to run for reading the text part of incoming messages */
                  'READERHEADER',  /* defines which message header entries of incoming messages are to be included in the reader file */
                  'RUNATTACHOUT',  /* defines a command to run on outgoing attachments before they are added to the sendable message file */
                  'RUNATTACHIN',  /* defines a command to run on incoming mail attachments right after they have been unpacked from a mesage file */
                  'RUNCOLLECT',  /* defines a command to run for collecting mail */
                  'RUNDISPATCH',  /* defines a command to run on sendable message files */
                  'SIGNALS',  /* Signals = YES causes RexxMail to sound signals after certain operations */
                  'SMTPSERVER',  /* The SMTP server to use, if any */
                  'STATIONERY',  /* defines the name of the stationery folder */
                  'TITLEADDRESS',  /* TitleAddress = Name | Address | Full */
                  'TIMESERVERS',  /* the names of time servers to contact for constructing a time zone string */
                  'TIMEZONE',  /* a static time zone setting to use instead of contacting a time server */
                  'VERSION',  /* The build number of the last RexxMail update */
                  'WARNICON'  /* WarnIcon = YES causes RexxMail to create a warning icon on the desktop if mail comes in while the main mail window is closed */

do Index = 1 to words(SettingsEntries)  /* for each word in the list */
 VarName = subword(SettingsEntries,Index,1)  /* get the variable name from the list */
 Global.VarName = ''  /* initialize the global variable of that name */
end

InvalidSettings = 0  /* we have no invalid settings yet */

if (stream(SettingsFile,'C','QUERY EXISTS') >< '') then  /* if the configuration file exists */
do
 if (OpenFile(SettingsFile,'READ')) then  /* if we can open the RexxMail settings file for reading */
 do
  SettingsCont = charin(SettingsFile,1,chars(SettingsFile))  /* get the settings file contents */
  call stream SettingsFile,'C','CLOSE'  /* close the RexxMail settings file */
  do while (SettingsCont >< '')  /* go on until we run out of content */
   parse var SettingsCont Line (Global.CRLF) SettingsCont  /* get the next line */
   Line = translate(Line,d2c(32),d2c(9))  /* convert TABs to spaces */
   Line = strip(Line,'B',' ')  /* get rid of excess spaces */
   if (Line >< '') then  /* if the line is not empty */
   do
    if (substr(Line,1,1) >< '#') then  /* if it is not a comment */
    do
     parse var Line SettingsEntry '=' SettingsValue  /* get the parts we need */
     SettingsEntry = translate(strip(SettingsEntry))  /* get rid of superfluous spaces and make it upper case so the variable tail will work */
     if (wordpos(SettingsEntry,SettingsEntries) > 0) then  /* if it is one of the range of possible entries */
     do
      Global.SettingsEntry = strip(SettingsValue)  /* get rid of superfluous spaces and use the value */
     end
     else  /* if it is not a valid entry */
     do
      InvalidSettings = 1  /* we have at least one invalid setting */
     end
    end
   end
  end
 end
end

drop SettingsEntries  /* clear up */

/**********************************************************************/
/* Set default values for any missing Boolean global variables        */
/**********************************************************************/

Confirmation = 'YES Y + 1'  /* these all count as a yes */

Global.ASCIIText = (pos(translate(Global.ASCIIText),Confirmation) > 0)  /* ASCIIText is true if the settings file entry is affirmative, i.e. it defaults to untrue  */
Global.ISODates = (pos(translate(Global.ISODates),Confirmation) > 0)  /* ISODates is true if the settings file entry is affirmative, i.e. it defaults to untrue  */
Global.MailLogs = (pos(translate(Global.MailLogs),Confirmation) > 0)  /* MailLogs is true if the settings file entry is affirmative, i.e. it defaults to untrue  */
Global.WarnIcon = (pos(translate(Global.WarnIcon),Confirmation) > 0)  /* WarnIcon is true if the settings file entry is affirmative, i.e. it defaults to untrue */
Global.Signals = (pos(translate(Global.Signals),Confirmation) > 0)  /* Signals is true if the settings file entry is affirmative, i.e. it defaults to untrue  */
Global.ErrorMail = (pos(translate(Global.ErrorMail),Confirmation) > 0)  /* ErrorMail is true if the settings file entry is affirmative, i.e. it defaults to untrue  */

drop Confirmation  /* clean up */

/**********************************************************************/
/* Override some global variables with command-line values            */
/**********************************************************************/

if (symbol('POP3Pw') = 'VAR') then  /* if we have a command-line value for the POP3 password */
do
 Global.POP3Password = POP3Pw  /* use that instead of the value from the settings file */
 drop POP3Pw  /* clean up */
end

/**********************************************************************/
/* Set default values for any other missing global variables          */
/**********************************************************************/

if (Global.Editor = '') then  /* if no editor is defined */
do
 Global.Editor = 'e.exe "%N"'  /* use e.exe */
end

if (Global.Reader = '') then  /* if no reader is defined */
do
 Global.Reader = 'e.exe "%N"'  /* use e.exe */
end

if (datatype(Global.ASCIILineLength,'W')) then  /* if ASCIILineLength is a whole number */
do
 if (Global.ASCIILineLength = 0) then  /* if it is 0 */
 do
  Global.ASCIILineLength = 72  /* make it 72 */
 end
end
else  /* if it is not a whole number */
do
 Global.ASCIILineLength = 72  /* make it 72 */
end

if (Global.ObjectTitle = '') then  /* if we have no entry for the object title strings */
do
 Global.ObjectTitle = 'DATE SENDER RECIPIENTS SUBJECT'  /* use this as the default value */
end

if (Global.OpenFolders = '') then  /* if we have no entry for the open folders parameter */
do
 Global.OpenFolders = 'ACCESSORIES ADDRESSES CONFIGURATION IN INARCHIVE OUT OUTARCHIVE'  /* use this as the default value */
end

if (Global.Stationery = '') then  /* if we have no stationery entry */
do
 Global.Stationery = 'blue_white'  /* use this */
end

Global.PaperDir = Global.IconDir||'\'||Global.Stationery  /* the stationery directory to use */
if (\CheckDir(Global.PaperDir)) then  /* if the directory does not exist */
do
 call Halt  /* quit */
end

/**********************************************************************/
/* See if we need to update the configuration file                    */
/**********************************************************************/

if (Global.Version = '') then  /* if we found no version number */
do
 Global.Version = 0  /* assume 0 */
end

if ((Global.Build >> Global.Version) | (InvalidSettings)) then  /* if we are running a later build of RexxMail, or if we have any invalid settings */
do
 if (\UpdateSettings(SettingsFile,Global.ProgDir||'\settupda.txt')) then  /* if we cannot update the configuration file */
 do
  call Adderror 'Cannot update the configuration file'  /* report */
  call Halt  /* and quit */
 end
end

/**********************************************************************/
/* initialize some more global strings                                */
/**********************************************************************/

Global.Warning = ''  /* the warning string to use */

Global.ESCtoASC = '_________'||d2c(9)||d2c(10)||'__'||d2c(13)||'__'||,  /* bytes 00 to 0F of the escape characters => ASCII conversion table */
                  '________________'  /* bytes 10 to 1F */

Global.850toASC = 'CueaaaaceeeiiiAA'||,  /* bytes 80 to 8F of the PC850 => ASCII conversion table */
                  'EaAooouuyOUoLOxf'||,  /* bytes 90 to 9F */
                  'aiounNao?r___!__'||,  /* bytes A0 to AF */
                  '_____AAAc____cY_'||,  /* bytes B0 to BF */
                  '______aA_______*'||,  /* bytes C0 to CF */
                  'dDEEEeIII____|I_'||,  /* bytes D0 to DF */
                  "O_OOoO___UUUyY_'"||,  /* bytes E0 to EF */
                  '-_____/__"-132*_'  /* bytes F0 to FF */

Global.850toISO = ''||,  /* bytes 80 to 8F of the PC850 => ISO-8859-1 conversion table */
                  'f'||,  /* bytes 90 to 9F */
                  'Ѫ'||,  /* bytes A0 to AF */
                  '__________'||,  /* bytes B0 to BF */
                  '_____________'||,  /* bytes C0 to CF */
                  'Ȁ_____'||,  /* bytes D0 to DF */
                  'յݯ'||,  /* bytes E0 to EF */
                  '_*'  /* bytes F0 to FF */

Global.ISOto850 = '_______________'||,  /* bytes 80 to 8F of the ISO-8859-1 => PC850 conversion table */
                  '________________'||,  /* bytes 90 to 9F */
                  'Ͼ'||,  /* bytes A0 to AF */
                  ''||,  /* bytes B0 to BF */
                  'ǎԐ'||,  /* bytes C0 to CF */
                  'ѥ噞'||,  /* bytes D0 to DF */
                  'Ƅ'||,  /* bytes E0 to EF */
                  'Ф'  /* bytes F0 to FF */

/**********************************************************************/
/* Run the command-line commands we collected earlier                 */
/**********************************************************************/

do Index = 1 to CommCount  /* run through the commands we collected */
 call ProcessCommand NewCommands.Index.!Switch,NewCommands.Index.!Argument  /* process the switch and the argument, if any */
end

call Halt  /* that's all, folks! */

/**********************************************************************/
Syntax:  /* handles syntax errors */
/**********************************************************************/

call AddError 'REXX did not recognize the instruction on line '||Sigl||':'||Global.CRLF||'    '||sourceline(Sigl)  /* report */
call Halt  /* and quit */

/**********************************************************************/
Halt:  /* handles halt condition */
/**********************************************************************/

if (symbol(OriginalCP) = 'VAR') then  /* if the original code page is defined */
do
 call syssetprocesscodepage OriginalCP  /* restore the original code page */
end

ScreenMess = 'RexxMail - build '||Global.Build||' - (c) 2000-2001 Marcus C. de Geus'||Global.CRLF  /* text to report */

if (symbol('User') = 'VAR') then  /* if we have a user name */
do
 ScreenMess = ScreenMess||'User          = "'||User||'"'||Global.CRLF  /* text to report */
end

if (symbol('Global.MainDir') = 'VAR') then  /* if we have a main folder name */
do
 ScreenMess = ScreenMess||'Main folder   = "'||Global.MainDir||'"'||Global.CRLF  /* text to report */
end

if (symbol('SettingsFile') = 'VAR') then  /* if we have a settings file name */
do
 ScreenMess = ScreenMess||'Settings file = "'||SettingsFile||'"'||Global.CRLF  /* text to report */
end

Errors = (Global.Error >< '')  /* do we have errors? */

if (Errors) then  /* if we do */
do

 if (Global.Signals) then  /* if a signal is wanted */
 do
  call beep 888,100  /* signal */
  call beep 111,200  /* signal */
 end

 say Global.Error  /* send the error text to standard output */
 say ''  /* add an empty line */
 say 'Type RexxMail /help for a list of available options'  /* report */

 ErrorText = 'RexxMail detected the following error:'  /* explanatory text */

 DTStamp = substr(date('S'),1,4)||'-'||,  /* start the date & time stamp with the year in 4 digits */
           substr(date('S'),5,2)||'-'||,  /* followed by the month in 2 digits */
           substr(date('S'),7,2)||' - '||,  /* followed by the day of the month in 2 digits */
           time('N')||' :'  /* followed by the normal time */
 ErrorLog = Global.ConfDir||'\errorlog.txt'  /* the name of the error log file */

 if (OpenFile(ErrorLog,'WRITE')) then  /* if we can open the file for writing */
 do
  call lineout ErrorLog,DTStamp||Global.Error  /* write the error text to the log file */
  call stream ErrorLog,'C','CLOSE'  /* close the log file */
 end
 else  /* if we cannot open the error log file */
 do
  call SystemMessage 'Error log write error.',ErrorText||Global.CRLF||'Cannot write to error log file:'||Global.CRLF||'  '||ErrorLog  /* report */
 end

 if (Global.ErrorMail) then  /* if we want an error message to be sent to the user */
 do
  if (Global.InDir >< '') then  /* if we have an incoming messages dir */
  do
   call SystemMessage 'RexxMail error.',ErrorText||Global.CRLF||Global.Error||Global.EmptyLine||ScreenMess  /* send a message to the user */
  end
 end

end

say ''  /* send an empty line to standard output */
say ScreenMess  /* send the screen message(s) to standard output */

exit Errors  /* quit */

/**********************************************************************/
AddError: procedure expose Global.  /* add an error string*/
/**********************************************************************/

parse arg Message  /* get the argument */

Global.Error = Global.Error||Global.CRLF||Message  /* add the message to the existing error string */

return  /* end of AddError */

/**********************************************************************/
AddressExport: procedure expose Global.  /* exports address book contents and edits address text file */
/**********************************************************************/

parse arg ExpFile  /* get the argument, if any */

if (ExpFile = '') then  /* if we have no argument */
do
 ExpFile = Global.ConfDir||'\mailaddr.txt'  /* the address file to edit */
end

say 'Exporting address data to : '||translate(ExpFile)  /* report */

BakFile = ExpFile||'.bak'  /* the backup file */

if (stream(ExpFile,'C','QUERY EXISTS') >< '') then  /* if the old address file already exists */
do
 if (stream(BakFile,'C','QUERY EXISTS') >< '') then  /* if the backup file already exists */
 do
  if (sysfiledelete(BakFile) > 0) then  /* if we cannot delete the old backup file */
  do
    call AddError 'Cannot delete "'||BakFile||'"'  /* report */
    return 0  /* and quit */
  end
 end
 if (\OpenFile(ExpFile,'READ')) then  /* if we can't open the export file for reading */
 do
  return 0  /* return without success */
 end
 AddrData = charin(ExpFile,1,chars(ExpFile))  /* get the export file contents */
 call stream ExpFile,'C','CLOSE'  /* and close the file */
 if (\OpenFile(BakFile)) then  /* if we can't open a new backup file for writing */
 do
  return 0  /* return without success */
 end
 call charout BakFile,AddrData  /* write the contents to the backup file */
 call stream BakFile,'C','CLOSE'  /* and close the file */
 if (sysfiledelete(ExpFile) > 0) then  /* if we cannot delete the existing export file */
 do
   call AddError 'Cannot delete "'||ExpFile||'"'  /* report */
   return 0  /* and quit */
 end
end

if (\GetAddressData(ExpFile)) then  /* if we cannot get the data from the template objects in the address folder */
do
 call AddError 'Cannot get address folder data'  /* report */
 return 0  /* and quit */
end

return 1  /* that's all, folks! */

/**********************************************************************/
AddressImport: procedure expose Global.  /* imports address book entries */
/**********************************************************************/

parse arg ImpFile  /* get the argument, if any */

if (ImpFile = '') then  /* if we have no argument */
do
 ImpFile = Global.ConfDir||'\mailaddr.txt'  /* use the standard file name */
end

say 'Importing address data from : '||translate(ImpFile)  /* report */

if (stream(ImpFile,'C','QUERY EXISTS') = '') then  /* if the address text file does not exist */
do
 call AddError 'Cannot find "'||ImpFile||'"'  /* report */
 return 0  /* and quit */
end

if (\PutAddressData(ImpFile,Global.AddrDir)) then  /* if we cannot put the template data from the emergency file into the address folder */
do
 call AddError 'An error occurred while writing the address folder data'  /* report */
end

return 1  /* that's all, folks! */

/**********************************************************************/
AttCreate: procedure expose Global.  /* create an attachment dir and EA */
/**********************************************************************/

parse arg MessFile  /* get the argument */

AttDir = systempfilename(Global.TempDir||'\RXML????')  /* get a unique directory name in the temp files directory */
ObjTitle = filespec('N',AttDir)  /* get the name bit */
Settings = 'WORKAREA=YES;ALWAYSSORT=YES;BACKGROUND=(none),,,C,204 204 0'  /* make it a work area, sort the folder contents and set the background colour */

if (syscreateobject('WPFolder',ObjTitle,Global.TempDir,Settings)) then  /* if we created the directory */
do
 Len = reverse(right(d2c(Length(AttDir)),2,'00'x))  /* set Len */
 Subject = reverse('FFFD'x)||Len||AttDir  /* prepare Subject */
 if (sysputea(MessFile,'.SUBJECT',Subject) >< 0) then  /* if we cannot attach it to the file */
 do
  call AddError 'Cannot link attachments folder "'||AttDir||'" to "'||MessFile||'"'  /* report */
  if (sysrmdir(AttDir) >< 0) then  /* if we cannot get rid of the dir we created */
  do
   call AddError 'Cannot remove attachments folder "'||AttDir||'"'  /* report */
  end
  AttDir = ''  /* we have no attachment dir to report back */
 end
end
else  /* if we could not create the directory */
do
 call AddError 'Cannot create attachments folder "'||AttDir||'"'  /* report */
 AttDir = ''  /* nothing to return */
end

return AttDir  /* end of AttCreate */

/**********************************************************************/
AttDelete: procedure expose Global.  /* delete the attachments directory and EA */
/**********************************************************************/

parse arg MessFile,AttDir  /* get the arguments */

Subject = reverse('FFFD'x)||'0000'x  /* prepare an empty Subject */
Success = (sysputea(MessFile,'.SUBJECT',Subject) = 0)  /* if we can set it (i.e. delete the Subject EA), we're O.K. */

return Success  /* end of AttDelete */

/**********************************************************************/
AttGetDir: procedure expose Global.  /* get the attachment directory name */
/**********************************************************************/

parse arg MessFile  /* get the argument */

AttDir = ''  /* start with zilch */

call sysgetea MessFile,'.SUBJECT','Subject'  /* look for Subject */
if (Subject >< '') then  /* if we found a subject */
do
 parse var Subject EAT 3 Len 5 Text  /* get the bits we want */
 if (EAT = reverse('FFFD'x)) then  /* if it is ASCII text */
 do
  AttDir = Text  /* the text must be the attachment directory */
 end
end

return AttDir  /* end of AttGetDir */

/**********************************************************************/
AttGetList: procedure expose Global.  /* gets a list of mail attachments from a 'sent' message file */
/**********************************************************************/

parse arg MessFile  /* get the argument */

AttList = ''  /* start with zilch */

call sysgetea MessFile,'.COMMENTS','Comments'  /* look for Comments */
if (Comments >< '') then  /* if we found comments */
do
 parse var Comments EAT 3 . 5 Count 7 Contents  /* get the bits we want */
 if (EAT = reverse('FFDF'x)) then  /* if it is a multi-value, multi-type entry */
 do
  Count = c2d(reverse(Count))  /* restore the count value to the right order */
  do Index = 1 to Count  /* run through them all */
   parse var Contents EAT 3 Len 5 Contents  /* get the bits we want  */
   if (EAT = reverse('FFFD'x)) then  /* if it is ASCII text */
   do
    Next = c2d(reverse(Len)) + 1  /* where the next entry starts */
    parse var Contents Text =(Next) Contents  /* get the bits we need */
    AttList = AttList||Text||Global.CRLF  /* add the text to the list */
   end
  end
 end
end

return AttList  /* end of AttGetList */

/**********************************************************************/
AttList: procedure expose Global.  /* sets a list of mail attachments */
/**********************************************************************/

parse arg MessFile,AttDir,DateString  /* get the arguments */

Subject = reverse('FFFD'x)||'0000'x  /* prepare an empty Subject */
if (sysputea(MessFile,'.SUBJECT',Subject) >< 0) then  /* if we cannot set it (i.e. delete the Subject EA) */
do
 return 0  /* return with no success */
end

Contents. = ''  /* clear the contents array */
Count = 0  /* we have no lines so far */

Count = Count + 1  /* up the counter */
Contents.Count = 'Sent:'  /* the first line of text */

Count = Count + 1  /* up the counter */
Contents.Count = ' '||DateString  /* add the date string */

if (AttDir >< '') then  /* if we have an attachments dir */
do
 call sysfiletree AttDir||'\*.*','Attach','FO'  /* look for attachment files in the attachments directory */
 if (Attach.0 > 0) then  /* if we have attachments */
 do
  if (Attach.0 > 1) then  /* if we have more than one */
  do
   Plural = 's'  /* we need a plural "s" */
  end
  else  /* if there is just one attachment */
  do
   Plural = ''  /* we do not need a plural "s" */
  end
  Count = Count + 1  /* up the counter */
  Contents.Count = 'Attachment'||Plural||':'  /* a new line of text to add */
  do Index = 1 to Attach.0  /* take each of the remaining attachment files, if any */
   Count = Count + 1  /* up the counter */
   Contents.Count = ' '||filespec('N',Attach.Index)  /* add the attachment file name with a leading space */
  end
 end
end

NewContents = ''  /* start with nothing */
do Index = 1 to Count  /* run through the text we have */
 Len = reverse(right(d2c(Length(Contents.Index)),2,'00'x))  /* set Len */
 NewContents = NewContents||reverse('FFFD'x)||Len||Contents.Index  /* add to the contents */
end

Count = reverse(right(d2c((Count)),2,'00'x))  /* set Count */
NewContents = reverse('FFDF'x)||'0000'x||Count||NewContents  /* prepare the rest of the contents */
Success = (sysputea(MessFile,'.COMMENTS',NewContents) = 0)  /* if we can attach them to the file, we're O.K. */

return Success  /* end of AttList */

/**********************************************************************/
CheckAddress: procedure expose Global.  /* check an address's components */
/**********************************************************************/

parse arg FirstBit,LastBit  /* get the arguments */

if ((words(FirstBit) >< 1) | (words(LastBit) >< 1)) then  /* if the first bit does not contain exactly one word, and the last bit likewise */
do
 return ''  /* return nothing */
end

GotStartBracket = (substr(FirstBit,1,1) = '<')  /* have we got a leading angle bracket on the first bit? */
GotEndBracket = (substr(LastBit,length(LastBit)) = '>')  /* have we got a trailing angle bracket on the last bit? */

if (GotStartBracket >< GotEndBracket) then  /* if the start of the first bit and the end of the last bit do not match */
do
 return ''  /* return nothing */
end

if (GotStartBracket) then  /* if the first bit has a leading angle bracket */
do
 FirstBit = strip(FirstBit,'L','<')  /* get rid of the leading angle bracket on the first bit */
 if (words(FirstBit) = 0) then  /* if the first bit no longer contains a word */
 do
  return ''  /* return nothing */
 end
end

if (GotEndBracket) then  /* if we have a trailing angle bracket on the last bit */
do
 LastBit = strip(LastBit,'T','>')  /* get rid of the trailing angle bracket */
 if (words(LastBit) = 0) then  /* if the last bit no longer contains a word */
 do
  return ''  /* return nothing */
 end
end

BadRange = '<>()[]@,;:\" '||,  /* we can't have these in a bare (unbracketed) e-mail address */
           xrange('00'x,'1F'x)||,  /* nor these */
           xrange('7F'x,'FF'x)  /* nor these */

if (verify(FirstBit,BadRange,'M',1) > 0) then  /* if the first bit contains illegal characters */
do
 return ''  /* return nothing */
end

if (verify(LastBit,BadRange,'M',1) > 0) then  /* if the last bit contains illegal characters */
do
 return ''  /* return nothing */
end

Address = FirstBit||'@'||LastBit  /* the complete address */

return Address  /* end of CheckAddress */

/**********************************************************************/
CheckDir: procedure expose Global.  /* checks the existence of a directory */
/**********************************************************************/

parse arg DirSpec  /* get the argument */

Success = 0  /* no success to begin with */

CurDir = directory()  /* store the current dir */

if (directory(DirSpec) = '') then  /* if we cannot change to the directory */
do
 call AddError 'Cannot find directory "'||DirSpec||'"'  /* report */
end
else  /* if we can change to the directory */
do
 if (directory(CurDir) = '') then  /* if we cannot change back to the original directory */
 do
  call AddError 'Cannot return to original directory: "'||CurDir||'"'  /* report */
 end
 else  /* if we can change back to the original directory */
 do
  Success = 1  /* all is well */
 end
end

return Success  /* end of CheckDir */

/**********************************************************************/
CollectMail: procedure expose Global.  /* Gets waiting mail messages from POP3 server */
/**********************************************************************/

/**********************************************************************/
/* Have we got an external command to run?                            */
/**********************************************************************/

if ((symbol('Global.RunCollect') = 'VAR') & (Global.RunCollect >< '')) then  /* if we have a RunCollect command line */
do
 say 'RunCollect configuration option parsed as : '||Global.RunCollect  /* report */
 Global.RunCollect  /* run the command */
 return (RC = 0)  /* if the collect command completed without error, all is well */
end

/**********************************************************************/
/* See if we have all the arguments we really need                    */
/**********************************************************************/

if (Global.POP3Server = '') then  /* if we have no server name */
do
 return 0  /* simply return, we are apparently using some other means of collecting mail */
end

if (Global.POP3User = '') then  /* if we have no user name */
do
 call AddError 'Missing configuration entry : POP3User'  /* report */
 return 0  /* return */
end

if (Global.POP3Password = '') then  /* if we have no password */
do
 call AddError 'Missing configuration entry : POP3Password'  /* report */
 return 0  /* return */
end

/**********************************************************************/
/* See if we can load the REXX sockets library                        */
/**********************************************************************/

if (LoadRexxSock() = 2) then  /* if we cannot load the REXX socket functions */
do
 call AddError 'Cannot load REXX socket library'  /* report and exit */
 return 0  /* and quit with no success */
end

/**********************************************************************/
/* Try to connect to the server                                       */
/**********************************************************************/

call charout StdOut,'Connecting to '||Global.POP3Server||' : '  /* report */

Socket = ServerConnect(Global.POP3Server,110)  /* get the socket number for a server connection through port 110 */

if (Socket = '') then  /* if we have no socket */
do
 say 'failed.'  /* report */
 return 0  /* quit without success */
end

Global.Socket.SocketBuffer = ''  /* start with an empty socket buffer */

if (\SocketAnswer(Socket,'+OK')) then  /* if we get the wrong return code */
do
 say 'failed.'  /* report */
 call AddError 'No response from '||Global.POP3Server  /* report */
 return 0  /* return with no socket */
end

say 'O.K.'  /* report */

/**********************************************************************/
/* Try to log on to the server                                        */
/**********************************************************************/

call charout StdOut,'Logging on as user '||Global.POP3User||' : '  /* report */

if (\SocketSendLine(Socket,'USER '||Global.POP3User,'+OK')) then  /* if we cannot send this and get the right reply */
do
 say 'failed.'  /* report */
 call AddError 'User not accepted'  /* report */
 call ServerDisconnect Socket,Global.POP3Server  /* disconnect from the POP3 server */
 return 0  /* retrun with no socket */
end

if (\SocketSendLine(Socket,'PASS '||Global.POP3Password,'+OK')) then  /* if we cannot send this and get the right reply */
do
 say 'failed.'  /* report */
 call AddError 'Password not accepted'  /* report */
 call ServerDisconnect Socket,Global.POP3Server  /* disconnect from the POP3 server */
 return 0  /* retrun with no socket */
end

say 'O.K.'  /* report */

/**********************************************************************/
/* Look for waiting mail                                              */
/**********************************************************************/

Reply = SocketSendLine(Socket,'STAT','?')  /* send this and get the reply */

Messages = word(Reply,2)  /* get the number of messages */

if ((Messages = 0) | (Messages = '')) then  /* if there are no messages */
do
 say 'There are no messages'  /* report */
end
else  /* if there are messages to collect */
do
 if (Messages > 1) then  /* if we have more than 1 message */
 do
  Plural = 's'  /* we need a plural "s" */
 end
 else  /* if we have exactly one message */
 do
  Plural = ''  /* we do not need a plural "s" */
 end
 Success = 1  /* assume success */
 say 'Collecting '||Messages||' message'||Plural||'...'  /* report */
 do Index = 1 to Messages  /* take each message */
  if (POP3RetrieveMess(Socket,Index)) then  /* if the message can be retrieved */
  do
   if (\POP3DeleteMess(Socket,Index)) then  /* if we cannot delete the message */
   do
    Success = 0  /* all is not well */
   end
  end
  else  /* if the message cannot be retrieved */
  do
   Success = 0  /* all is not well */
  end
 end
 if (Success) then  /* if all is well */
 do
  say 'O.K.'  /* report */
 end
end

call ServerDisconnect Socket,Global.POP3Server  /* disconnect */

return Messages  /* end of CollectMail */

/**********************************************************************/
DecodeB64: procedure expose Global.  /* decodes a B64 string */
/**********************************************************************/

parse arg B64Str  /* get the argument */

B64Chars = xrange('A','Z')||xrange('a','z')||xrange('0','9')||'+/'  /* define the base64 character set */

BitStr = ''  /* start with nothing */

do while (B64Str >< '')  /* as long as we have something left */
 parse var B64Str NextChar 2 B64Str  /* get the next character */
 if (NextChar >< '=') then  /* if it it is not padding */
 do
  CharVal = pos(NextChar,B64Chars) - 1  /* the value represented by the B64 character */
  CharBits = right(x2b(d2x(CharVal)),6,'0')  /* convert the value to 6 bits */
  BitStr = BitStr||CharBits  /* add the result to the bit string */
 end
end

TextStr = ''  /* start with nothing */
do while(length(BitStr) > 7)  /* as long as we have at least a whole byte left */
 parse var BitStr NextChar 9 BitStr  /* get the first 8 bits */
 TextStr = TextStr||x2c(b2x(NextChar))  /* convert to a character and add to the result */
end

return TextStr  /* end of DecodeB64 */

/**********************************************************************/
DecodeHeaderLine: procedure expose Global.  /* decodes header strings */
/**********************************************************************/

parse arg String  /* get the parameter */

CodeStart = '=?'  /* define the start of an encoded string */
CodeEnd = '?='  /* define the end of an encoded string */
GotCode = 0  /* we have not seen any encoded stuff yet */
NewString = ''  /* start with nothing */

do while (String >< '')  /* as long as we have a bit of string left */

 parse var String NextBit String  /* get the next bit */

 parse var NextBit FrontBit (CodeStart) CharSet '?' Encoding '?' CodedBit (CodeEnd) BackBit  /* extract the bits we want */

 if (CharSet >< '') then  /* if we found a character set code, we have encoded text */
 do

  select  /* do one of the following */

   when (translate(CharSet) = 'US-ASCII') then  /* if the character set is US-ASCII */
   do
    if (translate(Encoding) = 'Q') then  /* if it uses quoted-printable encoding */
    do
     NewStr = translate(CodedBit,' ','_')  /* decode underscores into spaces */
    end
    else  /* if it uses b64 encoding */
    do
     NewStr = DecodeB64(CodedBit)  /* decode it */
    end
   end

   when (translate(CharSet) = 'ISO-8859-1') then  /* if the character set is ISO-8859-1 */
   do
    if (translate(Encoding) = 'Q') then  /* if it uses quoted-printable encoding */
    do
     NewStr = ''  /* start with nothing */
     do while (CodedBit >< '')  /* as long as we have coded content */
      parse var CodedBit NextChar 2 CodedBit  /* get the first character of what we have left */
      if (NextChar = '=') then  /* if it is a "=" */
      do
       parse var CodedBit NextChar 3 CodedBit  /* get the next two characters */
       NewBit = x2c(NextChar)  /* convert them from hex to character */
      end
      else  /* if it is not a "=" */
      do
       NewBit = translate(NextChar,' ','_')  /* turn an underscore into a space */
      end
      NewStr = NewStr||NewBit  /* add the result to what we have */
     end
    end
    else  /* if it uses b64 encoding */
    do
     NewStr = DecodeB64(CodedBit)  /* decode it */
    end
    NewStr = translate(NewStr,Global.ISOto850,xrange('80'x,'FF'x))  /* convert high bits from ISO 8859-1 to PC850 */
   end

   otherwise  /* if none of the above apply */
   do
    NewStr = '(...)'  /* we have nothing to show until this bit gets added */
   end

  end

  NewStr = FrontBit||NewStr||BackBit  /* stick any front and back bits back on */

  if (\GotCode) then  /* if the previous string contained no code */
  do
   NewStr = ' '||NewStr  /* start with a space */
   GotCode = 1  /* set the code flag */
  end

 end

 else  /* if it is not an encoded string */

 do
  NewStr = ' '||NextBit  /* use it as it is, preceded by a space */
  GotCode = 0  /* reset the code flag */
 end

 NewString = NewString||NewStr  /* add the new bit to the new string */

end

NewString = strip(NewString,'B',' ')  /* remove excess spaces */

return NewString  /* end of DecodeHeaderLine */

/**********************************************************************/
DeleteDir: procedure  expose Global.  /*  deletes a directory and its contents */
/**********************************************************************/

parse arg Directory  /* get the argument */

call sysfiletree Directory||'\*','Files.','FOS'  /* get any files in the directory and its subdirectories */

do Index = 1 to Files.0  /* take each of the files found */
 call sysfiledelete Files.Index  /* and delete it */
end

call sysfiletree Directory||'\*','Dirs.','DOS'  /* get any directories in the directory and its subdirectories */

do Index = Dirs.0 to 1 by -1  /* take each of the directories found, starting at the bottom */
 call sysrmdir Dirs.Index  /* and delete it */
end

call sysrmdir Directory  /* zap the topmost directory */

return  /* end of Deletedir */

/**********************************************************************/
EditMessage: procedure expose Global.  /* edits mail file and sets mail message comment */
/**********************************************************************/

parse arg MessFile  /* get the argument */

MessageDir = strip(filespec('D',MessFile)||filespec('P',MessFile),'T','\')  /* the directory the message file is in */
IsAddress = (pos(translate(Global.AddrDir),translate(MessageDir)) > 0)  /* if it is an address folder file, set a flag */

if (IsAddress) then  /* if it's an address folder file */
do
 EditFile = MessFile  /* we will just edit the file later on */
end
else  /* if it's not an address folder file or the new message template */
do
 call MessageSettings MessFile,'*-******','Change'  /* make the file Not O.K. for processing (to prevent accidental sending) */
 if (Global.ASCIIText) then  /* if all messages are to be sent as ASCII text */
 do
  call MessageSettings MessFile,'***+****','Change'  /* set the ASCII text flag */
 end
 AttDir = SetAttachments(MessFile)  /* get the attachments folder, creating it if necessary */
 call MessageContents MessFile  /* get the message contents */
 call sysopenobject AttDir,1,1  /* open the attachments folder */
 if (GetHeaderEntry(Global.MessHead,'FROM:') = '') then  /* if there is no "From: " line yet */
 do
  Global.MessHead = 'From: '||MakeFromLine()||Global.CRLF||Global.MessHead  /* Add a "From: " line containing the default sender */
  CloseFile = Global.ConfDir||'\closing.txt'  /* the name of the closing text file */
  if (OpenFile(CloseFile,'READ')) then  /* if we can open the file, i.e. it exists and is ready */
  do
   Closing = charin(CloseFile,1,chars(CloseFile))  /* copy the contents of the closing phrase file into a buffer */
   call stream CloseFile,'C','CLOSE'  /* close the closing phrase file */
   Global.MessBody = Global.MessBody||Global.EmptyLine||Closing  /* add the contents of the closing phrase file after an empty line */
  end
 end
 call HeadCheck  /* check the header entries */
 EditText = MakeHeaderLines(Global.MessHead)||Global.CRLF||Global.MessBody  /* the text to write to the edit file */
 EditFile = AttDir||'.EDIT'  /* the matching edit file name */

 if (\OpenFile(EditFile,'WRITE')) then  /* if we cannot open the file for writing */
 do
  return 0  /* just quit */
 end

 call charout EditFile,EditText  /* write the edit text to the edit file */
 call stream EditFile,'C','CLOSE'  /* close the edit file */
end

parse var Global.Editor FirstBit '%N' LastBit  /* look for the filename macro in the editor command line */
CommLine = FirstBit  /* this is the command to start with */
if (pos('%N',Global.Editor) > 0) then  /* if the settings line contains a file spec macro */
do
 CommLine = CommLine||EditFile||LastBit  /* this is the complete command */
end
say 'Editor configuration option parsed as : '||CommLine  /* report */
address cmd CommLine  /* run the command */

if (\IsAddress) then  /* if it's not an address folder file */
do
 if ((stream(MessFile,'C','QUERY EXISTS') >< '') & (stream(EditFile,'C','QUERY EXISTS') >< '')) then  /* if both files still exist */
 do
  OrgSettings = MessageSettings(MessFile)  /* get the message settings from the original file (they may have changed in the meantime) */
  call MessageContents EditFile  /* get the new edit message contents */
  call sysfiledelete MessFile  /* zap the original message file */

  if (\OpenFile(MessFile)) then  /* if we cannot open a new message file for writing */
  do
   return 0  /* quit with an error */
  end

  call HeadCheck  /* check the header entries */
  call charout MessFile,MakeHeaderLines(Global.MessHead)||Global.CRLF||Global.MessBody  /* write the new header, a separating empty line, and the body to the message file */
  call stream MessFile,'C','CLOSE'  /* close the message file */
  call MessageSettings MessFile,OrgSettings,'CHANGE'  /* replace the original message status settings */
  NewName = MakeTitle(MessFile,Global.MessHead)  /* get a new name for the mail file */
  if (pos(Global.Warning,NewName) > 0) then  /* if the header is not O.K. */
  do
   call MessageSettings MessFile,'*-******','CHANGE'  /* set the new message status setting to not O.K. for processing */
  end
  else  /* if the header is O.K. */
  do
   call MessageSettings MessFile,'*+******','CHANGE'  /* set the new message status setting to O.K. for processing */
  end
  call sysfiletree AttDir||'\*.*','Objects.','BO'  /* is there anything in the attachments folder? */
  if (Objects.0 > 0) then  /* if we have attachments */
  do
   NewStore = SetAttachments(MessFile,'CREATE')  /* get a new attachments folder name */
   do Index = 1 to Objects.0  /* take each of the objects */
    if (\sysmoveobject(Objects.Index,NewStore)) then  /* if we cannot move it to the new folder */
    do
     call AddError 'Cannot move "'||Objects.Index||'" to "'||NewStore||'"'  /* report */
    end
   end
  end
  else  /* if there was nothing in the attachments folder */
  do
   call SetAttachments MessFile,'DELETE'  /* get rid of the old attachments folder EA */
  end
  call syssetobjectdata MessFile,'TITLE='||NewName||';'  /* rename the file */
  call sysfiledelete EditFile  /* get rid of the edit file */
 end
 call sysdestroyobject AttDir  /* get rid of the original attachments folder */
end

return 1  /* end of EditMessage */

/**********************************************************************/
ExpandMailList:  /* expand the list of recipients with any mailing list entries we may find */
/**********************************************************************/

NewRecipients = ''  /* start with nothing */
AddressError = 0  /* we have no address errors so far */

do while (Recipients >< '')  /* as long as we have recipients left */

 parse var Recipients NextAddress Recipients  /* get the next address from the list */

 if (pos('@REXXMAIL',translate(NextAddress)) = 0) then  /* if the address does not contain a 'rexxmail' part */
 do
  NewRecipients = NewRecipients||' '||NextAddress  /* just add the address to the new list */
 end
 else  /* if the address contains a 'rexxmail' part */
 do
  parse var NextAddress ListName '@' .  /* get the possible list name from the address */
  ListFile = translate(Global.ConfDir||'\lists\'||ListName)  /* the list file name to look for */
  if (stream(ListFile,'C','QUERY EXISTS') = '') then  /* if there is no mail list file of this name */
  do
   call AddError 'Cannot find mailing list file "'||ListFile||'"'  /* report */
   return 0  /* and quit */
  end
  else  /* if there is a mail list file of that name */
  do
   say 'Including mailing list file "'||ListFile||'"'  /* report */

   if (\OpenFile(ListFile,'READ')) then  /* if we cannot open the mailing list file for reading */
   do
    return 0  /* and quit */
   end

   ListCont = charin(ListFile,1,chars(ListFile))  /* get the contents of the list file */
   do while (ListCont >< '')  /* as long as we have content left */
    parse var ListCont InLine (Global.CRLF) ListCont  /* get the next line */
    if (InLine >< '') then  /* if it is not an empty line */
    do
     if (substr(InLine,1,1) >< '#') then  /* if it is not a comment */
     do
      AddRecipients = FormatAddresses(InLine,,1,0)  /* get a properly formatted and checked address, with all the bits in place */
      if (pos(Global.Warning,AddRecipients) > 0) then  /* if the new address contains a warning marker */
      do
       call AddError 'Malformed recipient address : "'||InLine||'" in "'||ListFile||'"'  /* report */
       AddressError = 1  /* register the error state */
      end
      else  /* if all went well */
      do
       NewRecipients = NewRecipients||' '||AddRecipients  /* add it to the "personalized" list */
      end
     end
    end
   end
   call stream ListFile,'C','CLOSE'  /* close the mailing list file */
  end
 end
end

return (\AddressError)  /* end of ExpandMailList */

/**********************************************************************/
FormatAddresses: procedure expose Global.  /* formats a string of addres(es) from a mail message header line */
/**********************************************************************/

parse arg Addresses,Indent,Check,KeepAll  /* get the arguments */

First = 1  /* a flag to mark the first address */
NewAddresses = ''  /* start with no new addresses */
Check = (Check = 1)  /* if the check arg is 1, we need to check the addresses */
KeepAll = (KeepAll = 1)  /* if the keep all arg is 1, we need to add all comments, whether empty or not  */

do while (Addresses >< '')  /* as long as we have addresses */

 do while ((Addresses >< '') & (pos(substr(Addresses,1,1),' ,') > 0))  /* while the first character of the addresses string is either a blank or a comma (and we still have something left to process) */
  Addresses = substr(Addresses,2)  /* remove the first character from the addresses string */
 end

 if (Addresses >< '') then  /* if we still have something */
 do

  if (pos(Global.Warning,Addresses) = 1) then  /* if the next address starts with a marker */
  do
   Addresses = substr(Addresses,length(Global.Warning)+1)  /* use the rest */
  end

  OrgAddresses = Addresses  /* save the original addresses string */

  Comment = ''  /* we have no leading comment yet */
  EndComment = ''  /* we have no trailing comment yet */

  if (substr(Addresses,1,1) = '"') then  /* if it starts with a quotation mark, it must be a comment preceding the actual address */
  do
   parse var Addresses '"' Comment '"' Addresses  /* get the comment bit */
   Comment = strip(Comment,'B',"'")  /* get rid of any single quotes around the comment */
   Addresses = strip(Addresses,'L',' ')  /* remove any leading blanks from the addresses string */
  end

  parse var Addresses FirstBit '@' Addresses  /* get the first bit of the actual address */

  if ((Comment = '') & (words(FirstBit) > 1)) then  /* if we have no comment yet, and the first address bit contains more than one word, part of it must be a leading comment */
  do
   Comment = subword(FirstBit,1,words(FirstBit)-1)  /* use all but the last word of the first bit for the comment */
   Comment = strip(Comment,'T','"')  /* get rid of any trailing quotation mark to correct asymmetry */
   FirstBit = word(FirstBit,words(FirstBit))  /* use the last word for the first address bit */
  end

  LastBit = ''  /* start with nothing in the last bit of the address */
  GotLastBit = 0  /* we have no last bit of the address yet */

  do until (GotLastBit)  /* go on until we have what we need */
   if (Addresses = '') then  /* if we have no addresses string left */
   do
    GotLastBit = 1  /* this is all we get */
   end
   else  /* if we have addresses stuff left */
   do
    NextChar = substr(Addresses,1,1)  /* get the next character from the remaining addresses string */
    if (pos(NextChar,', (') > 0) then  /* if it is a comma, this is where a new address starts; if it is a space, an end comment may follow, if it is a left parenthesis, we have an end comment */
    do
     GotLastBit = 1  /* we have all we need */
    end
    else  /* if the next character is not a comma, a space, or a left parenthesis */
    do
     LastBit = LastBit||NextChar  /* add it to what we have */
     Addresses = substr(Addresses,2)  /* remove the first character from the remaining addresses string */
    end
   end
  end

  if (substr(word(Addresses,1),1,1) = '(') then  /* if the first word of the remaining addresses string starts with a left parenthesis, we have an end comment */
  do
   parse var Addresses '(' EndComment ')' Addresses  /* extract the end comment */
  end

  Address = CheckAddress(FirstBit,LastBit)  /* see if the address components are O.K., i.e. they make up a new address */

  if (Address = '') then  /* if the address was zapped (because it was bad) */
  do
   CutPoint = length(OrgAddresses) - length(Addresses) + 1  /* where the next original address, if any, starts */
   parse var OrgAddresses Address =(CutPoint) OrgAddresses  /* get the original address */
   if (Check) then  /* if we are checking the address */
   do
    Address = Global.Warning||Address  /* add a marker */
   end
  end
  else  /* if all is well */
  do
   if (pos('<'||Address||'>',NewAddresses) > 0) then  /* if we already have the address in the list of new addresses */
   do
    Address = ''  /* zap it */
   end
   else  /* if it is a new address */
   do
    if (KeepAll) then  /* if we want to keep all the entries */
    do
     Address = '"'||Comment||'" <'||Address||'> ('||EndComment||')'  /* add the separators, and all the comments, even if they are empty */
    end
    else  /* if we do not have to keep everything */
    do
     if (Indent >< '') then  /* if we have an indent value (even if it is 0), we want more than just the bare address */
     do
      Address = '<'||Address||'>'  /* add angle brackets to the bare address */
      if (EndComment >< '') then  /* if we have an end comment */
      do
       if (Comment >< '') then  /* if we have a front comment */
       do
        Comment = Comment||' - '  /* stick on a separator */
       end
       EndComment = translate(EndComment,' ','"')  /* remove any double quotation marks */
       Comment = Comment||EndComment  /* append the end comment to the front comment, if any */
      end
      if ((Comment >< '') & ('<'||Comment||'>' >< Address)) then  /* if we have a comment and it's not the same as the address */
      do
       Address = '"'||Comment||'" '||Address  /* stick it on the front, between double quotes */
      end
     end
    end
   end
  end

  if (Address >< '') then  /* if we have something  */
  do
   if (First) then  /* if this was the first address */
   do
    First = 0  /* the next is no longer the first */
   end
   else  /* if it wasn't the first address */
   do
    if (Indent = '') then  /* if we have no indent value */
    do
     Address = ' '||Address  /* add a blank space */
    end
    else  /* if we have an indent value */
    do
     Address = ','||Global.CRLF||copies(' ',Indent)||Address  /* start an (indented) new line after a comma */
    end
   end
   NewAddresses = NewAddresses||Address  /* add it to the result */
  end

 end

end

return NewAddresses  /* end of FormatAddresses */

/**********************************************************************/
GetAddressData: procedure expose Global.  /* collects template data from the address folder */
/**********************************************************************/

parse arg ExpFile  /* get the argument */

if (\OpenFile(ExpFile,'WRITE')) then  /* if we cannot open the address file for writing */
do
 return 0  /* and quit */
end

OutText = ''  /* start with nothing */

call sysfiletree Global.AddrDir||'\*.*','Files.','FOS'  /* get the address book contents */

do Index = 1 to Files.0  /* take each of the objects */
 if (\OpenFile(Files.Index,'READ')) then  /* try to open the file for reading, and if we cannot do so */
 do
  return 0  /* and quit */
 end

 Lines = charin(Files.Index,1,chars(Files.Index))  /* store the contents of the file in an input buffer */

 call stream Files.Index,'C','CLOSE'  /* close the file */

 parse var Files.Index (Global.AddrDir) '\' Name  /* get the bit of the file spec we want */
 OutText = OutText||'<TITLE>'||Global.CRLF||Name||Global.CRLF  /* add the file name to the output buffer with a header */
 if (Lines >< '') then  /* if we have lines */
 do
  OutText = OutText||'<TEXT>'||Global.CRLF||Lines||Global.CRLF  /* add them to the output buffer with a header */
 end
 KeyPhrases = GetKeyPhrases(Files.Index)  /* get the file's key phrases, if any */
 if (KeyPhrases >< '') then  /* if we have something */
 do
  OutText = OutText||'<KEYWORDS>'||Global.CRLF||KeyPhrases||Global.CRLF  /* add it to the output buffer with a header */
 end
 OutText = OutText||'<END>'||Global.CRLF||Global.CRLF  /* add an ending to the output buffer, and an empty line */
end

InfoFile = Global.Progdir||'\addimpex.txt'  /* the file containing the informational header for the address text file */

if (stream(InfoFile,'C','QUERY EXISTS') >< '') then  /* if the info file exists */
do
 if (OpenFile(InfoFile,'READ')) then  /* try to open the info file, and if we can */
 do
  call charout ExpFile,charin(InfoFile,1,chars(InfoFile))  /* write the contents of the info file to the address text file */
  call stream InfoFile,'C','CLOSE'  /* close the info file */
 end
end

call lineout ExpFile,''  /* write a new line to the file */

call charout ExpFile,OutText  /* write the contents of the output buffer to the file */

call stream ExpFile,'C','CLOSE'  /* close the file */

return 1  /* end of GetAddressData */

/**********************************************************************/
GetFileEntry: procedure expose Global.  /* looks for key word in files and returns last line with matching content */
/**********************************************************************/

parse arg FileName,SeekStr  /* the usual stuff */

SeekStr = ' '||SeekStr||';'  /* add a space and semi-colon */

if (sysfilesearch(SeekStr,FileName,'Found.') >< 0) then  /* if we cannot search the file for the seek string */
do
 call AddError 'Cannot search for "'||SeekStr||'" in "'||FileName||'"'  /* report */
 return ''  /* and quit with nothing */
end

OutLine = ''  /* start with no result */

do Index = 1 to Found.0  /* look through the lines we found */
 if (substr(Found.Index,1,1) >< '#') then  /* if the line we found is not a comment */
 do
  parse var Found.Index OutLine '=' .  /* the bit before the '=' is what we're looking for */
 end
end

if (OutLine >< '') then  /* if we have a result */
do
 OutLine = translate(OutLine,d2c(32),d2c(9))  /* convert TABs into spaces */
 OutLine = strip(OutLine)  /* get rid of any excess spaces */
end

return OutLine  /* end of GetFileEntry */

/**********************************************************************/
GetHeaderEntry: procedure expose Global.  /* returns entry for matching key word in message header */
/**********************************************************************/

parse arg HeaderLines,SeekStr,More  /* get the header lines, the seek string, and any "MORE" parameter */

More = (More = 1)  /* if we have no "More = 1" parameter, default to 0 */

SeekStr = translate(SeekStr)  /* convert the seek string to upper case */

OutLine = ''  /* start with nothing */

do while ((HeaderLines >< '') & ((OutLine = '') | (More)))  /* go on until we run out of header content or find what we want */
 parse var HeaderLines HeaderLine (Global.CRLF) HeaderLines  /* get the relevant bits */
 if (SeekStr = translate(word(HeaderLine,1))) then  /* if the uppercased first word is the one we're looking for */
 do
  Entry = subword(HeaderLine,2)  /* the bit we want */
  if (Entry = '') then  /* if there is nothing in it */
  do
   Entry = d2c(9)  /* use a single tab */
  end
  if (OutLine >< '') then  /* if we already have something */
  do
   Entry = Global.CRLF||Entry  /* add a new line */
  end
  OutLine = OutLine||Entry  /* add the new bit to the output text */
 end
end

return OutLine  /* end of GetHeaderEntry */

/**********************************************************************/
GetHeaderStuff:  /* Get the sender, recipient, and subject info from the message header */
/**********************************************************************/

parse var MessCont MessHead (Global.EmptyLine) MessBody  /* split the message into its header and body */

Sender = ''  /* start with nothing */
Recipients = ''  /* we have no recipients yet */
Subject = ''  /* we have no subject yet */
LookFold = 0  /* no need to start looking for folded lines yet */
GotBcc = 0  /* we are not in a "Bcc:" bit yet */

Header = MessHead  /* copy the header contents */
MessHead = ''  /* start again */
RecpHead = ''  /* start with nothing */

do while (Header >< '')  /* as long as we have header contents left */
 parse var Header NextLine (Global.CRLF) Header  /* get the next line */
 NextLine = translate(NextLine,d2c(32),d2c(9))  /* convert TABs to spaces */
 if ((LookFold) & (substr(NextLine,1,1) = d2c(32))) then  /* if we are looking for folded lines, and the line starts with whitespace */
 do
  NewRecipient = FormatAddresses(NextLine,,0,0)  /* extract the bare address from the line */
  if (wordpos(NewRecipient,Recipients) = 0) then  /* if the address is not in the list */
  do
   Recipients = Recipients||' '||NewRecipient  /* add it */
  end
 end
 else  /* if we aren't looking for folded lines, or if the line is not a folded bit */
 do
  if (wordpos(translate(word(NextLine,1)),'TO: CC: BCC:') > 0) then  /* if it is a recipient line */
  do
   NewRecipient = FormatAddresses(subword(NextLine,2),,0,0)  /* extract the bare e-mail addresses from the second and any following words */
   if (wordpos(NewRecipient,Recipients) = 0) then  /* if the address is not in the list */
   do
    Recipients = Recipients||' '||NewRecipient  /* add it */
   end
   LookFold = 1  /* look out for folded lines that may follow */
   GotRec = 1  /* we have a recipient line */
   GotBcc = (translate(word(NextLine,1)) = 'BCC:')  /* if it is a "Bcc:" line, set a flag */
  end
  else  /* if this is not a recipient line */
  do
   if (Subject = '') then  /* if we have no subject yet */
   do
    if (translate(word(NextLine,1)) = 'SUBJECT:') then  /* if this is the first subject line */
    do
     Subject = subword(NextLine,2)  /* the rest of the line must be the subject */
    end
   end
   if (Sender = '') then  /* if we have no sender yet */
   do
    if (translate(word(NextLine,1)) = 'FROM:') then  /* if it is the "From:" line */
    do
     Sender = FormatAddresses(subword(NextLine,2),,0,0)  /* extract the bare e-mail address from the second and any following words */
    end
   end
   LookFold = 0  /* no need to look (further) for folded lines */
   GotRec = 0  /* we are not in a recipient line */
   GotBcc = 0  /* it is not a "Bcc:" line */
  end
 end
 if (\GotBcc) then  /* if we are not in a "Bcc:" bit */
 do
  if (GotRec) then  /* if this is a recipient line */
  do
   RecpHead = RecpHead||NextLine||Global.CRLF  /* add the line to the new recipient header lines */
  end
  else  /* if it is not a recipient line */
  do
   MessHead = MessHead||NextLine||Global.CRLF  /* add the line to the new normal header lines */
  end
 end
end

if ((Sender = '') | (Recipients = '')) then  /* if we have a missing sender or recipient */
do
 call AddError 'Missing sender or recipients'  /* report */
 Success = 0  /* we have no success */
end
else  /* if we have both sender and recipient(s) */
do
 Success = 1  /* all is well */
end

return Success  /* end of GetHeaderStuff */

/**********************************************************************/
GetKeyPhrases: procedure expose Global.  /* gets key phrases from an address book entry */
/**********************************************************************/

parse arg Entry  /* get the argument */

OutLines = ''  /* nothing yet */

call sysgetea Entry,'.KEYPHRASES','KeyPhrases'  /* look for key phrases */
if (KeyPhrases >< '') then  /* if we found key phrases */
do
 parse var KeyPhrases EAT 3 CodePage 5 Count 7 Contents  /* get the bits we want */
 if (EAT = reverse('FFDF'x)) then  /* if it is a multiple value, multiple type entry */
 do
  Count = c2d(reverse(Count))  /* extract the number of keyword lines */
  do Index = 1 to Count  /* take each line entry */
   parse var Contents EAT 3 Len 5  /* get the bits we want */
   if (EAT = reverse('FFFD'x)) then  /* it it's ASCII text (which it should be) */
   do
    Length = c2d(reverse(Len))  /* get the length of the next line */
    parse var Contents 5 Text +(Length) Contents  /* get the bits we want */
    if (Length > 0) then  /* if we have something */
    do
     OutLines = OutLines||Text||Global.CRLF  /* add a CRLF and Text */
    end
   end
  end
 end
end

if (OutLines >< '') then  /* if we found something */
do
 OutLines = substr(OutLines,1,length(OutLines)-2)  /* get rid of the excess CRLF at the end */
end

return OutLines  /* end of GetKeyPhrases */

/**********************************************************************/
GetLogEntry: procedure expose Global.  /* gets header entries for the messages log */
/**********************************************************************/

parse arg LookFor  /* get the argument */

Entry = ''  /* nothing yet */

do until ((Entry >< '') | (LookFor = ''))  /* go on until we find something or run out of options */
 parse var LookFor NextOne LookFor  /* get the next option */
 Entry = GetHeaderEntry(Global.MessHead,NextOne)  /* look for the entry in the message header */
end

return Entry  /* end of GetLogEntry */

/**********************************************************************/
GetMoveDir: procedure expose Global.  /* check for a move directory in the move file */
/**********************************************************************/

parse arg MoveFile,SearchText  /* get the arguments */

MoveDir = ''  /* start with nothing */

MoveDir = GetFileEntry(MoveFile,SearchText)  /* look in the "Move" file for a folder to go with the address */

if ((MoveDir = '') & (pos('@',SearchText) > 0)) then  /* if we find nothing and if the search text contains an @ */
do
 parse var SearchText . '@' Domain  /* split the text into the bits on either side of the @ */
 MoveDir = GetFileEntry(MoveFile,Domain)  /* look in the "Move" file for a folder to go with the domain name */
end

return MoveDir  /* end of GetMoveDir */

/**********************************************************************/
GetNames: procedure expose Global.  /* extract a list of names from a formatted list of addresses */
/**********************************************************************/

parse arg Addresses  /* get the argument */

Names = ''  /* start with nothing */
Separator = ','||Global.CRLF  /* the separator between full addresses */

do while (Addresses >< '')  /* as long as we have bits left */
 if (Names >< '') then  /* if we already have something */
 do
  Names = Names||'. '  /* add a separator */
 end
 parse var Addresses Address (Separator) Addresses  /* get the next address */
 if (words(Address) > 1) then  /* if the address contains more than one word */
 do
  NamePart = strip(subword(Address,1,words(Address)-1),'B','"')  /* all but the last must be the name, and get rid of any quotation marks */
  Address = subword(Address,words(Address))  /* the last is the address */
 end
 else  /* if the address contains just a single word */
 do
  NamePart = ''  /* we have no name part */
 end

 Global.TitleAddress = translate(substr(Global.TitleAddress,1,1))  /* use just the first character, in upper case */

 select  /* do one of the following */
  when (Global.TitleAddress = 'F') then  /* if we want it all */
  do
   if (NamePart >< '') then  /* if we have a name part */
   do
    NewName = NamePart||' '||Address  /* use this */
   end
   else  /* if we do not have a name part */
   do
    NewName = Address  /* use this */
   end
  end
  when (Global.TitleAddress = 'N') then  /* if we want just the name */
  do
   if (NamePart >< '') then  /* if we have a name part */
   do
    NewName = NamePart  /* use it */
   end
   else  /* if we do not have a name part */
   do
    NewName = Address  /* use the address */
   end
  end
  otherwise  /* if we want just the address */
  do
   Address = strip(Address,'L','<')  /* get rid of the leading angle bracket */
   Address = strip(Address,'T','>')  /* get rid of the trailing angle bracket */
   NewName = Address  /* use the address */
  end
 end

 Names = Names||NewName  /* add the result to the names */
end

return Names  /* end of GetNames */

/**********************************************************************/
GetTimeZone: procedure expose Global.  /* gets the time zone by comparing local time with an RFC 868 time server using TCP */
/**********************************************************************/

TimeZone = ''  /* start with nothing */

do while ((TimeZone = '') & (Global.TimeServers >< ''))  /* as long as we have no result, and we have time servers left */
 parse var Global.TimeServers TimeServer Global.TimeServers  /* get the next entry */
 call charout StdOut,'Getting standard time from '||TimeServer||' : '  /* report */
 Socket = ServerConnect(TimeServer,37)  /* get the socket number for a server connection through port 25 */
 if (Socket = '') then  /* if we have no socket */
 do
  say 'failed.'  /* report a miss */
 end
 else  /* if we have a socket */
 do
  say 'O.K.'  /* report */
  SocketBytes = sockrecv(Socket,'TimeBytes',4)  /* see how many bytes we get from the server; look for 4 */
  call SocketClose Socket  /* close the socket */
  if (SocketBytes = 4) then  /* if we got 4 bytes */
  do
   TimeZone = TimeZoneString(TimeBytes)  /* get the time from the server and construct the time zone */
  end
 end
end

if (TimeZone = '') then  /* if we still have no time zone */
do
 TimeZone = '-0000'  /* use this */
end

return TimeZone  /* end of GetTimeZone */

/**********************************************************************/
GetTitleDate: procedure expose Global.  /* gets the date for the object title */
/**********************************************************************/

parse arg MessHead,MessFile  /* get the arguments */

TitleDate = GetHeaderEntry(MessHead,'DATE:')  /* get the date */
if ((TitleDate = '') | (TitleDate = d2c(9))) then  /* if we have no date */
do
/*
   TitleDate = stream(MessFile,'C','QUERY TIMESTAMP')  /* use the last write date & time */
   parse var TitleDate DDate ' ' TTime                /* separate the two components */
   DoW = ' ('||substr(date('W'),1,3)||')'              /* the first three characters of the day of the week in brackets */
   TitleDate = DDate||' '||TTime||DoW        /* stick all together with just one space between the date and time */
   */
 TitleDate = sysgetfiledatetime(MessFile,'A')||' ('||substr(date('W'),1,3)||')'  /* use the last access time and DOW string between parentheses */
end
else  /* if we have a date */
do
 TitleDate = SortedDate(TitleDate)  /* rework the date string */
end

return TitleDate  /* end of GetTitleDate */

/**********************************************************************/
GetTitleRecipients: procedure expose Global.  /* gets the recipients for the object title */
/**********************************************************************/

parse arg MessHead,EmptyStart,EmptyEnd,OutGoing  /* get the arguments */

TitleRecipients = GetHeaderEntry(MessHead,'TO:')  /* get the recipient */

if (TitleRecipients = '') then  /* if we have no recipient */
do
 TitleRecipients = GetHeaderEntry(MessHead,'APPARENTLY-TO:')  /* get the sender this way */
end

if ((TitleRecipients = '') | (TitleRecipients = d2c(9))) then  /* if we still have no recipient */
do
 TitleRecipients = EmptyStart||'no recipient'||EmptyEnd  /* use this */
end
else  /* if we have something */
do
 TitleRecipients = FormatAddresses(TitleRecipients,0,OutGoing,0)  /* get a reformatted address string, no indent, check if outgoing, no end comments */
 if (pos(Global.Warning,TitleRecipients) > 0) then  /* if it contains a marker */
 do
  TitleRecipients = Global.Warning||' illegal recipient'  /* insert a warning */
 end
 else  /* if there is no marker */
 do
  TitleRecipients = GetNames(TitleRecipients)  /* extract a list of names */
 end
end

TitleCc = GetHeaderEntry(MessHead,'CC:')  /* look for Cc addresses */
if (TitleCc >< '') then  /* if CC addresses were found */
do
 TitleCc = FormatAddresses(TitleCc,0,OutGoing,0)  /* get a reformatted address string, no indent, check if outgoing, no end comments */
 if (pos(Global.Warning,TitleCc) > 0) then  /* if it contains a marker */
 do
  TitleRecipients = Global.Warning||' illegal recipient'  /* insert a warning */
 end
 else  /* if there is no marker */
 do
  TitleCc = GetNames(TitleCc)  /* extract a list of names */
  TitleRecipients = TitleRecipients||' CC: '||TitleCc  /* add them to the list of recipients */
 end
end

TitleBcc = GetHeaderEntry(MessHead,'BCC:')  /* look for Bcc addresses */
if (TitleBcc >< '') then  /* if BCC addresses were found */
do
 TitleBcc = FormatAddresses(TitleBcc,0,OutGoing,0)  /* get a reformatted address string, no indent, check if outgoing, no end comments */
 if (pos(Global.Warning,TitleBcc) > 0) then  /* if it contains a marker */
 do
  TitleRecipients = Global.Warning||' illegal recipient'  /* insert a warning */
 end
 else  /* if there is no marker */
 do
  TitleBcc = GetNames(TitleBcc)  /* extract a list of names */
  TitleRecipients = TitleRecipients||' BCC: '||TitleBcc  /* add them to the list of recipients */
 end
end

if (pos(Global.Warning,TitleRecipients) = 0) then  /* if the title contains no marker */
do
 TitleRecipients = DecodeHeaderLine(TitleRecipients)  /* decode MIME stuff */
end

return TitleRecipients  /* end of GetTitleRecipients */

/**********************************************************************/
GetTitleSender: procedure expose Global.  /* gets the sender for the object title */
/**********************************************************************/

parse arg MessHead,EmptyStart,EmptyEnd,OutGoing  /* get the arguments */

TitleSender = GetHeaderEntry(MessHead,'FROM:')  /* get the sender */

if (TitleSender = '') then  /* if we have nothing */
do
 TitleSender = GetHeaderEntry(MessHead,'APPARENTLY-FROM:')  /* get the sender this way */
 if (TitleSender = '') then  /* if we have no sender */
 do
  TitleSender = GetHeaderEntry(MessHead,'SENDER:')  /* get the sender this way */
  if (TitleSender = '') then  /* if we have no sender */
  do
   TitleSender = GetHeaderEntry(MessHead,'FROM')  /* get the sender this way */
  end
 end
end

if ((TitleSender = '') | (TitleSender = d2c(9))) then  /* if we still have no sender */
do
 TitleSender = EmptyStart||'no sender'||EmptyEnd  /* use this */
end
else  /* if we have something */
do
 TitleSender = FormatAddresses(TitleSender,0,OutGoing,0)  /* get a reformatted address string, no indent, check if outgoing, no end comments */

 if (pos(Global.CRLF,TitleSender) > 0) then  /* if it contains more than one address */
 do
  TitleSender = Global.Warning||' multiple senders'  /* insert a warning */
 end
 else  /* if there is only one address */
 do
  if (pos(Global.Warning,TitleSender) > 0) then  /* if it contains a marker */
  do
   TitleSender = Global.Warning||' illegal sender'  /* insert a warning */
  end
  else  /* if there is no marker */
  do
   TitleSender = GetNames(TitleSender)  /* extract a list of names */
   TitleSender = DecodeHeaderLine(TitleSender)  /* decode any MIME stuff */
  end
 end
end

return TitleSender  /* end of GetTitleSender */

/**********************************************************************/
GetTitleSubject: procedure expose Global.  /* gets the subject for the object title */
/**********************************************************************/

parse arg MessHead,EmptyStart,EmptyEnd  /* get the arguments */

TitleSubject = GetHeaderEntry(MessHead,'SUBJECT:')  /* get the subject */

if ((TitleSubject = '') | (TitleSubject = d2c(9))) then  /* if we have no subject */
do
 TitleSubject = EmptyStart||'no subject'||EmptyEnd  /* use this */
end
else  /* if we have something */
do
 TitleSubject = DecodeHeaderLine(TitleSubject)  /* decode any MIME stuff */
end

return TitleSubject  /* end of GetTitleSubject */

/**********************************************************************/
HeadCheck: procedure expose Global.  /* checks header contents */
/**********************************************************************/

OKKeywords = 'FROM:',  /* allow a from string */
             'TO:',  /* allow a to string */
             'CC:',  /* allow a cc string */
             'BCC:',  /* allow a bcc string */
             'SENDER:',  /* allow a sender string */
             'REPLY-TO:',  /* allow a reply to string */
             'RESENT-FROM:',  /* allow a resent from string */
             'RESENT-TO:',  /* allow a resent to string */
             'APPARENTLY-FROM:',  /* allow an apparently-from string */
             'APPARENTLY-TO:',  /* allow an apparently-to string */
             'DATE:',  /* allow a date string */
             'SUBJECT:',  /* allow a subject string */
             'COMMENTS:',  /* allow a comments string */
             'IN-REPLY-TO:',  /* allow an in reply to string */
             'REFERENCES:',  /* allow a references string */
             'KEYPHRASES:'  /* allow a keyphrases string */

NewList = ''  /* start afresh */
NewHeader = ''  /* start with nothing */
HeadText = Global.MessHead  /* copy the unfolded header contents */

do while (HeadText >< '')  /* go on while we have bits of unfolded header left */
 parse var HeadText HeadLine (Global.CRLF) HeadText  /* get the next line */
 if (pos(Global.Warning,HeadLine) = 1) then  /* if the line starts with a marker */
 do
  HeadLine = substr(HeadLine,length(Global.Warning)+1)  /* use the remainder */
 end
 FirstWord = word(HeadLine,1)  /* get the first word of the line */
 UpFirstWord = translate(Firstword)  /* the uppercase version of the first word */
 if (wordpos(UpFirstword,NewList) > 0) then  /* if it is in the new keywords list, we already have this one */
 do
  HeadLine = Global.Warning||HeadLine  /* add a marker */
 end
 else  /* if it is a new keyword */
 do
  NewList = NewList||' '||UpFirstWord  /* add the keyword to the list of new keywords */
  if (wordpos(UpFirstword,OKKeywords) = 0) then  /* if it is not in the O.K. keywords list */
  do
   if ((substr(UpFirstword,1,2) >< 'X-') | (substr(Firstword,length(FirstWord),1) >< ':')) then  /* if it is not an X-string followed by a colon, it's a rogue entry */
   do
    HeadLine = Global.Warning||HeadLine  /* so add a marker */
   end
  end
 end
 NewHeader = NewHeader||HeadLine||Global.CRLF  /* add this line to the unfolded header */
end

Global.MessHead = NewHeader  /* copy the unfolded header back to the global variable */

return 1  /* end of HeadCheck */

/**********************************************************************/
LoadRexxSock: procedure expose Global.  /* try to load the REXX socket functions library if necessary */
/**********************************************************************/

if (rxfuncquery('SockLoadFuncs') >< 0) then  /* if the REXX socket functions lib is not already loaded */
do
 if (rxfuncadd('SockLoadFuncs','RxSock','SockLoadFuncs') = 0) then  /* if we can register the general loading function */
 do
  if (sockloadfuncs() = 0) then  /* if we cannot load the functions */
  do
   Result = 2  /* bad result */
  end
  else  /* if we can load them */
  do
   Result = 1  /* indicate we loaded the library */
  end
 end
end
else  /* if the library had already been loaded */
do
 Result = 0  /* indicate that the REXX socket library had already been loaded */
end

return Result  /* end of LoadRexxSock */

/**********************************************************************/
LoadRexxUtil: procedure expose Global.  /* try to load the REXX utilities library */
/**********************************************************************/

if (rxfuncquery('SysLoadFuncs')) then  /* if it is not already loaded */
do
 if (rxfuncadd('SysLoadFuncs','RexxUtil','SysLoadFuncs') = 0) then  /* if we can add it */
 do
  if (SysLoadFuncs() = 0) then  /* if we cannot load it */
  do
   Result = 2  /* bad result */
  end
  else  /* if we can load it */
  do
   Result = 1  /* indicate we loaded the library */
  end
 end
end
else  /* if the library had already been loaded */
do
 Result = 0  /* indicate that the REXX utilities library had already been loaded */
end

return Result  /* end of LoadRexxUtil */

/**********************************************************************/
LogMessage: procedure expose Global.  /* log a sent or collected message */
/**********************************************************************/

if (Global.MailLogs) then  /* if we are to log messages */
do
 MailLog = Global.ConfDir||'\maillogs.txt'  /* the name of the mail log file */

 if (\OpenFile(MailLog,'WRITE')) then  /* if we cannot open the file for writing */
 do
  MessText = 'Cannot open log file : "'||MailLog||'"'  /* the text to display/send */
  say MessText  /* report to StdOut */
  call SystemMessage 'Mail log file write error.',MessText  /* send a message to the user if required */
  return 0  /* return with no success */
 end

 Separator = ' -- '  /* define a separator */

 DateStr = date('S')  /* get the sorted date */
 LogLine = substr(DateStr,1,4)||'-'||,  /* start the date & time stamp with the year in 4 digits */
           substr(DateStr,5,2)||'-'||,  /* followed by the month in 2 digits */
           substr(DateStr,7,2)||' '||,  /* followed by the day of the month in 2 digits */
           time('N')  /* followed by the normal time */

 parse arg Direction,Sender,Recipients,Subject,MessID  /* get the arguments */

 LogLine = LogLine||Separator||word('<== ==>',wordpos(translate(Direction),'IN OUT'))  /* translate "in" or "out" into an appropriate marker and add it to the log line */

 if (Recipients = '') then  /* if we have no recipients */
 do
  MessFile = Sender  /* we must be looking for a file name */

  if (\OpenFile(MessFile,'READ')) then  /* if we cannot open the file for reading */
  do
   return 0  /* quit */
  end

  call MessageContents MessFile  /* get the message contents */

  call stream MessFile,'C','CLOSE'  /* close the file */

  LookFor = 'From:',  /* look for this entry in the header, */
            'Apparently-From:',  /* then this one */
            'Sender:',  /* then this one */
            'From:'  /* then this one */
  Sender = FormatAddresses(GetLogEntry(LookFor),,0,0)  /* get the bare sender string */

  LookFor = 'To:',  /* look for this entry in the header, */
            'Apparently-To:'  /* then this one */
  Recipients = FormatAddresses(GetLogEntry(LookFor),,0,0)  /* get the bare recipient(s) string */

  LookFor = 'Cc:'  /* look for this entry in the header */
  MoreRecipients = FormatAddresses(GetLogEntry(LookFor),,0,0)  /* get the bare CC recipient(s) string */
  if (MoreRecipients >< '') then  /* if we find more recipients */
  do
   Recipients = Recipients||', Cc: '||MoreRecipients  /* add them to what we have */
  end

  LookFor = 'Bcc:'  /* look for this entry in the header */
  MoreRecipients = FormatAddresses(GetLogEntry(LookFor),,0,0)  /* get the bare CC recipient(s) string */
  if (MoreRecipients >< '') then  /* if we find more recipients */
  do
   Recipients = Recipients||', Bcc: '||MoreRecipients  /* add them to what we have */
  end

  LookFor = 'Subject:'  /* look for this entry in the header */
  Subject = GetLogEntry(LookFor)  /* get the subject string */

  LookFor = 'Message-ID:'  /* look for this entry in the header */
  MessID = GetLogEntry(LookFor)  /* get the subject string */

 end

 LogLine = LogLine||Separator||'From: '||Sender  /* add the sender */
 LogLine = LogLine||Separator||'To: '||Recipients  /* add the recipients */
 LogLine = LogLine||Separator||'Subject: '||Subject  /* add the subject */
 LogLine = LogLine||Separator||'Message-Id: '||MessID  /* add the message ID */

 call lineout MailLog,LogLine  /* write the log line */
 call stream MailLog,'C','CLOSE'  /* close the log file */
end

return 1  /* end of LogMessage */

/**********************************************************************/
MailWarning: procedure expose Global.  /* shows that mail has arrived, or deletes its own program object */
/**********************************************************************/

parse arg Delete  /* get the parameter */

Delete = (Delete = 1)  /* default to no delete */

Success = 0  /* no success yet */

ProgID = '<RexxMail_Warning_Object>'  /* the "Mail!" warning program object ID */

if (Delete) then  /* if this is the parameter */
do
 if (sysdestroyobject(ProgID)) then  /* if we can remove the warning object */
 do
  Success = 1  /* all is well */
 end
end
else  /* if there is no such parameter */
do
 if (Global.WarnIcon) then  /* if we want a warning icon */
 do
  GotMain = 0  /* we have not spotted the main mail folder yet */
  if (sysqueryswitchlist('Stem.') = 0) then  /* if we can store the window list contents in Stem. */
  do Index = 1 to Stem.0  /* take each entry */
   if (translate(Stem.Index) = filespec('N',Global.MainDir)) then  /* if it is identical to the main mail folder */
   do
    GotMain = 1  /* the mail folder must be open on the desktop */
   end
  end
  if (\GotMain) then  /* if the main mail folder is not open on the desktop */
  do
   Settings = 'EXENAME='||Global.ProgDir||'\'||Global.ProgName||';'||,  /* opening the object will call RexxMail */
              'PARAMETERS=/OPEN;MINIMIZED=YES;'||,  /* the program will run minimized, with the "/Open" switch */
              'OBJECTID='||ProgID||';'||,  /* use the object ID defined earlier */
              'ICONFILE='||Global.IconDir||'\mailwarn.ico'  /* attach the MailWarn icon file */
   Success = (syscreateobject('WPProgram','Mail!','<WP_DESKTOP>',Settings))  /* all is well if we can create the program object */
  end
 end
end

return Success  /* end of MailWarning */

/**********************************************************************/
MakeAddressEntry: procedure expose Global.  /* extracts address from mail file and adds a template to address book folder */
/**********************************************************************/

parse arg MessFile  /* get the argument */

call MessageContents MessFile  /* get the message contents */

if (MessageSettings(MessFile,'+*******','MATCH')) then  /* if it is an incoming message */
do
 Address = GetHeaderEntry(Global.MessHead,'FROM:')  /* use this address */
end
else  /* if it is not an incoming message */
do
 if (MessageSettings(MessFile,'-*******','MATCH')) then  /* if it is an outgoing message */
 do
  Address = GetHeaderEntry(Global.MessHead,'TO:')  /* use this address */
 end
 else  /* if it is not an outgoing message either */
 do
  call AddError 'Not a valid mail message'  /* report */
  return 0  /* and quit */
 end
end

Address = DecodeHeaderLine(Address)  /* decode the address if necessary */
Address = FormatAddresses(Address,4,0,0)  /* reformat the address, indent to match "To: ", no check, no end comments */

if ((pos(Address,d2c(9)||' ') > 0) | (pos(Global.Warning,Address) > 0)) then  /* if there is no address, or if it contains a marker */
do
 call AddError 'No valid address found'  /* report */
 return 0  /* and quit */
end

Title = translate(Address," ()",'"<>')  /* change dangerous (command line!) characters for the object title */
Title = strip(Title,'B',' ')  /* get rid of excess spaces */
Title = '! '||Title||d2c(255)  /* stick on a marker, and a hard space to prevent file filter problems */

OutFile = Global.AddrDir||'\'||Title  /* the file to write to */

if (stream(OutFile,'C','QUERY EXISTS') >< '') then  /* if such a file already exists in the address directory */
do
 call AddError 'Address template already exists'  /* report */
 return 0  /* and quit */
end

OutText = 'To: '||Address||Global.CRLF||'Subject: '  /* line up the output text */

if (\OpenFile(OutFile)) then  /* if we cannot open the file */
do
 return 0  /* and quit */
end

call charout OutFile,OutText  /* write the output text to the file */
call stream OutFile,'C','CLOSE'  /* close the file */

if (\MessageSettings(OutFile,'--------','Change')) then  /* if we can not make it a fresh outgoing mail message file */
do
 call AddError 'Cannot set address template message attributes'  /* report */
 return 0  /* and quit */
end

if (\syssetobjectdata(OutFile,'TEMPLATE=YES;')) then  /* if we cannot make the file a template */
do
 call AddError 'Cannot set address template attribute'  /* report */
 return 0  /* and quit */
end

return 1  /* end of MakeAddressEntry */

/**********************************************************************/
MakeAttachMessage: procedure expose Global.  /* generate an attachment message in the 'Out' folder */
/**********************************************************************/

parse arg AttachFile,Switch  /* get the arguments */

AttachFileName = filespec('N',AttachFile)  /* get the name part of the attachment file spec */
AttachFileDir = filespec('D',AttachFile)||filespec('P',AttachFile)  /* get the path part of the attachment file spec */

if (Switch = 'ATTACHHERE') then  /* if we want the message file in the same directory as the attachment file */
do
 MessFile = systempfilename(AttachFileDir||'RXML????.MESS')  /* get a unique file name */
end
else  /* if not, it's going into the 'Out' folder */
do
 MessFile = systempfilename(Global.OutDir||'\RXML????.MESS')  /* get a unique file name */
end

if (\OpenFile(MessFile)) then  /* if we cannot open the message read file */
do
 return 0  /* return with an error */
end

OutText = 'To: '||Global.CRLF||'Subject: '||AttachFileName||Global.CRLF  /* the text to put into the file */

call charout MessFile,OutText  /* write the text to the message file */
call stream MessFile,'C','CLOSE'  /* close the message file */

AttDir = SetAttachments(MessFile)  /* create an attachments dir */

if (\syscopyobject(AttachFile,AttDir)) then  /* if we cannot copy the attachment file to the attachment dir */
do
 call AddError 'Cannot copy "'||AttachFile||'" to "'||AttDir||'"'  /* report; not fatal */
 return 0  /* return with an error */
end

call MessageSettings MessFile,'--------','CHANGE'  /* make it an outgoing mail message file */

NewName = 'New Message'||d2c(10)||'Attachment : '||AttachFileName||d2c(255)  /* the new title to use */

if (\syssetobjectdata(MessFile,'TITLE='||NewName)) then  /* if we cannot rename the file */
do
 call AddError 'Cannot rewrite title of "'||MessFile||'"'  /* report; not fatal */
end

return 1  /* end of MakeAttachMessage */

/**********************************************************************/
MakeFromLine: procedure expose Global.  /* make a "From:" line */
/**********************************************************************/

if (Global.Address >< '') then  /* if we have a default sender's address */
do
 FromLine = strip(Global.Address,'L','<')  /* get rid of any leading angle bracket */
 FromLine = strip(FromLine,'T','>')  /* get rid of any trailing angle bracket */
 FromLine = '<'||FromLine||'>'  /* add angle brackets */
 if (Global.Name >< '') then  /* if we have a default sender's name */
 do
  FromName = strip(Global.Name,'B','"')  /* get rid of any double quotes */
  FromName = strip(FromName,'B',"'")  /* get rid of any single quotes */
  FromLine = '"'||FromName||'" '||FromLine  /* add double quotes and put the name in front of the address */
 end
end
else  /* if we do not have a default sender's address */
do
 FromLine = ''  /* start with nothing */
end

return FromLine  /* end of MakeFromLine */

/**********************************************************************/
MakeHeaderLines: procedure expose Global.  /* gets header info from original message file and creates header text block */
/**********************************************************************/

parse arg MessHead,KeywordList  /* get the argument */

Header. = ''  /* start with nothing */
HeadCount = 0  /* start counting header lines at 0 */

if (KeywordList = '') then  /* if we do not have a list of keywords */
do
 Outgoing = 1  /* the message is outgoing */
 do while (MessHead >< '')  /* as long as we have a bit of header left */
  parse var MessHead HeadLine (Global.CRLF) MessHead  /* get the next header line */
  HeadCount = HeadCount + 1  /* up the counter */
  Header.HeadCount = HeadLine  /* store the header line */
 end
end
else  /* if we have a list of keywords */
do
 Outgoing = 0  /* the message is incoming */
 do while (KeywordList >< '')  /* while we have something left in this variable */
  parse var KeywordList Keyword KeywordList  /* get the next word from the list of keywords */
  if (substr(Keyword,1,1) = '*') then  /* if it starts with "*" */
  do
   Keyword = substr(Keyword,2)  /* use the rest */
   More = 1  /* we want more than the first occurrence */
  end
  else  /* if the word does not start with "*" */
  do
   More = 0  /* we don't want more */
  end
  HeadLine = GetHeaderEntry(MessHead,Keyword||':',More)  /* look for this entry in the message header */
  if (HeadLine >< '') then  /* if we have something */
  do
   HeadCount = HeadCount + 1  /* up the counter */
   Header.HeadCount = Keyword||': '||HeadLine  /* store the header line */
  end
 end
end

Reformats = 'FROM:',  /* reformat addresses */
            'TO:',  /* reformat addresses */
            'REPLY-TO:',  /* reformat addresses */
            'CC:',  /* reformat addresses */
            'BCC:',  /* reformat addresses */
            'APPARENTLY-FROM:',  /* reformat addresses */
            'SENDER:',  /* reformat addresses */
            'RESENT-FROM:',  /* reformat addresses */
            'RESENT-TO:',  /* reformat addresses */
            'ATTACHMENTS:'  /* reformat attachments list */

DateLines = 'DATE:',  /* reformat date if required */
            'RESENT:',  /* reformat date if required */
            'RECEIVED:',  /* reformat date if required */
            'SENT:'  /* reformat date if required */

Indent = 0  /* start with an indent of 0 */

MaxIndent = 16  /* the maximum indent value */

do Index = 1 to HeadCount  /* run through the header lines */
 NewLength = length(word(Header.Index,1))  /* get the length of the first word */
 if ((NewLength > Indent) & (NewLength <= MaxIndent)) then  /* if this word is longer than any of the previous ones, but not over the limit */
 do
  Indent = NewLength  /* adjust the indent value accordingly */
 end
end

Indent = Indent + 2  /* add extra space after the colon */

NewHeader = ''  /* start with nothing */

do Index = 1 to HeadCount  /* run through the header lines again */

 HeadLine = Header.Index  /* get the header line */
 parse var HeadLine Keyword NextEntry  /* get the two components */

 NextEntry = DecodeHeaderLine(NextEntry)  /*  decode any MIME stuff */

 if (substr(Keyword,length(Keyword)) = ':') then  /* if it is a real keyword ending with a colon */
 do
  if (wordpos(translate(Keyword),Reformats) > 0) then  /* if this is one we want to reformat */
  do
   NextEntry = FormatAddresses(NextEntry,Indent,Outgoing,0)  /* get a formatted string of addresses, indented, checked if outgoing, no end comments */
  end
  else  /* if this is not one we want to reformat */
  do
   if (wordpos(translate(Keyword),DateLines) > 0) then  /* if it is one with a date in it */
   do
    if (translate(Keyword) = 'RECEIVED:') then  /* if it is this one */
    do
     parse var NextEntry RouteInfo ';' NextEntry  /* try to get just the received date */
    end
    NextEntry = SortedDate(NextEntry)  /* rewrite the date if necessary */
   end
   else  /* if it is not a date entry */
   do
    NextEntry = WordWrap(NextEntry,78-Indent,Indent,1)  /* word-wrap it at 78 chars, indent, no word breaks */
   end
  end
  NewHeader = NewHeader||Keyword||copies(' ',Indent-length(Keyword))||NextEntry||Global.CRLF  /* add the keyword and entry to the new header */
 end
 else  /* if it is not a real keyword, it must be already marked as an error */
 do
  NewHeader = NewHeader||HeadLine||Global.CRLF  /* add the original line to the new header */
 end

end

return NewHeader  /* end of MakeHeaderLines */

/**********************************************************************/
MakeReplyMessage: procedure expose Global.  /* generate a reply message */
/**********************************************************************/

parse arg MessFile,Switch  /* get the arguments */

AttDir = UnpackMessage(MessFile,AttDir)  /* unpack the mail file in the folder and get the attachments folder name in return */

if (AttDir = '') then  /* if something is wrong */
do
 call AddError 'Cannot unpack "'||MessFile||'"'  /* report */
 return 0  /* and quit */
end

ReadFile = AttDir||'.READ'  /* the reader file name */

MessFileDir = filespec('D',MessFile)||filespec('P',MessFile)  /* get the path to the mail message */

if (pos(translate(Global.MainDir),translate(MessFileDir)) > 0) then  /* if it is anywhere in the RexxMail (sub)folders */
do
 ReplyDir = Global.OutDir||'\'  /* create the reply in the 'Out' folder */
end
else  /* if the original message is not in any of the RexxMail folders */
do
 ReplyDir = MessFileDir  /* create the reply file in the same folder as the original */
end

if (\OpenFile(ReadFile,'READ')) then  /* if we cannot open the message read file for reading */
do
 return 0  /* and return with an error */
end

ReadText = charin(ReadFile,1,chars(ReadFile))  /* copy the contents of the file into a buffer */
call stream ReadFile,'C','CLOSE'  /* close the read file */

parse var ReadText . (Global.EmptyLine) BodyText  /* get the message body text */

ReplyFile = systempfilename(ReplyDir||'RXML????.REPL')  /* create a unique temporary file name */

if (\OpenFile(ReplyFile,'WRITE')) then  /* if we cannot open the reply file for writing */
do
 return 0  /* and return with an error */
end

Sender = GetHeaderEntry(Global.MessHead,'FROM:')  /* get the sender; we need it anyway to quote in the message */
if (Sender = '') then  /* if we have nothing */
do
 Sender = GetHeaderEntry(Global.MessHead,'SENDER:')  /* look for the real sender */
 if (Sender = '') then  /* if we have nothing */
 do
  Sender = GetHeaderEntry(Global.MessHead,'APPARENTLY-FROM:')  /* look for the apparent sender */
  if (Sender = '') then  /* if we have nothing */
  do
   Sender = GetHeaderEntry(Global.MessHead,'FROM')  /* look for the envelope sender */
   if (Sender >< '') then  /* if we have something this time */
   do
    Sender = word(Sender,1)  /* use just the first word */
   end
  end
 end
end

select  /* do one of the following */

 when (Switch = 'REPLYTOALL') then  /* if we are replying to everybody */
 do
  SendTo = Sender  /* start with the sender's address */
  ReplyToAddress = GetHeaderEntry(Global.MessHead,'REPLY-TO:')  /* look for "Reply-To:" addresses */
  if (ReplyToAddress >< '') then  /* if we have something */
  do
   SendTo = SendTo||', '||ReplyToAddress  /* add it to the recipients */
  end
  ToAddress = GetHeaderEntry(Global.MessHead,'TO:')  /* look for "To:" addresses */
  if (ToAddress >< '') then  /* if we have something */
  do
   SendTo = SendTo||', '||ToAddress  /* add it to the recipient list */
  end
  CcAddress = GetHeaderEntry(Global.MessHead,'CC:')  /* look for "Cc:" addresses */
  if (CcAddress >< '') then  /* if we have something */
  do
   SendTo = SendTo||', '||CcAddress  /* add it to the recipient list */
  end
 end

 when (Switch = 'REPLYTOSENDER') then  /* if we are replying to the "From:" address */
 do
  SendTo = Sender  /* use the sender's address */
 end

 when (Switch = 'REPLYTORECIPIENTS') then  /* if we are replying to the "To:" addresses */
 do
  SendTo = GetHeaderEntry(Global.MessHead,'TO:')  /* get the "To:" addresses */
  if (SendTo = '') then  /* if there is no address */
  do
   SendTo = GetHeaderEntry(Global.MessHead,'APPARENTLY-TO:')  /* look for an "Apparently-To:" address */
  end
 end

 otherwise  /* if we have none of the above special cases */
 do
  SendTo = GetHeaderEntry(Global.MessHead,'REPLY-TO:')  /* try to find a "Reply-To:" address */
  if (SendTo = '') then  /* if there is none */
  do
   SendTo = Sender  /* use the sender address */
  end
 end

end

if (SendTo = '') then  /* if we have no address */
do
 call AddError 'No address found'  /* report */
 return 0  /* and return with an error */
end

OutText = 'From: '||MakeFromLine()||Global.CRLF  /* Start the output text with a "From: " line containing the default sender, if one is available */

SendTo = DecodeHeaderLine(SendTo)  /* decode any MIME stuff in the address line */

OutText = OutText||'To: '||SendTo||Global.CRLF  /* add the address line to the output text */

MessageID = GetHeaderEntry(Global.MessHead,'Message-ID:')  /* try to get a message ID */
if (MessageID <> '') then  /* if there is a message ID */
do
 OutText = OutText||'In-Reply-To: '||MessageID||Global.CRLF  /* add the reply info line */
end

Subject = GetHeaderEntry(Global.MessHead,'SUBJECT:')  /* the original subject */
if (Subject = '') then  /* if we have no subject */
do
 Subject = '(no subject)'  /* substitute this */
end
else  /* if we have a subject */
do
 if (translate(word(Subject,1)) >< 'RE:') then  /* if it does not start with "Re:" */
 do
  Subject = 'Re: '||Subject  /* add it to the header */
 end
end
Subject = DecodeHeaderLine(Subject)  /* decode any MIME stuff in the subject line */
Subject = WordWrap(Subject,78,9,1)  /* word-wrap the subject line, 78 characters max., 9 characters indent, no word breaks */
OutText = OutText||'Subject: '||Subject||Global.EmptyLine  /* add the subject line to the output text followed by an empty line */

if (Sender >< '') then  /* if we have a sender */
do
 Sender = FormatAddresses(sender,0,1,0)  /* reformat the address, no indent, check, no end comments */
 Sender = DecodeHeaderLine(Sender)  /* decode any MIME stuff in the sender line */
end
else  /* if we still have nothing */
do
 Sender = 'an unknown sender'  /* use this */
end

OutText = OutText||Global.EmptyLine||'In reply to a message from '||Sender  /* add an empty line and an info line to the message */

MessDate = GetHeaderEntry(Global.MessHead,'DATE:')  /* try to get a sent date */

if (MessDate >< '') then  /* if we have a date */
do
 OutText = OutText||' dated '||SortedDate(MessDate)  /* rewrite the date and stick some text in front */
end

OutText = OutText||':'||Global.EmptyLine  /* add a colon and an empty line */

do while (BodyText >< '')  /* while we have something in the body text */
 parse var BodyText OutLine (Global.CRLF) BodyText  /* take each line */
 OutText = OutText||'> '||OutLine||Global.CRLF  /* and add it to the output text, preceded by a quote sign */
end

CloseFile = Global.ConfDir||'\closing.txt'  /* the name of the closing text file */
if (OpenFile(CloseFile,'READ')) then  /* if we can open the file, i.e. it exists and is ready */
do
 Closing = charin(CloseFile,1,chars(CloseFile))  /* copy the contents of the closing phrase file into a buffer */
 call stream CloseFile,'C','CLOSE'  /* close the closing phrase file */
 OutText = OutText||Global.CRLF||Closing  /* add the contents of the closing phrase file after an empty line */
end

call charout ReplyFile,OutText  /* write the output text to the reply message file */
call stream ReplyFile,'C','CLOSE'  /* close the reply file */

call MessageSettings ReplyFile,'--------','CHANGE'  /* make it an outgoing mail message file */

call MessageContents ReplyFile  /* get the contents of the new file */

NewName = MakeTitle(ReplyFile,Global.MessHead)  /* get the new name */

if (\syssetobjectdata(ReplyFile,'TITLE='||NewName)) then  /* if we cannot rename the file */
do
 call AddError 'Cannot rewrite title of "'||ReplyFile||'"'  /* report; not fatal */
end

call DeleteDir AttDir  /* get rid of the attachments folder if it is still there */

if (stream(ReadFile,'C','QUERY EXISTS') >< '') then  /* if the read file still exists */
do
 call sysfiledelete ReadFile  /* get rid of the read file -- no check, it may already have been deleted by another process */
end

return 1  /* end of MakeReplyMessage */

/**********************************************************************/
MakeSendFile: procedure expose Global.  /* prepares the actual file to send */
/**********************************************************************/

parse arg MessFile,DateString  /* get the arguments */

ASCIIMessage = MessageSettings(MessFile,'-**+****','MATCH')  /* if we find an ASCII EA, this message should be sent as ASCII */

AttDir = SetAttachments(MessFile,'GETDIR')  /* extract the attachments directory name if there is one */

SendFile = systempfilename(Global.TempDir||'\RXML????.SEND')  /* prepare a unique file name */

if (\OpenFile(MessFile)) then  /* if we cannot open the message file */
do
 return ''  /* return with no result */
end

MessCont = charin(MessFile,1,chars(MessFile))  /* read the contents of the message file into MessCont */
call stream MessFile,'C','CLOSE'  /* close the message file */

parse var MessCont MessHead (Global.EmptyLine) MessBody  /* split the message content into a header and a body */

MessHead = translate(MessHead,d2c(32),d2c(9))  /* convert header TABs to spaces */
MessHead = translate(MessHead,Global.ESCtoASC,xrange('00'x,'1F'x))  /* convert header escape codes to ASCII */
MessBody = translate(MessBody,Global.ESCtoASC,xrange('00'x,'1F'x))  /* convert body text escape codes to ASCII */

if (ASCIIMessage) then  /* if we want ASCII text */
do
 MessHead = translate(MessHead,Global.850toASC,xrange('80'x,'FF'x))  /* convert the header to ASCII*/
 MessBody = translate(MessBody,Global.850toASC,xrange('80'x,'FF'x))  /* convert the body text to ASCII */
 MessBody = translate(MessBody,d2c(32),d2c(9))  /* convert body TABs to spaces */
end
else  /* if we want quoted-printable ISO 8859-1 text */
do
 MessHead = translate(MessHead,Global.850toISO,xrange('80'x,'FF'x))  /* convert the header to ISO 8859-1 */
 MessBody = translate(MessBody,Global.850toISO,xrange('80'x,'FF'x))  /* convert the body text to ISO 8859-1 */
end

NewHead = ''  /* start with nothing */

do until (MessHead = '')  /* go on until we run out of header */

 MIMEPrevWord = 0  /* there is no previous word, so it cannot have been a MIME word */

 parse var MessHead NextLine (Global.CRLF) MessHead  /* get a line from the header */

 if (substr(NextLine,1,1) = d2c(32)) then  /* if the line starts with a space */
 do
  Indent = wordindex(NextLine,1)-1  /* the indent goes up to where the first word starts, minus one space because the next word also gets a space in front of it */
  parse var NextLine NewLine =(Indent) NextLine  /* start the new line with the indent */
 end
 else  /* if the line does not start with a space */
 do
  parse var NextLine NewLine NextLine  /* start the new line with the first word, which must be a keyword */
 end

 do while (NextLine >< '')  /* go on until we run out of line */

  parse var NextLine NextWord NextLine  /* get the next word in the line */

  if (verify(NextWord,xrange(x2c(00),x2c(7F)),'NOMATCH') > 0) then  /* if the current word contains any high-bit characters */

  do

   NewWord = ''  /* start with nothing */
   ISOStart = '=?iso-8859-1?Q?'  /* define the start of an ISO string */
   ISOEnd = '?='  /* define the end of an ISO string */

   if (substr(NextWord,1,1) = '"') then  /* if the word starts with a quotation mark */
   do
    NextWord = substr(NextWord,2)  /* cut it off */
    ISOStart = '"'||ISOStart  /* add the quotation mark to the start of the ISO start code */
   end

   if (substr(NextWord,length(NextWord),1) = '"') then  /* if the word ends with a quotation mark */
   do
    NextWord = substr(NextWord,1,length(NextWord)-1)  /* cut it off */
    ISOEnd = ISOEnd||'"'  /* add the quotation mark to the end of the ISO end code */
   end

   do Index = 1 to length(NextWord)  /* run through the message text */
    NextChar = substr(NextWord,Index,1)  /* take each character */
    if ((c2d(NextChar) > 126) | (pos(NextChar,'_=') > 0)) then  /* if it's not in the OK range, or if we are in a MIME word and it is an underscore or equal sign */
    do
     NewWord = NewWord||'='||c2x(NextChar)  /* substitute an equals sign followed by the character's hex code */
    end
    else  /* if the character is in the OK range */
    do
     NewWord = NewWord||NextChar  /* simply add it to the output buffer */
    end
   end

   if (MIMEPrevWord) then  /* if the previous word was also a MIME word */
   do
    NewWord = '_'||NewWord  /* stick on an underscore */
   end
   else  /* if the previous word was not a MIME word */
   do
    NewWord = ' '||ISOStart||NewWord  /* stick on a space and an ISO start string */
    MIMEPrevWord = 1  /* set the MIME flag */
   end

  end

  else  /* if the word does not contain any high-bit characters */

  do
   if (MIMEPrevWord) then  /* if the previous word was a MIME word */
   do
    NewWord = ISOEnd||' '||NextWord  /* stick on an ISO end string and a space */
    MIMEPrevWord = 0  /* reset the MIME flag */
   end
   else  /* if the previous word was not a MIME word */
   do
    NewWord = ' '||NextWord  /* just stick on a space */
   end
  end

  NewLine = NewLine||NewWord  /* add it to the rest of the line */

 end

 NewHead = NewHead||NewLine||Global.CRLF  /* add the new line to the header */

end

NewHead = 'Date: '||DateString||Global.CRLF||NewHead  /* add the date to the message header */
NewHead = NewHead||'X-mailer: RexxMail - build '||Global.Build||' - (c) 2000-2001 Marcus C. de Geus'||Global.CRLF  /* add an "X-mailer" line to the header */

GotAttach = 0  /* no attachments yet */
if (AttDir >< '') then  /* if there is an attachment storage directory */
do
 call sysfiletree AttDir||'\*.*','Attach.','FOS'  /* look for attachment files in the attachments directory and any subfolders */
 if (Attach.0 > 0) then  /* if we found any */
 do
  GotAttach = 1  /* we've got MIME attachments */
  MIMETypeFile = Global.ConfDir||'\mimetype.txt'  /* the list of MIME types */
  if (Global.RunAttachOut >< '') then  /* if we are to run a command in the attachments folder */
  do
   CurDir = directory()  /* store the current directory name */
   if (directory(AttDir) >< '') then  /* if we can change to the attachments directory */
   do
    parse var Global.RunAttachOut FirstBit '%N' LastBit  /* look for the filename macro in the RunAttachOut command line */
    CommLine = FirstBit  /* this is the command to start with */
    if (pos('%N',Global.RunAttachOut) > 0) then  /* if the settings line contains a file spec macro */
    do
     CommLine = CommLine||date('D')||time('S')||LastBit  /* this is the complete command */
    end
    say 'RunAttachOut configuration option parsed as : '||CommLine  /* report */
    address cmd CommLine  /* run the command */
    call sysfiletree AttDir||'\*.*','Attach.','FOS'  /* look again for remaining attachment files in the attachments directory and any subdirectories */
    if (directory(CurDir) = '') then  /* if we cannot change back to the original directory */
    do
     call AddError 'Cannot change directory to: "'||CurDir||'"'  /* report */
     return ''  /* and quit */
    end
   end
   else  /* if we cannot change to the attachments directory */
   do
    call AddError 'Cannot change directory to: "'||AttDir||'"'  /* report */
    return ''  /* and quit empty-handed */
   end
  end
 end
end

if ((GotAttach) | (\ASCIIMessage)) then  /* if we've got MIME attachments, or we want quoted-printable text */
do
 NewHead = NewHead||'Mime-version: 1.0'||Global.CRLF  /* add the MIME version header line */
end

if (GotAttach) then  /* if we've got MIME attachments */
do
 MIMEBound = '=='||date('S')||'=='||time('L')||'=='  /* the MIME boundary to use (any unique string will do) */
 if (Attach.0 > 1) then  /* if we have more than one attachment */
 do
  Plural = 's'  /* we need a plural "s" */
 end
 else  /*if there is only one attachment */
 do
  Plural = ''  /* we don't need the "s" */
 end
 NewHead = NewHead||'Content-type: multipart/mixed; boundary="'||MIMEBound||'"'||Global.CRLF  /* add the MIME boundary header line */
 NewHead = NewHead||Global.CRLF  /* add an empty line */
 NewHead = NewHead||'This is a multi-part message in MIME format.'||Global.CRLF  /* add a plain text message line */
 NewHead = NewHead||'In addition to the message below it contains '||,  /* add an info line */
           Attach.0||' b64-encoded attachment'||Plural||'.'||Global.CRLF  /* rest of the info line */
 NewHead = NewHead||Global.CRLF||'--'||MIMEBound||Global.CRLF  /* add an empty line followed by a MIME boundary */
end

if (ASCIIMessage) then  /* if we want ASCII text */
do
 if (GotAttach) then  /* if this is a MIME-enhanced message */
 do
  NewHead = NewHead||'Content-Type: text/plain; charset="us-ascii"'||Global.CRLF  /* the first type line */
  NewHead = NewHead||'Content-Transfer-Encoding: 7bit'||Global.CRLF  /* another one */
 end
end
else  /* if we want quoted-printable text */
do
 NewHead = NewHead||'Content-Type: text/plain; charset="iso-8859-1"'||Global.CRLF  /* the first type line */
 NewHead = NewHead||'Content-Transfer-Encoding: quoted-printable'||Global.CRLF  /* another one */
end

NewHead = NewHead||Global.CRLF  /* add a separating empty line to the header */

NewBody = ''  /* start with nothing */

do while (length(MessBody) > 0)  /* go through the contents of the body text */
 parse var MessBody NextLine (Global.CRLF) MessBody  /* and each time, take the next line from the body text */
 if (ASCIIMessage) then  /* if we want ASCII text */
 do
  if (Global.ASCIILineLength > 998) then  /* if the user set a line length that exceeds the limit of RFC 2822 */
  do
   Global.ASCIILineLength = 998  /* reduce the length */
  end
  NewBody = NewBody||WordWrap(NextLine,Global.ASCIILineLength)||Global.CRLF  /* add the word-wrapped line to the body text */
 end
 else  /* if we don't want ASCII text, we get quoted-printable */
 do
  OutLine = ''  /* start with an empty line */
  do while (length(NextLine) > 0)  /* as long as we have characters left in the line */
   parse var NextLine NextChar 2 NextLine  /* get the next character from the line */
   if ((c2d(NextChar) > 126) | (NextChar = '=')) then  /* if it's not in the OK range, or if it is an equal sign */
   do
    NextChar = '='||c2x(NextChar)  /* substitute an equals sign followed by the character's hex code */
   end
   if ((length(OutLine||NextChar) > 76) | ((length(OutLine||NextChar) = 76) & (length(NextLine) > 0))) then  /* if we end up with more than 76 characters (the maximum according to the RFC spec) */
   do
    NewBody = NewBody||OutLine||'='||Global.CRLF  /* add the line to the body text, end the line with = and start a new line */
    OutLine = ''  /* we now have an empty line */
   end
   OutLine = OutLine||NextChar  /* add the new character to the new line */
  end
  NewBody = NewBody||OutLine||Global.CRLF  /* add to the body, with a CRLF */
 end
end

if (\OpenFile(SendFile,'WRITE')) then  /* if we cannot open the temporary file for writing */
do
 return ''  /* no result */
end

call charout SendFile,NewHead  /* write the new header to the sendable file */
call charout SendFile,NewBody  /* write the new message body to the sendable file */

Success = 1  /* assume success */

if (GotAttach) then  /* if we've got one or more MIME attachments */
do
 Encoder = Global.ProgDir||'\mmencode.exe'  /* the encoder file spec */
 if (stream(Encoder,'C','QUERY EXISTS') = '') then  /* if we can't find the encoder */
 do
  call AddError 'Cannot find encoder: "'||Encoder||'"'  /* report */
  Success = 0  /* no success */
 end
 else  /* if we find the decoder */
 do Index = 1 to Attach.0  /* take each of the attachment files */
  AttachName = filespec('N',Attach.Index)  /* get the name of the attachment file */
  DotPos = lastpos('.',AttachName)  /* look for the last dos position in the file name */
  if (DotPos > 0) then  /* if we have a last dot */
  do
   AttachExt = substr(AttachName,DotPos+1)  /* get the extension */
  end
  MIMEType = GetFileEntry(MIMETypeFile,AttachExt)  /* try to match the extension with a MIME type */
  if (MIMEType >< '') then  /* if we can match it */
  do
   parse var MIMEType MIMEType '|' .  /* use the first one if there are several (which should not be the case) */
  end
  else  /* if we cannot match it (or if there is no MIME types list) */
  do
   MIMEType = 'application/octet-stream'  /* use this */
  end
  MIMEStart = Global.CRLF||Global.CRLF||'--'||MIMEBound  /* start with an empty line followed by a MIME boundary */
  MIMEStart = MIMEStart||Global.CRLF||'Content-Type: '||MIMEType||'; name="'||AttachName||'"'  /* add a content line */
  MIMEStart = MIMEStart||Global.CRLF||'Content-Transfer-Encoding: base64'  /* add another one */
  MIMEStart = MIMEStart||Global.CRLF||'Content-Disposition: attachment; filename="'||AttachName||'"'  /* and another one */
  MIMEStart = MIMEStart||Global.CRLF||Global.CRLF  /* add an empty line */
  call charout SendFile,MIMEStart  /* write it to the temporary file */
  call stream SendFile,'C','CLOSE'  /* close the new message file to allow MMENCODE.EXE access */
  Encoder||' < "'||Attach.Index||'" >> "'||SendFile||'"'  /* convert SourceFile to B64 code and add the result to SendFile [EXTERNAL] */
  if (RC >< 0) then  /* if we get an error */
  do
   Success = 0  /* we have no success overall */
  end
  call OpenFile SendFile,'WRITE'  /* reopen the new message file for more writing */
 end
 call charout SendFile,Global.CRLF||Global.CRLF||'--'||MIMEBound||'--'  /* write an empty line followed by a MIME end boundary */
end

call stream SendFile,'C','CLOSE'  /* close the new message file */

if (\Success) then  /* if something went wrong */
do
 call sysfiledelete SendFile  /* get rid of the temp file -- no check, it may already have been deleted by another process */
 SendFile = ''  /* return nothing */
end

return SendFile  /* end of MakeSendFile */

/**********************************************************************/
MakeTitle: procedure expose Global.  /* returns new title for mail file */
/**********************************************************************/

parse arg MessFile,MessHead  /* get the parameter */

TitleWords = Global.ObjectTitle  /* copy the contents of the global object title words string */

OutGoing = (MessageSettings(MessFile,'-*******','MATCH'))  /* set a message type flag for outgoing */

if (OutGoing) then  /* if the message is outgoing */
do
 EmptyStart = Global.Warning||' '  /* start of an empty line */
 EmptyEnd = ''  /* end of an empty line */
end
else  /* if the message is incoming */
do
 EmptyStart = '('  /* start of an empty line */
 EmptyEnd = ')'  /* end of an empty line */
end

TitleCount = 0  /* start at 0 */
Title. = ''  /* start with nothing */

do while (TitleWords >< '')  /* as long as we have object title words left */
 TitleCount = TitleCount + 1  /* up the title lines counter */
 parse upper var TitleWords NextWord TitleWords  /* get the next word in upper case */
 select  /* do one of the following */
  when (NextWord = 'DATE') then  /* if the next word is 'DATE' */
  do
   Title.TitleCount = GetTitleDate(MessHead,MessFile)  /* get the date */
  end
  when (NextWord = 'SENDER') then  /* if the next word is 'SENDER' */
  do
   Title.TitleCount = GetTitleSender(MessHead,EmptyStart,EmptyEnd,Outgoing)  /* get the sender */
  end
  when (NextWord = 'RECIPIENTS') then  /* if the next word is 'RECIPIENTS' */
  do
   Title.TitleCount = GetTitleRecipients(MessHead,EmptyStart,EmptyEnd,Outgoing)  /* get the recipients */
  end
  when (NextWord = 'SUBJECT') then  /* if the next word is 'SUBJECT' */
  do
   Title.TitleCount = GetTitleSubject(MessHead,EmptyStart,EmptyEnd)  /* get the subject */
  end
  otherwise  /* if it is not one of the above */
  do
   call AddError 'Illegal keyword in ObjectTitle setting: '||NextWord  /* report a non-fatal error */
   TitleCount = TitleCount - 1  /* reduce the counter back to its previous value */
  end
 end
end

if (pos(Global.CRLF||Global.Warning,Global.CRLF||MessHead) > 0) then  /* if the header contains a line that starts with a marker */
do
 TitleCount = TitleCount + 1  /* up the title lines counter */
 Title.TitleCount = EmptyStart||'illegal keyword'||EmptyEnd  /* add a warning */
end

if (Global.MessBody = '') then  /* if we have no message body */
do
 TitleCount = TitleCount + 1  /* up the title lines counter */
 Title.TitleCount = EmptyStart||'no message body'||EmptyEnd  /* add a warning */
end

do Index = 2 to TitleCount  /* for each line in the title, except the first one */
 Title.Index = d2c(10)||Title.Index  /* add a leading new line */
end

NewTitle. = ''  /* start with nothing */
do Index = 1 to TitleCount  /* for each line in the title */
 NewTitle.Index = Title.Index  /* copy the contents */
end

FullTitle = ''  /* start with nothing */
MaxLineLength = 100  /* the maximum no. of characters in a file name line */
do Index = 1 to TitleCount  /* for each line in the title */
 if (length(NewTitle.Index) > MaxLineLength) then  /* if this part exceeds the maximum line length */
 do
  NewTitle.Index = substr(NewTitle.Index,1,MaxLineLength)  /* reduce this one to the maximum line length */
 end
 FullTitle = FullTitle||NewTitle.Index  /* add it to the full title */
end

MaxTitleLength = 200 - length(filespec('D',MessFile)||filespec('P',MessFile))  /* the maximum no. of characters in the title */
if (length(FullTitle) > MaxTitleLength) then  /* if the resulting title is too long */
do
 CutLength = MaxTitleLength  /* start by reducing all the strings to the maximum allowable total length */
 do while (length(FullTitle) > MaxTitleLength)  /* as long as the full title is too long */
  do Index = 1 to TitleCount  /* for each line in the title */
   if (length(NewTitle.Index) > CutLength) then  /* if this part exceeds the cutting length */
   do
    NewTitle.Index = substr(NewTitle.Index,1,CutLength)  /* reduce this one to the cutting length */
   end
  end
  FullTitle = ''  /* start with nothing once more */
  do Index = 1 to TitleCount  /* for each line in the title */
   FullTitle = FullTitle||NewTitle.Index  /* add it to the full title */
  end
  CutLength = CutLength - 1  /* reduce the cutting length by one */
 end
end

FullTitle = ''  /* start with nothing once more */
do Index = 1 to TitleCount  /* for each line in the title */
 NewTitle.Index = strip(NewTitle.Index,'T',' ')  /* get rid of trailing blanks */
 if (NewTitle.Index >< Title.Index) then  /* if the contents are not what we originally had */
 do
  NewTitle.Index = NewTitle.Index||'...'  /* add an ellipsis */
 end
 FullTitle = FullTitle||NewTitle.Index  /* add it to the full title */
end

FullTitle = translate(FullTitle,'()()_________..'||"''",'<>[]~^?%&*\|/;,"'||"`")  /* remove difficult characters from the title string */

FullTitle = FullTitle||d2c(255)  /* add a hard space to prevent problems with file filter associations */

return FullTitle  /* end of MakeTitle */

/**********************************************************************/
MessageContents: procedure expose Global.  /* gets data from mail file */
/**********************************************************************/

parse arg MessFile  /* get the arguments */

Global.MessHead = ''  /* reset the message header content var */
Global.MessBody = ''  /* reset the message body content var */

Success = 0  /* no success yet */

if (OpenFile(MessFile,'READ')) then  /* if we can open the message file */
do
 Global.MessBody = charin(MessFile,1,chars(MessFile))  /* copy the contents */
 call stream MessFile,'C','CLOSE'  /* and close the file */
 if (Global.MessBody >< '') then  /* if we have message content */
 do
  if (substr(Global.MessBody,1,2) = Global.CRLF) then  /* if it starts with a CRLF */
  do
   Header = ''  /* we have no header */
   Global.MessHead = ''  /* we have no header content */
  end
  else  /* if it starts with text */
  do
   parse var Global.MessBody Header (Global.EmptyLine) Global.MessBody  /* split it into a header and a body */
  end
  do while(substr(Global.MessBody,1,2) = Global.CRLF)  /* as long as the body starts with a CRLF */
   Global.MessBody = substr(Global.MessBody,3)  /* get rid of the CRLF */
  end
  if (Header >< '') then  /* if we have a header */
  do
   do while(substr(Header,length(Header)-1,2) = Global.CRLF)  /* as long as the header ends with a CRLF */
    Header = substr(Header,1,length(Header)-2)  /* get rid of the CRLF */
   end
   Header = translate(Header,' ',D2C(9))  /* convert any TABs to SPACEs */
   FoldPoint = Global.CRLF||' '  /* define a folding point */
   do until (Header = '')  /* go on util we have no header left */
    parse var Header Line (FoldPoint) Header  /* get a line */
    Global.MessHead = Global.MessHead||Line  /* add the line to the new header */
    if (Header >< '') then  /* if we have any header content left */
    do
     Header = strip(Header,'L',' ')  /* remove excess leading spaces */
     Global.MessHead = Global.MessHead||' '  /* add a single space to the new header */
    end
   end
  end
 end
 Success = 1  /* all should be well */
end

return Success  /* end of MessageContents */

/**********************************************************************/
MessageDate: procedure expose Global.  /* prepares message file parts and date string */
/**********************************************************************/

parse arg NoZone  /* get the argument */

NoZone = (NoZone = 1)  /* if we have this, we want no time zone (i.e. no time server contacting) */

NTime = time('N')  /* get the normal time */
NDate = date('N')  /* get the normal date  */

Wday = substr(date('W'),1,3)  /* use only the first three letters to comply with RFC 822 */

if ((Global.TimeZone >< '') | (NoZone)) then  /* if we have a time zone setting, or we may not contact a time server */
do
 TimeZone = Global.TimeZone  /* use the Global.TimeZone setting, if any */
end
else  /* if we do not have a time zone setting, and we are allowed to contact a time server */
do
 TimeZone = GetTimeZone()  /* get the time zone */
end

if (TimeZone = '') then  /* if we still have no time zone */
do
 TimeZone = '-0000'  /* use this instead */
end

DateString = Wday||', '||NDate||' '||NTime||' '||TimeZone  /* prepare the message date string */

return DateString  /* end of MessageDate */

/**********************************************************************/
MessageSettings: procedure expose Global.  /* handles EA settings */
/**********************************************************************/

/**********************************************************************/
/* Flags:                                                             */
/*   Position 0 :                                                     */
/*     UNSET = outgoing message                                       */
/*     SET   = incoming message                                       */
/*   Position 1 :                                                     */
/*     UNSET = not O.K. for dispatch                                  */
/*     SET   = O.K. for dispatch                                      */
/*   Position 2 :                                                     */
/*     UNSET = not processed                                          */
/*     SET   = processed                                              */
/*   Position 3 :                                                     */
/*     UNSET = non-ASCII (i.e. quoted-printable text)                 */
/*     SET   = ASCII (i.e. 7-bit, word-wrapped, no TABs)              */
/**********************************************************************/
/* Switch options:                                                    */
/*   CHANGE  : Changes the file settings to the settings provided,    */
/*             with "+" indicating "set", "-" indicating "reset", and */
/*             "*" indicating "leave unchanged".                      */
/*   MATCH   : Looks for a match between the file settings and the    */
/*             settings provided. A "*" character serves as a "don't  */
/*             care" placeholder.                                     */
/**********************************************************************/

parse upper arg MessFile,Setting,Action  /* get the arguments */

call sysgetea MessFile,'.VERSION','Version'  /* look for version */

if (Version >< '') then  /* if we found a version */
do
 parse var Version 5 CurSett  /* get the bits we want */
end
else  /* if no version was found */
do
 CurSett = '-'  /* start with '-' */
end

CurSett = left(CurSett,8,'-')  /* pad to 8 characters with "-" (i.e. "not set") */

if (Setting = '') then  /* if we received no action argument */
do
 return CurSett  /* return the current value */
end

Setting = left(Setting,8,'*')  /* pad to 8 characters with asterisks */

if (verify('Setting','*+-','M') = 0) then  /* if Setting contains only "*", "+', or "-" */
do

 do Index = 1 to 8  /* take each of the characters */
  if (substr(Setting,Index,1) = '*') then  /* if the new setting is '*' */
  do
   NewSett.Index = substr(CurSett,Index,1)  /* this position of NewSett. copies the current setting */
  end
  else  /* if the new setting is not "*" (but "+" or "-") */
  do
   NewSett.Index = substr(Setting,Index,1)  /* this position of NewSett. copies the new setting */
  end
 end
 ChgSett = ''  /* start with nothing */
 do Index = 1 to 8  /* take each of the members of NewSett. */
  ChgSett = ChgSett||NewSett.Index  /* and add it to ChgSett */
 end

 select  /* selects the action to take */

  when (Action = 'CHANGE') then  /* if we're asked to change the settings */
  do
   Success = 0  /* no success yet */
   if (NewSett.1 = '-') then  /* if the first position, i.e. the "Received" bit is not set */
   do
    TypeText = 'Mail Message Out'  /* make it an outgoing mail message file type */
   end
   else  /* if the "Received" bit is set */
   do
    TypeText = 'Mail Message In'  /* make it an incoming mail message file type */
   end
   EAT_MVMT = reverse('FFDF'x)  /* prepare extended attributes */
   CodePage = '0000'x  /* set code page */
   Count = reverse('0002'x)  /* set Count */
   EAT_ASCII = reverse('FFFD'x)  /* set EAT_ASCII */
   Len = reverse(right(d2c(Length(TypeText)),2,'00'x))  /* set Len */
   Type = EAT_MVMT||CodePage||Count||EAT_ASCII||Len||TypeText  /* set the message type */
   if (sysputea(MessFile,'.TYPE',Type) = 0) then  /* if we attached them to the file */
   do
    Len = reverse(right(d2c(Length(ChgSett)),2,'00'x))  /* set Len */
    Version = EAT_ASCII||Len||ChgSett  /* set version */
    if (sysputea(MessFile,'.VERSION',Version) = 0) then  /* if we attached them to the file */
    do
     ChgSett = translate(ChgSett,'10','+-','0')  /* do it and pad with zeroes */
     IconFile = Global.PaperDir||'\'||substr(ChgSett,1,4)||'0000.ico'  /* get full icon file name */
     Success = (syssetobjectdata(MessFile,'TEMPLATE=NO;ICONFILE='||IconFile))  /* if we can set the icon, we're O.K. (and set template to off) */
    end
   end
   ChgSett = Success  /* return a success value */
  end

  when (Action = 'MATCH') then  /* if the action is to check for a match */
  do
   ChgSett = (ChgSett = CurSett)  /* O.K.  if they match */
  end

  otherwise  /* if we have an unexpected action parameter */
  do
   ChgSett = 0  /* return an error status */
  end

 end

end

return ChgSett  /* end of MessageSettings */

/**********************************************************************/
MoveMessage: procedure expose Global.  /* moves mail message files to subfolders */
/**********************************************************************/

parse arg MessFile,MessHead  /* get the parameters */

Incoming = (MessageSettings(MessFile,'+*******','MATCH'))  /* is it an incoming message? */

if (InComing) then  /* if it is incoming */
do
 CompPath = Global.InDir||'\'  /* compare with the "In" folder */
 MoveFile = Global.ConfDir||'\mailread.txt'  /* the file with the "Read" folders */
end
else  /* if it is outgoing */
do
 CompPath = Global.OutDir||'\'  /* compare with the "Out" folder */
 MoveFile = Global.ConfDir||'\mailsent.txt'  /* the file with the "Sent" folders */
end

FilePath = filespec('D',MessFile)||filespec('P',MessFile)  /* the mail file path name */

if ((translate(FilePath) = translate(CompPath)) & (stream(MoveFile,'C','QUERY EXISTS') >< '')) then  /* if they are the same, i.e. the file is in the "Comp" folder, and we have a "Move" file */
do

 MoveDir = ''  /* start with nothing */

 if (Incoming) then  /* if the message is incoming */
 do
  SearchStr = GetHeaderEntry(MessHead,'FROM:')  /* get the sender's address */
  if (SearchStr = '') then  /* if we did not find a matching move directory */
  do
   SearchStr = GetHeaderEntry(MessHead,'APPARENTLY-FROM:')  /* get the sender's address */
   if (SearchStr = '') then  /* if we did not find a matching move directory */
   do
    SearchStr = GetHeaderEntry(MessHead,'SENDER:')  /* get the sender's address */
   end
  end
  if (SearchStr >< '') then  /* if we have something */
  do
   SearchStr = FormatAddresses(SearchStr,,0,0)  /* reformat the address, no indent, no check, no comments */
   do while ((SearchStr >< '') & (MoveDir = ''))  /* as long as we have addresses left and still no move directory */
    parse var SearchStr NextWord SearchStr  /* get the next address from the list */
    MoveDir = GetMoveDir(MoveFile,NextWord)  /* try to find a matching move directory */
   end
  end
 end

 if (MoveDir = '') then  /* if we did not find a matching move directory, or if the message is outgoing */
 do
  SearchStr = GetHeaderEntry(MessHead,'TO:')  /* use the list of recipients' addresses */
  if (SearchStr >< '') then  /* if we have something */
  do
   SearchStr = FormatAddresses(SearchStr,,0,0)  /* reformat the address, no indent, no check, no comments */
   do while ((SearchStr >< '') & (MoveDir = ''))  /* as long as we have addresses left and still no move directory */
    parse var SearchStr NextWord SearchStr  /* get the next address from the list */
    MoveDir = GetMoveDir(MoveFile,NextWord)  /* try to find a matching move directory */
   end
  end
 end

 if (MoveDir = '') then  /* if we did not find a matching move directory */
 do
  SearchStr = GetHeaderEntry(MessHead,'SUBJECT:')  /* use the subject line */
  do while ((SearchStr >< '') & (MoveDir = ''))  /* as long as we have text left and still no move directory */
   parse var SearchStr NextString SearchStr  /* get the next word from the list */
   NextString = translate(NextString,'','`~!@#$%^&*()-_=+[]{}\|;:",<.>/?'||"'")  /* turn all these into spaces */
   do while ((NextString >< '') & (MoveDir = ''))  /* as long as we have something left and no move dir */
    parse var NextString NextWord NextString  /* get the next word from the string */
    MoveDir = GetMoveDir(MoveFile,NextWord)  /* try to find a matching move directory */
   end
  end
 end

 if (MoveDir = '') then  /* if we did not find a matching move directory */
 do
  SearchStr = '*DEFAULT*'  /* use the default entry */
  MoveDir = GetMoveDir(MoveFile,SearchStr)  /* try to find a matching move directory */
 end

 if (MoveDir >< '') then  /* if we found a directory to move the message file to */
 do
  MoveDir = MoveMessFile(MessFile,MessHead,MoveDir,Incoming)  /* see if can move the message file to the directory */
 end

end

return MoveDir  /* end of MoveMessage */

/**********************************************************************/
MoveMessFile: procedure expose Global.  /* move a message file to a different directory */
/**********************************************************************/

parse arg MessFile,MessHead,MoveDir,InComing  /* get the arguments */

if (translate(MoveDir) = '*DELETE*') then  /* if it is the special delete string */
do
 MoveDir = Global.TempDir  /* use the temp dir to move the file out of sight to be deleted the next time the mail folders are opened */
end
else  /*if it is not the special delete string */
do
 if (substr(MoveDir,1,1) = '*') then  /* if the dir name starts with this, we need to add the In or Out Archive dir */
 do
  MoveDir = strip(substr(MoveDir,2),'L','\')  /* use the rest and strip off any leading backslash */
  if (Incoming) then  /* if the message is incoming */
  do
   MoveDir = Global.InArchDir||'\'||MoveDir  /* use this destination */
  end
  else  /* if it is outgoing */
  do
   MoveDir = Global.OutArchDir||'\'||MoveDir  /* use this destination */
  end
 end
end

OrgTitle = MakeTitle(MessFile,MessHead)  /* get the title of the message */

MessID = '<'||date('B')||time('L')||'>'  /* create a unique object ID */
call syssetobjectdata MessFile,'OBJECTID='||MessID||';'  /* give the message file object the unique ID */

TempName = systempfilename(MoveDir||'\RXML????.MOVE')  /* get a unique temporary file name in the move dir (so we can move the file even if an identical title exists in the target dir) */
TempTitle = filespec('N',TempName)  /* get the name bit */
call syssetobjectdata MessID,'TITLE='||TempTitle||';'  /* change the message file object title */

if (sysmoveobject(MessID,MoveDir)) then  /* if we can move the message file to the directory */
do
 call syssetobjectdata MessID,'TITLE='||OrgTitle||';'  /* change the message file object title back to what it was before the move */
end
else  /* if we cannot move the file */
do
 call AddError 'Cannot move message to "'||MoveDir||'"'  /* report */
 MoveDir = ''  /* return nothing */
end

return MoveDir  /* end of MoveMessFile */

/**********************************************************************/
OpenFile: procedure expose Global.  /* attempts to open a file */
/**********************************************************************/

parse arg FileSpec,Operation  /* get the arguments */

Attempts = 0  /* we haven't tried yet */

do until ((Success) | (Attempts = 10))  /* go on until we succeed or reach the maximum no. of attempts */
 Success = (stream(FileSpec,'C','OPEN '||Operation) = 'READY:')  /* if we can open the file, all is well */
 if (\Success) then  /* if we did not succeed */
 do
  Attempts = Attempts + 1  /* up the attempts counter */
  if (Attempts = 10) then  /* if we've reached the last try */
  do
   call AddError 'Cannot open file : '||FileSpec  /* report */
  end
  else  /* if we have more tries left */
  do
   call syssleep 0.1  /* wait a bit */
  end
 end
end

return Success  /* end of OpenFile */

/**********************************************************************/
OpenFolders: procedure expose Global.  /* opens the RexxMail message folders */
/**********************************************************************/

AlreadyOpen = 0  /* assume the main folder was not already open */

if (sysqueryswitchlist('ListItems.') = 0) then  /* if we can get a switch list */
do Index = 1 to ListItems.0  /* take each of the items found  */
 if (translate(ListItems.Index) = filespec('N',Global.MainDir)) then  /* if we find the main dir */
 do
  AlreadyOpen = 1  /* the main dir is open (or minimized) on the desktop */
 end
end

if (\syssetobjectdata(Global.MainDir,'WORKAREA=NO;')) then  /* if we cannot make the main folder a 'no work area' folder */
do
 call AddError 'Cannot set RexxMail folder "'||Global.MainDir||'" to "no work area"'  /* report */
 return 0  /* and quit */
end

if (\sysopenobject(Global.MainDir,0,1)) then  /* if we cannot open the main dir */
do
 call AddError 'Cannot open RexxMail folder "'||Global.MainDir||'"'  /* report */
 return 0  /* and quit */
end

if (\syssetobjectdata(Global.MainDir,'WORKAREA=YES;')) then  /* if we cannot make the main folder a 'work area' folder */
do
 call AddError 'Cannot set RexxMail folder "'||Global.MainDir||'" to "work area"'  /* report */
 return 0  /* and quit */
end

do while (Global.OpenFolders >< '')  /* as long as we have folder values */
 parse upper var Global.OpenFolders NextValue Global.OpenFolders  /* get the next value in upper case */
 select  /* do one of the following */
  when (NextValue = 'ACCESSORIES') then  /* if the value is 'ACCESSORIES' */
  do
   OpenFolder = Global.AccessDir  /* do the Accessories folder */
  end
  when (NextValue = 'ADDRESSES') then  /* if the value is 'ADDRESSES' */
  do
   OpenFolder = Global.AddrDir  /* do the Addresses folder */
  end
  when (NextValue = 'CONFIGURATION') then  /* if the value is 'CONFIGURATION' */
  do
   OpenFolder = Global.ConfDir  /* do the Configuration folder */
  end
  when (NextValue  = 'IN') then  /* if the value is 'IN' */
  do
   OpenFolder = Global.InDir  /* do the In folder */
  end
  when (NextValue  = 'INARCHIVE') then  /* if the value is 'INARCHIVE' */
  do
   OpenFolder = Global.InArchDir  /* do the In Archivefolder */
  end
  when (NextValue  = 'OUT') then  /* if the value is 'OUT' */
  do
   OpenFolder = Global.OutDir  /* do the Out folder */
  end
  when (NextValue  = 'OUTARCHIVE') then  /* if the value is 'OUTARCHIVE' */
  do
   OpenFolder = Global.OutArchDir  /* do the Out Archive folder */
  end
  otherwise  /* if none of the above applies */
  do
   call AddError 'OpenFolders: invalid value : "'||NextValue||'"'  /* report */
   OpenFolder = ''  /* no value */
  end
 end
 if (OpenFolder >< '') then  /* if we have a value */
 do
  if (\sysopenobject(OpenFolder,0,1)) then  /* if we cannot open the folder on the desktop */
  do
   call AddError 'Cannot open folder "'||OpenFolder||'"'  /* report */
  end
 end
end

call MailWarning 1  /* destroy the mail warning program object, if any */

if (\AlreadyOpen) then  /* if the main dir was not already open */
do

 AllAttDirs = ''  /* start with nothing */

 call sysfiletree Global.Outdir||'\*.*','Files.','FO'  /* look for files in the 'Out' folder */
 do Index = 1 to Files.0  /* take each of the objects found */
  AllAttDirs = AllAttDirs||' '||SetAttachments(Files.Index,'GETDIR')  /* get the file's attachment folder name, if any, and add it to the list */
 end

 call sysfiletree Global.TempDir||'\RXML????','Dirs.','DO'  /* look for RexxMail attachment folders in the temporary storage directory */
 do Index = 1 to Dirs.0  /* take each of the objects found */
  if (wordpos(Dirs.Index,AllAttDirs) = 0) then  /* if its name is not in the current attachment folders list */
  do
   call DeleteDir Dirs.Index  /* get rid of the folder */
  end
 end

 call sysfiletree Global.TempDir||'\*.*','Files.','FO'  /* look for remaining files in the temporary storage directory */
 do Index = 1 to Files.0  /* take each of the objects found */
  if (substr(Files.Index,lastpos('.',Files.Index)+1) >< 'POP3') then  /* if it is not a POP3 file */
  do
   call sysfiledelete Files.Index  /* delete it */
  end
 end

end

return 1  /* end of OpenFolders */

/**********************************************************************/
OpenInfo: procedure expose Global.  /* opens the Info index file */
/**********************************************************************/

Success = sysopenobject(Global.InfoDir||'\index.html',0,1)  /* open the index file */

return Success  /* that's all, folks! */

/**********************************************************************/
POP3DeleteMess:  procedure expose Global.  /* delete a message from the POP3 server */
/**********************************************************************/

parse arg Socket,Index  /* get the arguments */

if (SocketSendLine(Socket,'DELE '||Index,'+OK')) then  /* if we can send this and get the right reply */
do
 Success = 1  /* all is well */
end
else  /* if the command was not accepted */
do
 Call AddError 'Cannot delete message '||Index||' from '||Global.POP3Server  /* report */
 Success = 0  /* all is not well */
end

return Success  /* end of POP3DeleteMess */

/**********************************************************************/
POP3RetrieveMess: procedure expose Global.  /* retrieve a message from the POP3 server */
/**********************************************************************/

parse arg Socket,Index  /* get the arguments */

if (\SocketSendLine(Socket,'RETR '||Index,'+OK')) then  /* if we cannot send this and get the right reply */
do
 call AddError 'Could not retrieve message '||Index||' from '||Global.POP3Server  /* report */
 return 0  /* return with no success */
end

MailFile = SysTempFileName(Global.TempDir||'\RXML????.POP3')  /* get a unique file name in the temp folder */

if (\OpenFile(MailFile,'WRITE')) then  /* if we cannot open the file for writing */
do
 call AddError 'Cannot open '||MailFile  /* report */
 return 0  /* return with no success */
end

NextLine = SocketGetLine(Socket)  /* get a line from the POP3 socket */

do while ((NextLine >< '.') & (NextLine >< d2c(0)))  /* as long as it is not a full stop or a null byte */
 if (NextLine >< '') then  /* if it is not an empty line */
 do
  if (substr(NextLine,1,1) = '.') then  /* if the line starts with a period */
  do
   NextLine = substr(NextLine,2)  /* skip the first character */
  end
 end
 call lineout MailFile,NextLine  /* write the line to the file */
 NextLine = SocketGetLine(Socket)  /* get the next line */
end

call stream MailFile,'C','CLOSE'  /* close the file */

if (NextLine = '.') then  /* if we got to the termination sequence */
do
 call LogMessage 'IN',MailFile  /* log the message */
 Success = 1  /* all is well */
end
else  /* if we did not get to the terminating lone dot, we must have a null byte, i.e. an error */
do
 say 'failed.'  /* report */
 Success = 0  /* we failed */
end

return Success  /* end of POP3RetrieveMess */

/**********************************************************************/
ProcessCommand: procedure expose Global.  /* processes a command-line sequence */
/**********************************************************************/

parse arg Switch,Argument  /* get the arguments */

select  /* do one of the following */

 when (Switch = 'ADDRESS') then  /* if the switch is "address" */
 do
  if (Argument >< '') then  /* if we have an argument */
  do
   call sysfiletree Argument,'Files.','FO'  /* look for files matching the argument */
   if (Files.0 = 0) then  /* if we find no files */
   do
    call AddError 'No files found : '||Argument  /* report */
   end
   else  /* if we find files */
   do Index = 1 to Files.0  /* take each of the files we find */
    call MakeAddressEntry Files.Index  /* call the address book procedure */
   end
  end
  if (\sysopenobject(Global.AddrDir,0,1)) then  /* if we cannot open the address book folder */
  do
   call AddError 'Cannot open the address book folder'  /* report */
  end
 end

 when (substr(Switch,1,6) = 'ATTACH') then  /* if the switch is "attach..." */
 do
  if (Argument >< '') then  /* if we have an argument */
  do
   call sysfiletree Argument,'Files.','FO'  /* look for files matching the argument */
   if (Files.0 = 0) then  /* if we find no files */
   do
    call AddError 'No files found : '||Argument  /* report */
   end
   else  /* if we find files */
   do Index = 1 to Files.0  /* take each of the files we find */
    call MakeAttachMessage Files.Index,Switch  /* call the attachment message procedure */
   end
  end
  else  /* if we have no argument */
  do
   call AddError Switch||' : missing argument'  /* report */
  end
 end

 when (Switch = 'COLLECT') then  /* if the switch is "collect" */
 do
  if (CollectMail() > 0) then  /* if we collected any waiting messages from the POP3 server */
  do
   call sysfiletree Global.TempDir||'\rxml????.pop3','Files.','FO'  /* look for POP3 files in the temp dir */
   do CollectCounter = 1 to Files.0  /* take each of the files */
    call RegisterMessage Files.CollectCounter  /* register it */
   end
  end
  if (Argument >< '') then  /* if we have an argument */
  do
   call AddError Switch||' requires no argument'  /* report */
  end
 end

 when (Switch = 'DISPATCH') then  /* if the switch is "dispatch" */
 do
  drop Global.SendFiles.  /* start with nothing */
  SendCounter = 0  /* we have no outgoing mail yet */
  ReplyCounter = 0  /* we have made no replies yet */
  if (Argument = '') then  /* if we have no argument */
  do
   Argument = Global.OutDir||'\*'  /* use any files that may be in the Out dir */
  end
  call sysfiletree Argument,'Files.','FO'  /* look for files matching the argument */
  do Index = 1 to Files.0  /* take each of the files we find */
   if (MessageSettings(Files.Index,'+*******','Match')) then  /* if the file is an incoming message */
   do
    call MakeReplyMessage Files.Index  /* generate a reply message */
    ReplyCounter = ReplyCounter + 1  /* we have made one more reply */
   end
   else  /* if the file is an outgoing message */
   do
    if (MessageSettings(Files.Index,'-+-*****','Match')) then  /* if the file is outgoing and O.K. for processing */
    do
     SendCounter = SendCounter + 1  /* up the send files counter */
     Global.Sendfiles.SendCounter = Files.Index  /* store the file name */
    end
   end
  end
  if ((Files.0 = 0) | (SendCounter > 0) | (ReplyCounter = 0)) then  /* if we had no file args at all, or if we have any outgoing files, or if we had no replies */
  do
   if (CollectMail() > 0) then  /* if we collected any waiting messages from the POP3 server */
   do
    call sysfiletree Global.TempDir||'\rxml????.pop3','Files.','FO'  /* look for POP3 files in the temp dir */
    do Index = 1 to Files.0  /* take each of the files */
     call RegisterMessage Files.Index  /* register it */
    end
   end
   if (SendCounter > 0) then  /* if we have outgoing files */
   do
    Global.SendFiles.0 = SendCounter  /* make the send files counter accessible to the SendMail routine */
    call SendMail  /* send the outgoing messages */
   end
  end
 end

 when (Switch = 'EDIT') then  /* if the switch is  "edit" */
 do
  if (Argument >< '') then  /* if we have an argument */
  do
   call sysfiletree Argument,'Files.','FO'  /* look for files matching the argument */
   if (Files.0 = 0) then  /* if we find no files */
   do
    call AddError 'No files found : '||Argument  /* report */
   end
   else  /* if we find files */
   do Index = 1 to Files.0  /* take each of the files we find */
    if (MessageSettings(Files.Index,'-*+*****','MATCH')) then  /* if the file has been sent */
    do
     call ViewSentMessage Files.Index  /* call the view sent message procedure */
    end
    else  /* if the message has not been sent */
    do
     call EditMessage Files.Index  /* call the message edit procedure */
    end
   end
  end
  else  /* if we have no argument */
  do
   call AddError Switch||' : missing argument'  /* report */
  end
 end

 when (Switch = 'EXPORT') then  /* if the switch is "export */
 do
  call AddressExport Argument  /* call the mail address book export procedure */
 end

 when (Switch = 'HELP') then  /* if the switch is "help" */
 do
  call ShowHelp Argument  /* call the help message procedure */
 end

 when (Switch = 'IMPORT') then  /* if the switch is "import" */
 do
  call AddressImport Argument  /* call the mail address book import procedure */
 end

 when (Switch = 'INFO') then  /* if the switch is "info" */
 do
  call OpenInfo  /* call the info display procedure */
  if (Argument >< '') then  /* if we have an argument */
  do
   call AddError Switch||' requires no argument'  /* report */
  end
 end

 when (Switch = 'OPEN') then  /* if the switch is "open" */
 do
  call OpenFolders  /* call the open mail folders procedure */
  if (Argument >< '') then  /* if we have an argument */
  do
   call AddError Switch||' requires no argument'  /* report */
  end
 end

 when (Switch = 'RAW') then  /* if the switch is "raw" */
 do
  if (Argument >< '') then  /* if we have an argument */
  do
   call sysfiletree Argument,'Files.','FO'  /* look for files matching the argument */
   if (Files.0 = 0) then  /* if we find no files */
   do
    call AddError 'No files found : '||Argument  /* report */
   end
   else  /* if we find files */
   do Index = 1 to Files.0  /* take each of the files we find */
    call ViewRawMessage Files.Index  /* call the raw message viewing procedure */
   end
  end
  else  /* if we have no argument */
  do
   call AddError Switch||' : missing argument'  /* report */
  end
 end

 when (Switch = 'REGISTER') then  /* if the switch is "register" */
 do
  if (Argument >< '') then  /* if we have an argument */
  do
   call sysfiletree Argument,'Files.','FO'  /* look for files matching the argument */
   if (Files.0 = 0) then  /* if we find no files */
   do
    call AddError 'No files found : '||Argument  /* report */
   end
   else  /* if we find files */
   do Index = 1 to Files.0  /* take each of the files we find */
    call RegisterMessage Files.Index  /* call the mail register procedure */
   end
  end
  else  /* if we have no argument */
  do
   call AddError Switch||' : missing argument'  /* report */
  end
 end

 when (substr(Switch,1,5) = 'REPLY') then  /* if the switch starts with "reply" */
 do
  if (Argument >< '') then  /* if we have an argument */
  do
   call sysfiletree Argument,'Files.','FO'  /* look for files matching the argument */
   if (Files.0 = 0) then  /* if we find no files */
   do
    call AddError 'No files found : '||Argument  /* report */
   end
   else  /* if we find files */
   do Index = 1 to Files.0  /* take each of the files we find */
    call MakeReplyMessage Files.Index,Switch  /* call the message reply procedure with the switch as argument */
   end
  end
  else  /* if we have no argument */
  do
   call AddError Switch||' : missing argument'  /* report */
  end
 end

 when (Switch = 'SEND') then  /* if the switch is "send" */
 do
  SendCounter = 0  /* we have no outgoing mail yet */
  if (Argument = '') then  /* if we have no argument */
  do
   Argument = Global.OutDir||'\*'  /* use any files that may be in the Out dir */
  end
  call sysfiletree Argument,'Files.','FO'  /* look for files matching the argument */
  do Index = 1 to Files.0  /* take each of the files we find */
   if (MessageSettings(Files.Index,'-+-*****','Match')) then  /* if the file is outgoing and O.K. for processing */
   do
    SendCounter = SendCounter + 1  /* up the send files counter */
    Global.Sendfiles.SendCounter = Files.Index  /* store the file name */
   end
  end
  if (SendCounter > 0) then  /* if we have outgoing files */
  do
   Global.SendFiles.0 = SendCounter  /* make the send files counter accessible to the SendMail routine */
   call SendMail  /* send the outgoing messages */
  end
 end

 when (substr(Switch,1,3) = 'SET') then  /* if the switch starts with "set" */
 do
  if (Argument >< '') then  /* if we have an argument */
  do
   call sysfiletree Argument,'Files.','FO'  /* look for files matching the argument */
   if (Files.0 = 0) then  /* if we find no files */
   do
    call AddError 'No files found : '||Argument  /* report */
   end
   else  /* if we find files */
   do Index = 1 to Files.0  /* take each of the files we find */
    call SetMessageType Files.Index,Switch  /* call the message settings procedure with the switch as argument */
   end
  end
  else  /* if we have no argument */
  do
   call AddError Switch||' : missing argument'  /* report */
  end
 end

 when (Switch = 'VIEW') then  /* if the switch was "view" */
 do
  if (Argument >< '') then  /* if we have an argument */
  do
   call sysfiletree Argument,'Files.','FO'  /* look for files matching the argument */
   if (Files.0 = 0) then  /* if we find no files */
   do
    call AddError 'No files found : '||Argument  /* report */
   end
   else  /* if we find files */
   do Index = 1 to Files.0  /* take each of the files we find */
    call ViewMessage Files.Index  /* call the message viewing procedure */
   end
  end
  else  /* if we have no argument */
  do
   call AddError Switch||' : missing argument'  /* report */
  end
 end

 otherwise  /* if none of the above applies */
 do
  call AddError 'Invalid switch "'||Switch||'"'  /* report */
 end

end

return 1  /* end of ProcessCommand */

/**********************************************************************/
PutAddressData: procedure expose Global.  /* puts the address data from the edit file into the template objects in the address folder */
/**********************************************************************/

parse arg EditFile,AddDir  /* get the names of the edit ffile and the address folder */

if (\OpenFile(EditFile,'READ')) then  /* if we cannot open the address file for writing */
do
 return 0  /* and quit */
end

call sysfiletree AddDir||'\*.*','Objects','BO'  /* get the contents */

do Index = 1 to Objects.0  /* take each of the file objects */
 if (\sysdestroyobject(Objects.Index)) then  /* if we cannot get rid of the object */
 do
  call AddError 'Cannot delete "'||Objects.Index||'"'  /* report */
 end
end

InBuffer = charin(EditFile,1,chars(EditFile))  /* read the contents of the address file into an input buffer */
InBuffer = Global.CRLF||InBuffer||Global.CRLF  /* add a CRLF at both ends to make sure the block markers are recognized */
call stream EditFile,'C','CLOSE'  /* close the file */

Sep_1 = Global.CRLF||'<TITLE>'||Global.CRLF  /* define a separator */
Sep_2 = Global.CRLF||'<TEXT>'||Global.CRLF  /* define a separator */
Sep_3 = Global.CRLF||'<KEYWORDS>'||Global.CRLF  /* define a separator */
Sep_4 = Global.CRLF||'<END>'||Global.CRLF  /* define a separator */

LFCR = d2c(10)||d2c(13)  /* define a reverse CRLF */

EntryNo = 0  /* start at 0 */

do while (InBuffer >< '')  /* go on while we have contents left */
 EntryNo = EntryNo + 1  /* up the entry number */
 parse var InBuffer (Sep_1) Block (Sep_4) InBuffer  /* get the next block */
 if (Block = '') then  /* if the block is empty */
 do
  if (InBuffer >< '') then  /* if it not because we have run out of contents */
  do
   call AddError 'Entry number '||EntryNo||' contains no data'  /* report */
  end
 end
 else  /* if the block is not empty */
 do
  Name = ''  /* start with nothing */
  Text = ''  /* start with nothing */
  Comm = ''  /* start with nothing */
  parse var Block Block (Sep_3) Comm  /* extract any keywords */
  parse var Block Block (Sep_2) Text  /* extract any text content */
  Name = Block  /* the rest must be a name */
  do while (substr(Name,1,2) = Global.CRLF)  /* as long as the name starts with a CRLF */
   Name = substr(Name,3)  /* get rid of it */
  end
  RevName = reverse(Name)  /* reverse the name */
  do while (substr(RevName,1,2) = LFCR)  /* as long as the reverse name starts with a reverse CRLF */
   RevName = substr(RevName,3)  /* get rid of it */
  end
  Name = reverse(RevName)  /* turn the name back again */
  if (Name = '') then  /* if we have no name */
  do
   call AddError 'Entry number '||EntryNo||' contains no name'  /* report */
  end
  else  /* if we have a name */
  do
   do while (substr(Text,1,2) = Global.CRLF)  /* as long as the text starts with a CRLF */
    Text = substr(Text,3)  /* get rid of it */
   end
   RevText = reverse(Text)  /* reverse the text */
   do while (substr(RevText,1,2) = LFCR)  /* as long as the reverse text starts with a reverse CRLF */
    RevText = substr(RevText,3)  /* get rid of it */
   end
   Text = reverse(RevText)  /* turn the text back again */
   if (Text = '') then  /* if we have no text */
   do
    call AddError 'Entry number '||EntryNo||' contains no text'  /* report */
   end
   RevName = reverse(Name)  /* reverse the name bit */
   parse var RevName . '\' RevPath  /* get the (reverse) path */
   Path = reverse(RevPath)  /* reverse it to get the path */
   Target = AddDir  /* we start here */
   do while (Path >< '')  /* as long as the path continues */
    parse var Path Folder '\' Path  /* get the first part of the path */
    Settings = 'ALWAYSSORT=YES;ICONVIEW=FLOWED,MINI;WORKAREA=YES;'  /* the settings to use for the folder object */
    call syscreateobject 'WPFolder',Folder,Target,Settings,'FAIL'  /* try to create the folder, but fail if it already exists */
    Target = Target||'\'||Folder  /* move up the path for any next bit */
   end
   NewName = ''  /* start with nothing */
   do while (Name >< '')  /* as long as we have bits of name left */
    parse var Name NextBit (Global.CRLF) Name  /* get the bit up to the next CRLF */
    NewName = NewName||NextBit||' '  /* store the next bit */
   end
   NewName = translate(NewName,'()()_____-(..'||"''",'<>[]~?%&*|/;,"'||"`")  /* replace awkward characters */
   OutFile = AddDir||'\'||NewName  /* the name of the output file */
   if (OpenFile(OutFile,'WRITE')) then  /* if we can open the output file */
   do
    call charout OutFile,Text  /* write any text content to the file */
    call stream OutFile,'C','CLOSE'  /* close the file */
    call MessageSettings OutFile,'--------','Change'  /* make it a fresh outgoing mail message file */
    call syssetobjectdata OutFile,'TEMPLATE=YES;'  /* make the file a template */
    do while (substr(Comm,1,2) = Global.CRLF)  /* as long as the keywords start with a CRLF */
     Comm = substr(Comm,3)  /* get rid of it */
    end
    RevComm = reverse(Comm)  /* reverse the keywords */
    do while (substr(RevComm,1,2) = LFCR)  /* as long as the reverse keywords start with a reverse CRLF */
     RevComm = substr(RevComm,3)  /* get rid of it */
    end
    Comm = reverse(RevComm)  /* turn the keywords back again */
    if (Comm >< '') then  /* if we still have keywords */
    do
     call PutKeyPhrases OutFile,Comm  /* add the key phrases */
    end
   end
  end
 end
end

return 1  /* end of PutAddressData */

/**********************************************************************/
PutKeyPhrases: procedure expose Global.  /* adds key phrases to an address book entry */
/**********************************************************************/

parse arg File,KeyPhrases  /* get the arguments */

Contents = ''  /* start with nothing */
Count = 0  /* start at 0 */
do while (length(KeyPhrases) > 0)  /* run through the key phrases */
 parse var KeyPhrases Line (Global.CRLF) KeyPhrases  /* get the first line of KeyPhrases */
 Len = reverse(right(d2c(Length(Line)),2,'00'x))  /* set Len */
 Contents = Contents||reverse('FFFD'x)||Len||Line  /* prepare Contents */
 Count = Count + 1  /* up the counter */
end
CodePage = '0000'x  /* set code page */
Count = reverse(right(d2c((Count)),2,'00'x))  /* set Count */
KeyPhrases = reverse('FFDF'x)||CodePage||Count||Contents  /* prepare keywords */
Success = (sysputea(File,'.KEYPHRASES',KeyPhrases) = 0)  /* if we can attach them to the file, we're O.K. */

return Success  /* end of PutKeyPhrases */

/**********************************************************************/
RegisterMessage: procedure expose Global.  /* registers incoming messages */
/**********************************************************************/

parse arg MessFile  /* get the argument */

call MessageContents MessFile  /* get the file contents */

if (\MessageSettings(MessFile,'+-------','CHANGE')) then  /* if we cannot attach the unread received mail message setting */
do
 call AddError 'Cannot set mail message attributes of "'||MessFile||'"'  /* report */
 return 0  /* and quit */
end

if (\syscopyobject(MessFile,Global.InDir)) then  /* if the mail file cannot be copied to the In directory ("sysmoveobject" does not always set the EAs) */
do
 call AddError 'Cannot copy "'||MessFile||'" to "'||Global.Indir||'"'  /* report */
 return 0  /* and quit */
end

call sysfiledelete MessFile  /* get rid of the original file -- no check, it may already have been deleted by another process */

MessFile = Global.InDir||'\'||filespec('N',MessFile)  /* use a new file spec */

NewName = MakeTitle(MessFile,Global.MessHead)  /* get the new name for the mail file */

if (\syssetobjectdata(MessFile,'TITLE='||NewName)) then  /* if we cannot set the new file title */
do
 call AddError 'Cannot set message file title'  /* report */
end

if (Global.Signals) then  /* if a signal is wanted */
do
 call beep 888,50  /* signal */
 call beep 444,100  /* signal */
end

call MailWarning  /* create a warning icon if necessary */

return 1  /* end of RegisterMessage */

/**********************************************************************/
SendMail: procedure expose Global.  /* sends outgoing messages */
/**********************************************************************/

do Index = 1 to Global.SendFiles.0  /* take each of the outgoing mail files */

 MessFile = Global.SendFiles.Index  /* this is easier to use */

 call syssetobjectdata MessFile,'NODELETE=YES;NODRAG=YES;NOMOVE=YES'  /* prevent interim removal of the file object */

 call syssetfiledatetime MessFile  /* set a new file date/time stamp */

 DateString = MessageDate()  /* get a date string */

 call MessageContents MessFile  /* get the message file contents */

 SendFile = MakeSendFile(MessFile,DateString)  /* try to get a sendable file */

 call syssetobjectdata MessFile,'NODELETE=NO;NODRAG=NO;NOMOVE=NO'  /* make the message file object movable again */

 Sent = 0  /* no mail sent yet */

 if (SendFile >< '') then  /* if we get back a sendable file name */
 do
  if ((symbol('Global.RunDispatch') = 'VAR') & (Global.RunDispatch >< '')) then  /* if we have a RunDispatch command line */
  do
   parse var Global.RunDispatch FirstBit '%N' LastBit  /* look for the filename macro in the dispatch command line */
   CommLine = FirstBit  /* this is the command to start with */
   if (pos('%N',Global.RunDispatch) > 0) then  /* if the settings line contained a file spec macro */
   do
    CommLine = CommLine||SendFile||LastBit  /* this is the complete command */
   end
   say 'RunDispatch configuration option parsed as : '||CommLine  /* report */
   address cmd CommLine  /* run the command */
   Sent = (RC = 0)  /* if the dispatch command completed without error, all is well */
  end
  else  /* if we have no command line */
  do
   Sent = SendMailSMTP(SendFile)  /* use the internal routine to send the file */
  end
 end

 if (Sent > 0) then  /* if all went well */
 do
  AttDir = SetAttachments(MessFile,'SEND',DateString)  /* create an attachments list, rewrite the message EAs, and get the attachments folder name */
  call DeleteDir AttDir  /* get rid of the attachments folder */
  call sysfiledelete SendFile  /* get rid of the sendable file */
  call MessageSettings MessFile,'**+*****','Change'  /* new sent mail message setting: Processed */
  call MoveMessage MessFile,Global.MessHead  /* move the mail file to storage if necessary */
  if (Global.Signals) then  /* if a signal is wanted */
  do
   call beep 444,20  /* signal */
   call beep 888,80  /* signal */
  end
 end

end

return Sent  /* end of SendMail */

/**********************************************************************/
SendMailSMTP: procedure expose Global.  /* Sends mail to an SMTP mail relay */
/**********************************************************************/

/**********************************************************************/
/* See if we have all the arguments we really need                    */
/**********************************************************************/

if (Global.SMTPServer = '') then  /* if we have no server name */
do
 call AddError 'Missing configuration entry : SMTPServer'  /* report */
 return 0  /* and quit with no success */
end

/**********************************************************************/
/* See if we can load the REXX sockets library                        */
/**********************************************************************/

if (LoadRexxSock() = 2) then  /* if we cannot load the REXX socket functions */
do
 call AddError 'Cannot load REXX socket library'  /* report and exit */
 return 0  /* and quit with no success */
end

/**********************************************************************/
/* Get the message contents                                           */
/**********************************************************************/

parse arg SendFile  /* get the argument */

if (\OpenFile(SendFile,'READ')) then  /* if we cannot open the file for reading */
do
 return 0  /* quit with no success */
end

MessCont = charin(SendFile,1,chars(SendFile))  /* get the message contents */

call stream SendFile,'C','CLOSE'  /* close the message file */

/**********************************************************************/
/* Get some header info from the message header                       */
/**********************************************************************/

if (\GetHeaderStuff()) then  /* if we cannot find a sender and at least one recipient */
do
 return 0  /* and quit without success */
end

/**********************************************************************/
/* Expand the list of recipients                                      */
/**********************************************************************/

if (\ExpandMailList()) then  /* if we get an address error while trying to expand any mailing list addresses */
do
 return 0  /* quit */
end

/**********************************************************************/
/* Construct a local host name                                        */
/**********************************************************************/

parse var Sender . '@' Domain '>'  /* get the sender's domain name */

HostName = value('HOSTNAME',,'OS2Environment')  /* try to get a host name from the OS/2 environment */
if (HostName >< '') then  /* if we have a host name */
do
 HostName = HostName||'.'||Domain  /* add the domain name to it */
end
else  /* if we have no host name */
do
 HostName = Domain  /* just use the domain name */
end

/**********************************************************************/
/* Connect to the server                                              */
/**********************************************************************/

call charout StdOut,'Connecting to '||Global.SMTPServer||' : '  /* report */

Socket = ServerConnect(Global.SMTPServer,25)  /* get the socket number for a server connection through port 25 */

if (Socket = '') then  /* if we have no socket */
do
 return 0  /* quit without success */
end

Global.Socket.SocketBuffer = ''  /* start with an empty socket buffer */

if (\SocketAnswer(Socket,220)) then  /* if we don't get a 220 return code */
do
 call AddError 'No response from '||Global.SMTPServer  /* report */
 call SocketClose Socket  /* close the socket */
 return 0  /* return with no socket */
end

if (\SocketSendLine(Socket,'HELO '||Domain,250)) then  /* if don't get 250 in return when we send this line */
do
 call AddError Domain||' not accepted'  /* report */
 call ServerDisconnect Socket,Global.SMTPServer  /* disconnect */
 return 0  /* retrun with no socket */
end

say 'O.K.'  /* report */

/**********************************************************************/
/* Send the message(s)                                                */
/**********************************************************************/

Sent = 0  /* we haven't sent anything yet */

say 'Sending message...'  /* report */
Number = SMTPSendMessage()  /* the number of messages sent */

if (Number = 1) then  /* if we have exactly 1 recipient */
do
 Plural = ''  /* we need no plural 's' */
end
else  /* if we have 0 or multiple recipients */
do
 Plural = 's'  /* we need a plural 's' */
end

say 'Message sent to '||Number||' recipient'||Plural||'.'  /* report */

/**********************************************************************/
/* Disconnect from the server                                         */
/**********************************************************************/

call ServerDisconnect Socket,Global.SMTPServer  /* disconnect */

/**********************************************************************/
/* We're all done                                                     */
/**********************************************************************/

return Number  /* end of SendMailSMTP */

/**********************************************************************/
ServerConnect: procedure expose Global.  /* connects to the SMTP server */
/**********************************************************************/

parse arg Server,Port  /* get the arguments */

if (sockinit() >< 0) then  /* if we cannot initialize the socket buffers */
do
 call AddError 'Socket initialization error'  /* report */
 return ''  /* return with no socket */
end

Socket = socksocket('AF_INET','SOCK_STREAM','IPPROTO_TCP')  /* create a socket for TCP protocol */

if (Socket < 0) then  /* if we did not get a socket */
do
 call AddError 'Cannot get socket'  /* report */
 return ''  /* return with no socket */
end

if (\sockgethostbyname(Server,'Host.!')) then  /* if we cannot get the SMTP server's IP address by name */
do
 call AddError 'Cannot resolve '||Server  /* report */
 return ''  /* return with no socket */
end

Address.!family = 'AF_INET'  /* define the connection type */
Address.!port = Port  /* define the port to use */
Address.!addr = Host.!addr  /* define the connect address */

if (sockconnect(Socket,'Address.!') >< 0) then  /* if we cannot open the socket connection */
do
 call AddError 'Cannot connect to '||Server  /* report */
 return ''  /* return with no success */
end

return Socket  /* end of ServerConnect */

/**********************************************************************/
ServerDisconnect: procedure expose Global.  /* disconnect from the server */
/**********************************************************************/

parse arg Socket,Server  /* get the arguments */

call charout StdOut,'Disconnecting from '||Server||' : '  /* report */

if (SocketSendLine(Socket,'QUIT','221 +OK')) then  /* if we get 221 or +OK in return when we send this line */
do
 say 'O.K.'  /* there's little we can do now */
end
else  /* if an error occurred */
do
 say 'Error.'  /* report */
end

call SocketClose Socket  /* close the socket */

return  /* end of ServerDisconnect */

/**********************************************************************/
SetAttachments: procedure expose Global.  /* handles attachment file folders and settings */
/**********************************************************************/

/**********************************************************************/
/* List of switch options:                                            */
/* [none] : Looks for an existing attachments folder. If no reference */
/*          is found, or if the folder no longer exists, creates an   */
/*          attachments folder and returns its name; otherwise,       */
/*          returns the name of the existing folder.                  */
/* CREATE : Create a new attachments folder and set the EA.           */
/*          Returns the full name of the new folder.                  */
/* DELETE : Delete the attachments folder EA.                         */
/*          Returns 1 on success, otherwise 0                         */
/* GETDIR : The attachment folder name is retrieved from the EAs.     */
/*          Returns the folder name.                                  */
/* LIST   : An attachment list is retrieved.                          */
/*          Returns the list of attachments.                          */
/* SEND   : An attachment list is created, and the folder EA deleted. */
/*          Returns 1 on success, otherwise 0                         */
/**********************************************************************/

parse arg MessFile,Option,DateString  /* get the parameters */

if (Option = '') then  /* if no option was passed */
do
 AttDir = AttGetDir(MessFile)  /* get the name of the attachments directory, if any */
 if (AttDir = '') then  /* if no directory name was found */
 do
  AttDir = AttCreate(MessFile)  /* create it */
 end
 else  /* if a directory name was returned */
 do
  call sysfiletree AttDir,'Dirs','DO'  /* look for the actual directory */
  if (Dirs.0 = 0) then  /* if no such directory was found (i.e. it was deleted) */
  do
   AttDir = AttCreate(MessFile)  /* create it */
  end
 end
 Result = AttDir  /* return attachments folder name */
end
else  /* if an option was passed */
do
 Option = translate(Option)  /* make it uppercase */

 select  /* we have the following options */

  when (Option = 'CREATE') then  /* if a new attachments folder is to be created */
  do
   Result = AttCreate(MessFile)  /* if we can create a new attachments folder, we're O.K. */
  end

  when (Option = 'DELETE') then  /* if the attachment dir is to be deleted */
  do
   Result = 0  /* no result yet */
   if (AttGetDir(MessFile) >< '') then  /* if there is an attachment directory */
   do
    Result = AttDelete(MessFile,AttDir)  /* if we can delete the attachments directory EA, we're O.K. */
   end
  end

  when (Option = 'GETDIR') then  /* if the option is GetDir */
  do
   Result = AttGetDir(MessFile)  /* go get the attachment directory name, if any */
  end

  when (Option = 'LIST') then  /* if the message file was sent */
  do
   Result = AttGetList(MessFile)  /* get the attachments list */
  end

  when (Option = 'SEND') then  /* if the message file was sent */
  do
   Result = AttList(MessFile,AttGetDir(MessFile),DateString)  /* if we can set the date and the attachments list EA, we're O.K. */
  end

  otherwise  /* if it is an unknown option */
  do
   nop  /* do nothing */
  end

 end
end

return Result  /* that's all, folks! */

/**********************************************************************/
SetMessageType: procedure expose Global.  /* sets message type attributes */
/**********************************************************************/

parse arg MessFile,Switch  /* get the arguments */

MessSettings = (MessageSettings(MessFile))  /* get the message file's status settings */

Switch = translate(Switch)  /* convert to upper case */

if (Switch = 'SETASCIIQP') then  /* if we are to toggle the ASCII/Q-P setting */
do
 if (substr(MessSettings,4,1) = '+') then  /* if the ASCII flag has already been set */
 do
  Switch = SETQP  /* reset it */
 end
 else  /* if the ASCII flag has not been set yet */
 do
  Switch = SETASCII  /* set it */
 end
end

if (Switch = 'SETNONASCII') then  /* if we have this switch -- a legacy from previous releases */
do
 Switch = 'SETQP'  /* turn it into this */
end

BlockIncoming = 'SETNEW SETSENT SETASCII SETQP'  /* these will not be processed for incoming messages */
BlockOutgoing = 'SETRECEIVED SETVIEWED'  /* these will not be processed for outgoing messages */
BlockSent = 'SETASCII SETQP'  /* these will not be processed for sent messages */

if (substr(MessSettings,1,1) = '+') then  /* if the message is incoming */
do
 if (pos(Switch,BlockIncoming) > 0) then  /* if the switch is blocked */
 do
  call AddError 'Cannot use '||Switch||' on incoming message files'  /* report */
  return 0  /* and quit */
 end
end
else  /* if the message is outgoing */
do
 if (pos(Switch,BlockOutgoing) > 0) then  /* if the switch is blocked */
 do
  call AddError 'Cannot use '||Switch||' on outgoing message files'  /* report*/
  return 0  /* and quit */
 end
 else  /* if the switch is not blocked */
 do
  if ((substr(MessSettings,3,1) = '+') & (pos(Switch,BlockSent) > 0)) then  /* if the message was sent, and switch is blocked */
  do
   call addError 'Cannot use '||Switch||' on sent message files'  /* report */
   return 0  /* and quit */
  end
 end
end

Success = 0  /* no success yet */

SetTitleString = 'SETNEW SETSENT SETRECEIVED SETVIEWED'  /* change the title as well for these */

SwitchString = 'SETNEW SETSENT SETRECEIVED SETVIEWED SETASCII SETQP'  /* the available types */
SettingsString = '----**** -++***** +--***** +-+***** ***+**** ***-****'  /* the corresponding settings strings */
Settings = word(SettingsString,wordpos(Switch,SwitchString))  /* get the settings that correspond with the message type */

if (MessageSettings(MessFile,Settings,'CHANGE')) then  /* if we can attach the mail message settings */
do
 if (wordpos(Switch,SetTitleString) > 0) then  /* if the switch is in the list for which the title should be set */
 do
  call MessageContents MessFile  /* get the file contents */
  NewName = MakeTitle(MessFile,Global.MessHead)  /* get a new name for the mail file */
  TitleSetting = 'TITLE='||NewName  /* the title setting to use */
  end
 else  /* if the switch is not in the title set list */
 do
  TitleSetting = ''  /* we have no title setting */
 end
 if (syssetobjectdata(MessFile,'NODELETE=NO;NODRAG=NO;NOMOVE=NO;'||TitleSetting)) then  /* if we can set the file attributes */
 do
  Success = 1  /* we're O.K. */
 end
end

return Success  /* end of SetMessageType */

/**********************************************************************/
ShowHelp: procedure expose Global.  /* displays a help message */
/**********************************************************************/

parse upper arg Argument  /* get the argument */

Argument = strip(Argument,'B','"')  /* get rid of any quotes */
Argument = strip(Argument,'L','/')  /* get rid of any leading slash */

select  /* do one of the following */

 when (Argument = '') then  /* if there is no argument */
 do
  Argument = 'TOPICS'  /* use this */
 end

 when ((substr(Argument,1,3) = 'SET') & (Argument >< 'SETTINGSFILE')) then  /* if the argument starts with 'SET', but is not 'SETTINGSFILE' */
 do
  Argument = 'SET'  /* use just that bit */
 end

 when (substr(Argument,1,5) = 'REPLY') then  /* if the argument starts with 'REPLY' */
 do
  Argument = 'REPLY'  /* use just that bit */
 end

 when (substr(Argument,1,6) = 'ATTACH') then  /* if the argument starts with 'ATTACH' */
 do
  Argument = 'ATTACH'  /* use just that bit */
 end

 otherwise  /* in any other vase */
 do
  nop  /* do nothing */
 end

end

HelpFile = Global.HelpDir||'\'||Argument||'.txt'  /* the file to use */

call sysfiletree HelpFile,'Files.','FO'  /* look for the file */

if (Files.0 > 0) then  /* if we can find it */
do
 'more < '||HelpFile  /* call more to list the file */
end
else  /* if we cannot find the file */
do
 if (Argument = 'TOPICS') then  /* if the user asked for the topics list */
 do
  Topics = ''  /* start with nothing */
 end
 else  /* if we have an unknown topic */
 do
  Topics = 'Unknown topic : "'||Argument||'"'||Global.EmptyLine  /* start with this bit of text */
 end
 call sysfiletree Global.HelpDir||'\*.txt','Files.','FO'  /* see what we've got */
 if (Files.0 = 0) then  /* if we can't find any help files */
 do
  call AddError 'Cannot find any help text files in "'||Global.HelpDir||'"'  /* report */
 end
 else  /* if we have one or more help text files */
 do
  call sysstemsort 'Files.','A','I'  /* sort the topic files we found in ascending order, ignoring case */
  Topics = Topics||'Usage : RexxMail /help [topic]'||Global.EmptyLine  /* add this bit of text */
  Topics = Topics||'The following topics are available:'||Global.EmptyLine  /* add this bit of text */
  do Index = 1 to Files.0  /* take each of the files we find */
   FileName = filespec('N',Files.Index)  /* get the file name bit */
   Topics = Topics||' '||substr(FileName,1,length(FileName)-4)||Global.CRLF  /* add the topic to the topics we've got */
  end
  'echo '||Topics||' | more'  /* pipe the result to the more command */
 end
end

return 1  /* end of HelpText */

/**********************************************************************/
SMTPSendMessage:  /* send the message */
/**********************************************************************/

if (\SocketSendLine(Socket,'MAIL FROM:<'||Sender||'>',250)) then  /* if we don't get 250 in return when we send this line */
do
 return 0  /* and quit without success */
end

AllRecipients = strip(NewRecipients)  /* copy the list of recipients, and get rid of superfluous blanks */
Recipients = 0  /* we have no successful recipients yet */

do while (NewRecipients >< '')  /* as long as we have recipients left in the "normal" list */
 parse var NewRecipients NextRecipient NewRecipients  /* get the next recipient from the list */
 if (\SocketSendLine(Socket,'RCPT TO:<'||NextRecipient||'>',250 251)) then  /* if we don't get 250 or 251 in return when we send this line */
 do
  return 0  /* and quit without success */
 end
 else  /* if we successfully entered the recipient */
 do
  Recipients = Recipients + 1  /* up the counter */
 end
end

MessID = SMTPSendText(Socket,HostName,RecpHead||MessHead,MessBody)  /* if we can send these text parts, we get a message ID in return */
if (MessID >< '') then  /* if we have a message ID */
do
 call LogMessage 'OUT',Sender,AllRecipients,Subject,MessID  /* log this message */
end
else  /* if we cannot send the message  */
do
 Recipients = 0  /* reset the counter */
end

return Recipients  /* end of SMTPSendMessage */

/**********************************************************************/
SMTPSendText: procedure expose Global.  /* sends message text to the server */
/**********************************************************************/

parse arg Socket,HostName,FirstBit,LastBit  /* get the arguments */

if (\SocketSendLine(Socket,'DATA',354)) then  /* if we don't get 354 in return when we send this line */
do
 return 0  /* return with an error */
end

MessID = '<'||date('S')||'.'||translate(substr(time('L'),1,11),'.',':')||'.'||right(random(999),3,'0')||'@'||HostName||'>'  /* the ID string */
MessIDLine = 'Message-Id: '||MessID||Global.CRLF  /* the message ID line */

DataBlock = FirstBit||MessIDLine||Global.CRLF||LastBit  /* this is the complete message text block */

do while (DataBlock >< '')  /* as long as we have data to send */
 parse var DataBLock DataLine (Global.CRLF) DataBlock  /* get the next line */
 if (DataLine <> '') then  /* if it is not an empty line */
 do
  if (substr(DataLine,1,1) = '.') then  /* if the line start with a period */
  do
   DataLine = '.'||DataLine  /* stick an extra period on the front */
  end
 end
 if (\SocketSendLine(Socket,DataLine)) then  /* if we cannot send this to the socket */
 do
  MessID = ''  /* we have an error */
 end
end

if (MessID >< '') then  /* if all went well */
do
 if (\SocketSendLine(Socket,'.',250)) then  /* if we don't get 250 in return when we send this line */
 do
  MessID = ''  /* we have an error */
 end
end

return MessID  /* end of SMTPSendText */

/**********************************************************************/
SocketAnswer: procedure expose Global.  /* see if we got the right answer */
/**********************************************************************/

parse arg Socket,ReplyCodes  /* get the arguments */

Answer = SocketGetLine(Socket)  /* get the next line from the socket (buffer) */

if (Answer >< '') then  /* if we have something */
do
 if (ReplyCodes >< '') then  /* if we have reply codes to match */
 do
  if (wordpos(word(Answer,1),ReplyCodes) > 0) then  /* if the first word of the answer is in of the reply codes we want */
  do
   Answer = 1  /* return success */
  end
  else  /* if the first word is not in the list of reply codes */
  do
   call AddError 'Error from server : '||Answer  /* report */
   Answer = 0  /* no success */
  end
 end
end
else  /* if we get no answer */
do
 Answer = 0  /* no success */
end

return answer  /* end of SocketAnswer */

/**********************************************************************/
SocketClose: procedure expose Global.  /* closes the IP socket */
/**********************************************************************/

parse arg Socket  /* get the argument */

if (SockSoclose(Socket) >< 0) then  /* close the socket */
do
 call AddError 'Error closing socket'  /* report */
end

return  /* end of SocketClose */

/**********************************************************************/
SocketGetLine: procedure expose Global.  /* get a line and optionally see if we got the right answer */
/**********************************************************************/

parse arg Socket  /* get the argument */

do while (pos(Global.CRLF,Global.Socket.SocketBuffer) = 0)  /* as long as we have no CRLF in the buffer */
 if (sockrecv(Socket,'SocketData',8096) > 0) then  /* if we can get data from the socket */
 do
  Global.Socket.SocketBuffer = Global.Socket.SocketBuffer||SocketData  /* add the results to the buffer */
 end
 else  /* if we cannot get anything from the socket */
 do
  call AddError 'Connection closed'  /* report */
  return d2c(0)  /* and return a null byte */
 end
end

parse var Global.Socket.SocketBuffer NextLine (Global.CRLF) Global.Socket.SocketBuffer  /* take the bit before the CRLF from the buffer */

return NextLine  /* end of GetPOP3ResponseLine */

/**********************************************************************/
SocketSendLine: procedure expose Global.  /* send a line to a socket */
/**********************************************************************/

parse arg Socket,DataToSend,ReplyCodes  /* get the argument */

Answer = 1  /* assume success */

DataToSend = DataToSend||Global.CRLF  /* add a CRLF to the data to be sent */
DataLength = length(DataToSend)  /* get the length of the data to be sent */

do while (DataLength > 0)  /* as long as we have data left to send */
 DataLength = socksend(Socket,DataToSend)  /* see how many bytes we can send */
 if (DataLength > 0) then  /* if we sent data */
 do
  DataToSend = substr(DataToSend,DataLength+1)  /* get rid of the bit we sent */
  DataLength = length(DataToSend)  /* get the length of the remaining data to be sent */
 end
 else  /* if we sent no data or got an error */
 do
  call AddError 'Connection closed by server'  /* report */
  Answer = 0  /* all is not well */
 end
end

if ((Answer) & (ReplyCodes >< '')) then  /* if all went well and if we have reply codes to match */
do
 if (ReplyCodes = '?') then  /* if we want to see the answer itself */
 do
  ReplyCodes = ''  /* go on without the reply code */
 end
 Answer = SocketAnswer(Socket,ReplyCodes)  /* if we get the correct answer, all is well */
end

return Answer  /* end of SendString */

/**********************************************************************/
SortedDate: procedure expose Global.  /* returns sorted mail message date string */
/**********************************************************************/

parse arg DateStr  /* get the argument */

if (DateStr = '') then  /* if we have no date string */
do
 return ''  /* return with nothing */
end

if (\Global.ISODates) then  /* if we do not want the date sorted according to ISO specs */
do
 return DateStr  /* simply return the string we've got */
end

parse var DateStr DoW Rest  /* extract all the bits we need */

if (substr(DoW,length(DoW)) = ',') then  /* if a DoW reference was included, it should be followed by a comma */
do
 DoW = ' ('||substr(DoW,1,3)||')'  /* use the first 3 characters, between brackets, and with a leading space */
 parse var Rest Day Mon Year Time Zone  /* extract all the bits we need */
end
else  /* if there was no DoW reference */
do
 Day = DoW  /* the first bit was the day of the month */
 DoW = ''  /* we have no DoW */
 parse var Rest Mon Year Time Zone  /* extract all the bits we need */
end

if (Year < 100) then  /* if Year has only 2 digits */
do
 if (Year < 50) then  /* if it's less than 50 */
 do
  Year = '20'||Year  /* it must be the 21st century */
 end
 else  /* if it's 70 or over */
 do
  Year = '19'||Year  /* it must be the 20th century */
 end
end

Months = 'JAN FEB MAR APR MAY JUN JUL AUG SEP OCT NOV DEC'  /* the first three letters of the months in upper case */
Month = wordpos(translate(substr(Mon,1,3)),Months)  /* convert Mon to Month figure */

Month = right(Month,2,'0')  /* pad it with 0 if necessary */

Day = right(Day,2,'0')  /* pad it with 0 if necessary */

NewDate = Year||'-'||Month||'-'||Day||' '||Time||' '||Zone||DoW  /* this is our new date & time string */

return NewDate  /* end of SortedDate */

/**********************************************************************/
SystemMessage: procedure expose Global.  /* sends a system message to the current user */
/**********************************************************************/

parse arg Subject,MessText  /* get the arguments */

MessFile = systempfilename(Global.TempDir||'\RXML????.SYST')  /* get a unique file name */

if (\OpenFile(MessFile)) then  /* if we cannot open the file */
do
 return 0  /* quit */
end

if (Global.Name = '') then  /* if we do not have a user name */
do
 Global.Name = 'RexxMail user'  /* use this */
end

if (Global.Address = '') then  /* if we do not have a user name */
do
 Global.Address = 'user@localhost'  /* use this */
end

call lineout MessFile,'Date: '||MessageDate(1)  /* start with a date line, but do not contact a time server */
call lineout MessFile,'From: "RexxMail system messages" <messages@rexxmail>'  /* add a sender line */
call lineout MessFile,'To: "'||Global.Name||'" <'||Global.Address||'>'  /* add a recipient line */
call lineout MessFile,'Subject: '||Subject  /* add a subject line */
call lineout MessFile,''  /* add an empty line */
call charout MessFile,MessText  /* add the message text */

call stream MessFile,'C','CLOSE'  /* close the message file */

call RegisterMessage MessFile  /* register the message */

return 1  /* end of SystemMessage */

/**********************************************************************/
TimeZoneString: procedure expose Global.  /* constructs the time zone string */
/**********************************************************************/

parse arg TimeBytes  /* get the argument */

SysTime = ((date('B') - 693595) * 24 * 60 * 60) + time('S')  /* system seconds since midnight, 1900-01-01 */

TimeBytes = reverse(TimeBytes)  /* reverse the time bytes */
AbsTime = 0  /* start with nothing */

do Index = 1 to 4  /* take each of the bytes, starting with the low order one */
 AbsTime = AbsTime + (c2d(substr(TimeBytes,Index,1)) * (256 ** (Index-1)))  /* absolute seconds since midnight, 1900-01-01 */
end

TimeDiff = SysTime - AbsTime  /* the time difference in seconds, plus or minus */

if (sign(TimeDiff) = -1) then  /* if the time difference is negative */
do
 TimeSign = '-'  /* we need a minus sign */
 TimeDiff = -1 * TimeDiff  /* reverse the sign of the time difference */
end
else  /* if the time difference is not negative */
do
 TimeSign = '+'  /* we need a plus sign */
end

TimeDiff = (TimeDiff + 900) % 1800  /* the system offset, rounded to the nearest half hour */

if ((TimeSign = '-') & (TimeDiff = 0)) then  /* if we have a negative time difefrence, and the 30-minute offset is 0, i.e. we are only a bit behind */
do
 TimeSign = '+'  /* use a plus */
end

ZoneHours = (TimeDiff % 2) * 100  /* construct the hours number */

ZoneHalfHours= (TimeDiff // 2) * 30  /* construct the half hours number */

TimeZone = TimeSign||right((ZoneHours + ZoneHalfHours),4,'0')  /* the whole time zone */

return TimeZone  /* end of TimeZoneString */

/**********************************************************************/
UnpackMessage: procedure expose Global.  /* unpacks received mail files */
/**********************************************************************/

parse arg MessFile  /* get the mail file name */

AttDir = systempfilename(Global.TempDir||'\RXML????')  /* get a unique directory name in the temp dir */

if (sysmkdir(AttDir) >< 0) then  /* if we fail to create the dir */
do
 call AddError 'Cannot create attachments folder'  /* report */
 return ''  /* and return with nothing */
end

ReadFile = AttDir||'.READ'  /* the name for the reader file */

call MessageContents MessFile  /* extract the message contents */

HeadText = MakeHeaderLines(Global.MessHead,Global.ReaderHeader)  /* get the necessary header text from the mail file */

BodyText = ''  /* start with nothing */

InfoText = ''  /* we have no info text yet */

/**********************************************************************/
/* The following 6 lines of code are needed to work around a bug in   */
/* the "munpack.exe" decoder program.                                 */
/**********************************************************************/

MIME = (GetHeaderEntry(Global.MessHead,'Mime-version:') >< '')  /* it is a MIME message */
NoCTE = (GetHeaderEntry(Global.MessHead,'Content-transfer-encoding:') = '')  /* there is no "content-transfer-encoding" line */
XBit = (pos('BIT',translate(GetHeaderEntry(Global.MessHead,'Content-transfer-encoding:'))) > 0)  /* it is a (MIME) ASCII message */
UUE = (pos('UUENCODE',translate(GetHeaderEntry(Global.MessHead,'Encoding:'))) > 0)  /* some mailers still send UUE attachments */
PlainText = (pos('TEXT/PLAIN',translate(GetHeaderEntry(Global.MessHead,'Content-type:'))) > 0)  /* it is a plain text message */

if ((MIME & \(PlainText & (XBit | NoCTE))) | (UUE)) then  /* if the message is MIME, but not ASCII plain text (i.e. only if there is something to decode), or if it contains a UUE attachment */
do

 Decoder = Global.ProgDir||'\munpack.exe'  /* the decoder program spec */

 if (stream(Decoder,'C','QUERY EXISTS') = '') then  /* if the decoder does not exist */
 do
  call AddError 'Cannot find decoder: "'||Decoder||'"'  /* report */
 end
 else  /* if we can find the decoder */
 do

  call directory Global.MainDir  /* change to the user's main dir -- this is a kludge to make munpack.exe work */
  Decoder||' -C "'||AttDir||'" -e -q -t "'||MessFile||'"'  /* unpack the mail file into the directory, adding MIME type EAs, quiet, and including text parts [EXTERNAL] */

  call sysfiletree AttDir||'\*.*','AttFiles.','FO'  /* have we got any MIME text part files? */
  if (AttFiles.0 > 0) then  /* if so */
  do Index = 1 to AttFiles.0  /* run through the lot */

   AttFileName = filespec('N',AttFiles.Index)  /* get the file name bit */

   if (sysgetea(AttFiles.Index,'MIME-TYPE','TypeInfo') = 0) then  /* if we can get the MIME type from the file's EAs */
   do
    parse var TypeInfo 5 MIMEType  /* get the bit we want */
   end

   select  /* do one of the following */

    when (translate(MIMEType) = 'TEXT/PLAIN') then  /* if it's plain text */
    do
     NamePart = filespec('N',AttFiles.Index)  /* get the name part */
     if ((translate(substr(NamePart,1,4)) = 'PART') & (datatype(substr(NamePart,5),'W'))) then  /* if it's a message part file */
     do
      if (OpenFile(AttFiles.Index,'READ')) then  /* if we can open the file */
      do
       Buffer = charin(AttFiles.Index,1,chars(AttFiles.Index))  /* copy the file contents to a buffer */
       call stream AttFiles.Index,'C','CLOSE'  /* close the part file */
       do while (substr(Buffer,1,2) = Global.CRLF)  /* as long as the text starts with a CRLF */
        Buffer = substr(Buffer,3)  /* skip the CRLF and take the rest */
       end
       if (Buffer >< '') then  /* if we have anything left */
       do
        Buffer = translate(Buffer,Global.ISOto850,xrange('80'x,'FF'x))  /* just assume it's ISO-8859-1 text and convert any high-bit characters to PC850 */
        BodyText = BodyText||Buffer||Global.CRLF  /* add the file contents to the body text */
       end
      end
      call sysfiledelete AttFiles.Index  /* zap the part file */
     end
    end

    when (translate(MIMEType) = 'TEXT/HTML') then  /* if it's HTML */
    do
     FileName = filespec('N',AttFiles.Index)  /* store the original file name */
     DotPos = lastpos('.',AttFileName)  /* get the position of the last '.' in the file name, if any */
     if (DotPos > 0) then  /* if it was found */
     do
      if (pos('HTM',translate(substr(AttFileName,(DotPos + 1)))) = 0) then  /* if the file extension is not (*)HTM(L) */
      do
       FileName = filespec('N',AttFiles.Index)||'.html'  /* the new file name */
      end
     end
     else  /* if no dot was found */
     do
      FileName = filespec('N',AttFiles.Index)||'.html'  /* the new file title */
     end
     call syssetobjectdata AttFiles.Index,'TITLE='||FileName  /* set the (new) object title */
     InfoText = InfoText||'[HTML content written to "'||FileName||'"]'||Global.CRLF  /* the message to insert as body text */
    end

    otherwise  /* if none of the above apply */
    do
     if ((substr(AttFileName,1,4) = 'part') & (datatype(substr(AttFileName,5),'W'))) then  /* if it is a "part*" file, with "*" a whole number */
     do
      call syssetobjectdata AttFiles.Index,'TITLE='||MIMEType  /* change the title to show what's in it */
     end
     else  /* if it's not a "part?" file */
     do
      DotPos = lastpos('.',AttFileName)  /* get the position of the last '.' in the file name, if any */
      if (DotPos > 0) then  /* if it was found */
      do
       Extension = translate(substr(AttFileName,DotPos+1))  /* get the file extension in upper case */
       if (wordpos(Extension,translate(Global.Names83)) > 0) then  /* if it is a listed extension */
       do
        Name = translate(substr(AttFileName,1,DotPos-1))  /* get the file name in upper case */
        Name = translate(Name,'                ','<>[]~?%&*\|/;,`"')  /* change dangerous characters into spaces */
        NewName = ''  /* start with nothing */
        do WIndex = 1 to words(Name)  /* run through the words in the name part */
         NewName = NewName||word(Name,WIndex)  /* and add it to the new name */
        end
        if (length(NewName) > 8) then  /* if the file name is too long */
        do
         NewName = substr(NewName,1,8)  /* get the first 8 characters of the filename */
         NewName = NewName||'.'||Extension  /* create a new 8.3 file name */
         if (stream(AttDir||'\'||NewName,'C','QUERY EXISTS') >< '') then  /* if this 8.3 file name already exists */
         do
          Counter = 0  /* start at 0 */
          do until (stream(AttDir||'\'||NewName,'C','QUERY EXISTS') = '')  /* go on until we have a file name that is new */
           Counter = Counter + 1  /* up the counter */
           NewName = substr(Name,1,8-length(Counter))||Counter||'.'||Extension  /* create a new file name */
          end
         end
         call syssetobjectdata AttFiles.Index,'TITLE='||NewName  /* change the object's title */
         InfoText = InfoText||'[Renamed attachment "'||AttFileName||'" to "'||NewName||'"]'||Global.CRLF  /* additional text */
        end
       end
      end
     end
    end

   end

  end

 end

end

if ((BodyText = '') & (InfoText = "")) then  /* if we have no body or info text */
do
 if (pos('<HTML>',translate(substr(Global.MessBody,1,120))) > 0) then  /* if the message looks like HTML */
 do
  HTMLFile = systempfilename(AttDir||'\RXML????.HTML')  /* get a unique name to use */
  if (OpenFile(HTMLFile,'WRITE')) then  /* if we can open the file for writing */
  do
   call charout HTMLFile,Global.MessBody  /* write the body text to the file */
   call stream HTMLFile,'C','CLOSE'  /* close the file */
   InfoText = InfoText||'[HTML content written to "'||HTMLFile||'"]'||Global.CRLF  /* the additional text */
  end
 end
 else  /* if the message does not start with the <HTML> tag */
 do
  BodyText = Global.MessBody  /* use the complete original body text from the mail message in case the decoder missed something */
 end
end

if (OpenFile(ReadFile,'WRITE')) then  /* if we can open the new read file for writing */
do
 call charout ReadFile,HeadText  /* write the header */
 if (InfoText >< '') then  /* if we have additional text */
 do
  call charout ReadFile,copies('_',32)||Global.EmptyLine||InfoText  /* write a separator and the additional text */
 end
 call charout ReadFile,copies('_',32)||Global.EmptyLine||BodyText  /* write a separator and the body text */
 call stream ReadFile,'C','CLOSE'  /* close the file */
end

return AttDir  /* end of UnpackMessage */

/**********************************************************************/
UpdateSettings: procedure expose Global.  /* updates the current user's configuration file */
/**********************************************************************/

parse arg OldSettingsFile,NewSettingsFile  /* get the arguments */

say 'Updating configuration file'  /* report */

MessText = ''  /* start with no message text */

if (stream(OldSettingsFile,'C','QUERY EXISTS') >< '') then  /* if the RexxMail configuration file exists */
do
 if (\OpenFile(OldSettingsFile,'READ')) then  /* if we cannot open the old settings file for reading */
 do
  return 0  /* quit */
 end
 OldSettingsCont = charin(OldSettingsFile,1,chars(OldSettingsFile))  /* get the contents of the old settings file */
 call stream OldSettingsFile,'C','CLOSE'  /* close the old settings file */
end
else  /* if we have no file */
do
 OldSettingsCont = ''  /* we have no content */
end

if (\OpenFile(NewSettingsFile,'READ')) then  /* if we cannot open the new settings file for reading */
do
 return 0  /* quit */
end

NewSettingsCont = charin(NewSettingsFile,1,chars(NewSettingsFile))  /* get the contents of the new settings file */
call stream NewSettingsFile,'C','CLOSE'  /* close the new settings file */

OldSettings. = ''  /* start with nothing */
Counter = 1  /* start a counter */

do while(OldSettingsCont >< '')  /* as long as we have old settings content left */
 parse var OldSettingsCont NextLine (Global.CRLF) OldSettingsCont  /* get the next line */
 NextLine = strip(NextLine)  /* get rid of extra blanks */
 if (NextLine = '') then  /* if we have an empty line */
 do
  OldSettings.Counter._Comment = OldSettings.Counter._Comment||Global.CRLF  /* add an empty line to the comment part */
 end
 else  /* if it is not an empty line */
 do
  if (substr(NextLine,1,1) = '#') then  /* if it is a comment */
  do
   OldSettings.Counter._Comment = OldSettings.Counter._Comment||NextLine||Global.CRLF  /* add it to the comment part */
  end
  else  /* if it is not a comment */
  do
   OldSettings.Counter._Entry = NextLine  /* store the entry */
   OldSettings.Counter._KeyWord = translate(word(NextLine,1))  /* the keyword is the first word; use upper case */
   Counter = Counter + 1  /* up the counter to start a new entry */
  end
 end
end
OldSettings.0 = Counter  /* save  the counter */

NewSettings. = ''  /* start with nothing */
Counter = 0  /* restart the counter, this time at 0 */

do while(NewSettingsCont >< '')  /* as long as we have new settings content left */
 Counter = Counter +1  /* up the counter */
 parse var NewSettingsCont NextPart (Global.EmptyLine) NewSettingsCont  /* get the next line */
 NewSettings.Counter._Part = NextPart  /* save the part */
 do while (NextPart >< '')  /* as long as we have something left */
  parse var NextPart NextLine (Global.CRLF) NextPart  /* get the next line, so we end up with just the last line */
 end
 NewSettings.Counter._Keyword = translate(word(NextLine,2))  /* the keyword is the second word (the first is '#'); use upper case */
end

NewSettings.0 = Counter  /* save  the counter */
NewSettings = ''  /* start with nothing */

InvalidStuff = Global.CRLF||copies('#',72)||Global.CRLF||,  /* the first line of the warning to add before an invalid or obsolete entry */
               '# This setting is invalid or obsolete; please remove it '||copies('#',16)||Global.CRLF||,  /* the second line of the warning to add before an invalid or obsolete entry */
               copies('#',72)||Global.CRLF||,  /* the third line of the warning to add before an invalid or obsolete entry */
               '# '  /* the comment before an invalid or obsolete entry */

GotVersion = 0  /* we have no version setting yet */
GotUpdate = 0  /* we are not updating as far as we know (we may be correcting an invalid setting) */
GotInvalid = 0  /* we have no invalid or obsolete entries yet */

do Counter = 1 to OldSettings.0  /* run through the old settings */
 Found = 0  /* we haven't found anything yet */
 do Index = 1 to NewSettings.0  /* run through the new settings */
  if (OldSettings.Counter._KeyWord = NewSettings.Index._KeyWord) then  /* if the two keywords are the same */
  do
   NewSettings.Index._Keyword = ''  /* get rid of the new keyword */
   Found = 1  /* we've found it */
  end
 end
 if (Found) then  /* if the old keyword was found in the new file */
 do
  NewSettings = NewSettings||OldSettings.Counter._Comment||OldSettings.Counter._Entry||Global.CRLF  /* add the comment and entry to the new settings */
 end
 else  /* if the old keyword was not found in the new file */
 do
  NewSettings = NewSettings||OldSettings.Counter._Comment  /* always add the user's own comment part */
  if (OldSettings.Counter._KeyWord >< '') then  /* if we have an old keyword */
  do
   if (OldSettings.Counter._KeyWord = 'VERSION') then  /* if it is the 'Version' setting */
   do
    GotVersion = 1  /* we have a version setting */
    if (Global.Build >> Global.Version) then  /* if we are updating */
    do
     OldSettings.Counter._Entry = substr(OldSettings.Counter._Entry,1,length(OldSettings.Counter._Entry)-15)  /* get the user's indent for the version entry */
     NewSettings = NewSettings||OldSettings.Counter._Entry||Global.Build||Global.CRLF  /* insert the version string, retaining the original formatting */
     GotUpdate = 1  /* we have an update on our hands */
    end
    else  /* if we are not updating */
    do
     NewSettings = NewSettings||OldSettings.Counter._Entry||Global.CRLF  /* insert the original version string */
    end
   end
   else  /* if it is not the 'Version' setting */
   do
    NewSettings = NewSettings||InvalidStuff||OldSettings.Counter._Entry||Global.EmptyLine  /* add the commented-out entry to the new settings with a comment */
    GotInvalid = 1  /* we have at least one invalid or obsolete entry */
   end
  end
 end
end

if (GotInvalid) then  /* if we have commented out one or more invalid or obsolete entries */
do
 MessText = '- One or more invalid or obsolete entries have been commented out.'||Global.CRLF  /* this is the message text */
end

if (GotVersion) then  /* if we already had a version setting */
do
 if (GotUpdate) then  /* if we have performed an update */
 do
  MessText = '- The version number entry has been changed to '||Global.Build||'.'||Global.CRLF||MessText  /* add this to the start of the message text */
 end
end
else  /* if we have no version number */
do
 VersionStuff = copies('#',72)||Global.CRLF||,  /* start with a separator */
                '# RexxMail build number '||copies('#',48)||Global.CRLF||,  /* add a warning */
                '#'||Global.CRLF||,  /* add an empty comment line */
                '# Do not remove this entry!'||Global.CRLF||,  /* add a warning */
                '#'||Global.CRLF||,  /* add an empty comment line */
                copies('#',72)||Global.CRLF||,  /* add another separator */
                'Version = '||Global.Build  /* add the version entry */
 NewSettings = VersionStuff||Global.EmptyLine||NewSettings  /* add the lot to what we have */
 MessText = '- A version number entry has been added.'||Global.CRLF||MessText  /* add this to the start of the message text */
end

NewStuff = ''  /* no new stuff yet */

do Index = 1 to NewSettings.0  /* run through the new settings again */
 if (NewSettings.Index._KeyWord >< '') then  /* if the keyword still exists */
 do
  NewStuff = NewStuff||Global.EmptyLine||NewSettings.Index._Part  /* add the complete entry to the new stuff */
 end
end

if (NewStuff >< '') then  /* if we have new stuff */
do
 NewSettings = NewSettings||Global.CRLF||copies('#',72)||Global.CRLF  /* add a separator */
 NewSettings = NewSettings||'# New settings '||copies('#',57)||Global.CRLF  /* add a header */
 NewSettings = NewSettings||'#'||Global.CRLF  /* and an empty comment line */
 NewSettings = NewSettings||'# The following entries are new for this build of RexxMail, or'||Global.CRLF  /* add text */
 NewSettings = NewSettings||'# they were not found in the existing configuration file.'||Global.CRLF  /* add text */
 NewSettings = NewSettings||copies('#',72)||NewStuff  /* and another separator and the new stuff */
 MessText = MessText||'- One or more new configuration entries have been added;'||Global.CRLF  /* add this to the message text */
 MessText = MessText||'  please edit the new configuration entries to activate them.'||Global.CRLF  /* add this to the message text */
end

call sysfiledelete OldSettingsFile  /* delete the old settings file */

if (\OpenFile(OldSettingsFile,'WRITE')) then  /* if we cannot open a new RexxMail settings file */
do
 return 0  /* quit */
end

do while (substr(NewSettings,length(NewSettings)-1,2) = Global.CRLF)  /* as long as the new settings end with a CRLF */
 NewSettings = substr(NewSettings,1,length(NewSettings)-2)  /* remove it */
end

call charout OldSettingsFile,NewSettings  /* write the new settings to the RexxMail settings file */
call stream OldSettingsFile,'C','CLOSE'  /* close the updated settings file */

if (MessText >< '') then  /* if we have something to report */
do
 MessText = 'Your configuration file '||Global.CRLF||,  /* add this to the front of the message text */
            '  '||OldSettingsFile||Global.CRLF||,  /* followed by this */
            'has been updated:'||Global.CRLF||MessText  /* and this */
 call SystemMessage 'Configuration file updated.',MessText  /* send a message to the user */
end
return 1  /* end of UpdateSettings */

/**********************************************************************/
ViewMessage: procedure expose Global.  /* views received mail messages */
/**********************************************************************/

parse arg MessFile  /* get the argument */

call MessageSettings MessFile,'**+*****','CHANGE'  /* new read mail message setting: Processed */

AttDir = UnpackMessage(MessFile)  /* unpack the mail file in the folder and get the attachments folder name in return */

if (AttDir = '') then  /* if something is wrong */
do
 call AddError 'Cannot unpack "'||MessFile||'"'  /* report */
 return 0  /* and quit */
end

ReadFile = AttDir||'.READ'  /* the reader file name */

call sysfiletree AttDir||'\*.*','PreFiles.','FOS'  /* see what's in the attachments dir */

if (PreFiles.0 > 0) then  /* if the folder contains files */
do

 if (Global.RunAttachIn >< '') then  /* if we have a command to run in the attachments folder */
 do

  CurDir = directory()  /* store the current directory name */

  if (directory(AttDir) = '') then  /* if we cannot change to the attachments directory */
  do
   call AddError 'Cannot change to "'||AttDir||'"'  /* report */
   return 0  /* and quit */
  end

  parse var Global.RunAttachIn FirstBit '%N' LastBit  /* look for the filename macro in the RunAttachIn command line */
  CommLine = FirstBit  /* this is the command to start with */
  if (pos('%N',Global.RunAttachIn) > 0) then  /* if the settings line contains a file spec macro */
  do
   CommLine = CommLine||date('D')||time('S')||LastBit  /* this is the complete command */
  end
  say 'RunAttachIn configuration option parsed as : '||CommLine  /* report */
  address cmd CommLine  /* run the command */

  call directory CurDir  /* change back to the original directory */

 end

 call sysfiletree AttDir||'\*.*','PreFiles.','FOS','*****','-****'  /* see what's in the attachments dir now, resetting all archive attributes */

 if (PreFiles.0 > 0) then  /* if the folder still contains files */
 do
  call sysopenobject AttDir,1,1  /* open the attachments folder on the desktop */
 end

end

parse var Global.Reader FirstBit '%N' LastBit  /* look for the filename placeholder in the reader command line */
CommLine = FirstBit  /* this is the command to start with */

if (pos('%N',Global.Reader) > 0) then  /* if the settings line contains a file spec macro */
do
 CommLine = CommLine||ReadFile||LastBit  /* this is the complete command */
end

say 'Reader configuration option parsed as : '||CommLine  /* report */
address cmd CommLine  /* run the command */

Changed = 0  /* no changes yet */

call sysfiletree AttDir||'\*.*','PostFiles.','FOS'  /* what have we got (left) in the attachments dir? */

do Index = 1 to PostFiles.0  /* run through the list of files we have now */
 Existing = 0  /* assume the file is not an existing one (i.e. it was not there before) */
 do Counter = 1 to PreFiles.0  /* run through the list of files we had before */
  if (PostFiles.Index = PreFiles.Counter) then  /* if we have a match */
  do
   call sysfiletree PostFiles.Index,'ArchFiles.','FO','+****'  /* see if the archive attribute of the file was set */
   Existing = (ArchFiles.0 = 0)  /* if it has not, the file hasn't changed */
  end
 end
 if (\Existing) then  /* if the file was not an existing one */
 do
  Changed = 1  /* we have at least 1 change */
 end
end

if (\Changed) then  /* if we have no important changes */
do
 call sysdestroyobject AttDir  /* get rid of the attachments folder if it is still there */
end

if (stream(ReadFile,'C','QUERY EXISTS') >< '') then  /* if the read file still exists */
do
 call sysfiledelete ReadFile  /* get rid of the read file -- no check, it may already have been deleted by another process */
end

if (stream(MessFile,'C','QUERY EXISTS') >< '') then  /* if the message file still exists */
do
 call MoveMessage MessFile,Global.MessHead  /* move the file to an archive folder if needed */
end

return 1  /* end of ViewMessage */

/**********************************************************************/
ViewRawMessage: procedure expose Global.  /* shows the raw message contents */
/**********************************************************************/

parse arg MessFile  /* look for a FileName */

TempFile = systempfilename(Global.TempDir'\RXML????.RAW')  /* get a temp file name */

if (\OpenFile(TempFile,'WRITE')) then  /* if we cannot open the temp file for writing */
do
 return 0  /* and quit */
end

if (\OpenFile(MessFile,'READ')) then  /* if we cannot open the message file for writing */
do
 return 0  /* and quit */
end

call charout TempFile,charin(MessFile,1,chars(MessFile))  /* copy the whole contents of the message file into the temp file */

call stream MessFile,'C','CLOSE'  /* close the message file */

call stream TempFile,'C','CLOSE'  /* close the temp file */

if (MessageSettings(MessFile,'**-*****','MATCH')) then  /* if the message is unprocessed */
do
 call MessageSettings MessFile,'**+*****','CHANGE'  /* new edited mail message setting: Processed */
end

parse var Global.Reader FirstBit '%N' LastBit  /* look for the filename placeholder in the reader command line */

CommLine = FirstBit  /* this is the command to start with */

if (pos('%N',Global.Reader) > 0) then  /* if the settings line contains a file spec macro */
do
 CommLine = CommLine||TempFile||LastBit  /* this is the complete command */
end

say 'Reader configuration option parsed as : '||CommLine  /* report */
address cmd CommLine  /* run the command */

call sysfiledelete TempFile  /* get rid of the temporary file */

return 1  /* end of ViewRawMessage */

/**********************************************************************/
ViewSentMessage: procedure expose Global.  /* views a sent mail file */
/**********************************************************************/

parse arg MessFile  /* get the argument */

call MessageContents MessFile  /* get the message contents */

ReadFile = systempfilename(Global.TempDir'\RXML????.SENT')  /* get a unique read file name */

ListText = SetAttachments(MessFile,'LIST')  /* get a list of attachments, if any */

parse var ListText SentLine (Global.CRLF) SentDate (Global.CRLF) Attachments  /* get the relevant bits */

if (\OpenFile(ReadFile,'WRITE')) then  /* if we cannot open the file for writing */
do
 return 0  /* just quit */
end

call charout ReadFile,MakeHeaderLines(Global.MessHead||Global.CRLF||SentLine||SentDate)  /* write a header to the read file, */
if (Attachments >< '') then  /* if we have a list of attachments */
do
 call charout ReadFile,copies('_',32)||Global.EmptyLine||Attachments  /* write a separator and the list to the read file */
end
call charout ReadFile,copies('_',32)||Global.EmptyLine||Global.MessBody  /* write a separator and the body text to the read file */

call stream ReadFile,'C','CLOSE'  /* close the read file */

parse var Global.Reader FirstBit '%N' LastBit  /* look for the filename macro in the reader command line */
CommLine = FirstBit  /* this is the command to start with */
if (pos('%N',Global.Reader) > 0) then  /* if the settings line contains a file spec macro */
do
 CommLine = CommLine||ReadFile||LastBit  /* this is the complete command */
end
say 'Reader configuration option parsed as : '||CommLine  /* report */
address cmd CommLine  /* run the command */

call sysfiledelete ReadFile  /* get rid of the read file */

return 1  /* end of ViewSentMessage */

/**********************************************************************/
WordWrap: procedure expose Global.  /* word-wraps a line of text */
/**********************************************************************/

parse arg TextLine,MaxLength,Indent,NoBreak  /* get the arguments */

NoBreak = (NoBreak = 1)  /* this one is true if the argument is 1 */
Indent = (Indent = 1)  /* this one is true if the argument is 1 */

if (MaxLength = '') then  /* if we have no maximum line length value */
do
 MaxLength = 78  /* use this */
end

NewText = ''  /* start with nothing */

do while (length(TextLine) > MaxLength)  /* as long as the line is longer than the limit */
 CutPoint = MaxLength + 1  /* start the cut at ASCIILineLength plus one */
 parse var TextLine NextBit =(CutPoint) TextLine  /* get the next bit of the maximum length */
 if ((substr(NextBit,length(NextBit),1) = ' ') | (substr(TextLine,1,1) = ' ')) then  /* if we cut the line just before or after a space, i.e. we cut between two words */
 do
  TextLine = strip(TextLine,'L',' ')  /* get rid of leading spaces on the remaining bit of the line */
 end
 else  /* if we cut through a word */
 do
  LastSpace = lastpos(' ',NextBit)  /* look for the last space in this bit */
  if (LastSpace >< 0) then  /* if we found a space */
  do
   parse var NextBit NextBit =(LastSpace) MoreBit  /* cut off the bit after the space and save it for the next line */
   TextLine = strip(MoreBit,'L',' ')||TextLine  /* get rid of the leading space and stick this bit back on to what's left of the original text line */
  end
  else  /* if we found no space */
  do
   if (NoBreak) then  /* if we must not break words */
   do
    parse var TextLine MoreBit TextLine  /* get the first word of the remaining bit of the line */
    NextBit = NextBit||MoreBit  /* and add it to what we have */
   end
  end
 end
 NextBit = strip(NextBit,'T',' ')  /* get rid of trailing spaces */
 NewText = NewText||NextBit  /* add the bit we have to the new text */
 if (TextLine >< '') then  /* if there is more to come */
 do
  NewText = NewText||Global.CRLF||copies(' ',Indent)  /* add a new line with an indent */
 end
end

NewText = NewText||TextLine  /* add the remaining bit of the line to the text */

return NewText  /* end of WordWrap */
